"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var jsm_service_1 = require("bootstrap/api/services/jsm-service");
var v0_rules_migration_1 = require("legacy/app/migration/v0-rules-migration");
var log_1 = require("app/log");
var settings_migration_1 = require("app/migration/settings-migration");
var rules_services_module_1 = require("app/services/rules/rules-services.module");
var v0_rules_service_1 = require("app/services/rules/v0-rules-service");
var version_info_service_1 = require("app/services/version-info-service");
var async_settings_1 = require("app/storage/async-settings");
var cached_settings_1 = require("app/storage/cached-settings");
var setting_specs_1 = require("app/storage/setting-specs");
var initial_setup_1 = require("app/ui/initial-setup");
var constants_1 = require("data/constants");
var compareVersions = require("lib/third-party/mozilla-version-comparator");
var app_background_module_1 = require("./app.background.module");
var browser_settings_module_1 = require("./browser-settings/browser-settings.module");
var migration_module_1 = require("./migration/migration.module");
var policy_module_1 = require("./policy/policy.module");
var ruleset_storage_1 = require("./policy/ruleset-storage");
var subscriptions_1 = require("./policy/subscriptions");
var services_module_1 = require("./services/services.module");
var uri_service_1 = require("./services/uri-service");
var storage_module_1 = require("./storage/storage.module");
var ui_module_1 = require("./ui/ui.module");
var localStorageArea = browser.storage.local;
var settingsMigration = new settings_migration_1.SettingsMigration(log_1.log, browser.storage.local);
var storageReadyPromise = settingsMigration.whenReady;
var jsmService = constants_1.C.EXTENSION_TYPE === "legacy" ? new jsm_service_1.JSMService(Cu) : null;
var mozServices = constants_1.C.EXTENSION_TYPE === "legacy" ?
    jsmService.getServices() : null;
var xpcApi = constants_1.C.EXTENSION_TYPE === "legacy" ? {
    prefsService: mozServices.prefs,
    rpPrefBranch: LegacyApi.rpPrefBranch,
    tryCatchUtils: LegacyApi.tryCatchUtils,
} : null;
var rulesetStorage = new ruleset_storage_1.RulesetStorage(log_1.log, localStorageArea);
var subscriptions = new subscriptions_1.Subscriptions(log_1.log, rulesetStorage, localStorageArea);
var policy = new policy_module_1.Policy(log_1.log, subscriptions, rulesetStorage);
var asyncSettings = new async_settings_1.AsyncSettings(log_1.log, browser.storage, browser.storage.local, setting_specs_1.SETTING_SPECS.defaultValues, storageReadyPromise);
log_1.dAsyncSettings.resolve(asyncSettings);
var cachedSettings = new cached_settings_1.CachedSettings(log_1.log, setting_specs_1.SETTING_SPECS, storageReadyPromise, LegacyApi.rpPrefBranch, LegacyApi.prefsService);
var storage = new storage_module_1.Storage(log_1.log, asyncSettings, cachedSettings, storageReadyPromise);
var browserSettings = new browser_settings_module_1.BrowserSettings(log_1.log, cachedSettings, browser.privacy.network);
var uriService = new uri_service_1.UriService(log_1.log);
var v0RulesService = new v0_rules_service_1.V0RulesService(log_1.log, uriService, xpcApi);
var rulesServices = new rules_services_module_1.RulesServices(log_1.log, v0RulesService);
var versionComparator = { compare: compareVersions };
var versionInfoService = new version_info_service_1.VersionInfoService(log_1.log, versionComparator, cachedSettings, browser.management, browser.runtime);
var rpServices = new services_module_1.RPServices(log_1.log, rulesServices, uriService, versionInfoService);
var v0RulesMigration = constants_1.C.EXTENSION_TYPE === "legacy" ?
    new v0_rules_migration_1.V0RulesMigration(log_1.log, policy, v0RulesService, versionInfoService) : null;
var migration = new migration_module_1.Migration(log_1.log, settingsMigration, v0RulesMigration);
var initialSetup = new initial_setup_1.InitialSetup(log_1.log, cachedSettings, versionInfoService, xpcApi);
var ui = new ui_module_1.Ui(log_1.log, initialSetup);
exports.rp = new app_background_module_1.AppBackground(log_1.log, browserSettings, migration, policy, rpServices, storage, ui);