"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var module_1 = require("lib/classes/module");
var environment_1 = require("lib/environment");
var subscription_1 = require("lib/subscription");
var listener_factories_1 = require("lib/utils/listener-factories");
var RULESET_NOT_EXISTING = {};
var Subscriptions = (function (_super) {
    __extends(Subscriptions, _super);
    function Subscriptions(log, rulesetStorage, storageArea) {
        var _this = _super.call(this, "rules", log) || this;
        _this.rulesetStorage = rulesetStorage;
        _this.storageArea = storageArea;
        _this.subscriptionRulesets = {};
        _this.events = listener_factories_1.createListenersMap(["onRulesChanged"]);
        _this.onRulesChanged = _this.events.interfaces.onRulesChanged;
        return _this;
    }
    Subscriptions.prototype.getRulesets = function () {
        return this.subscriptionRulesets;
    };
    Subscriptions.prototype.loadSubscriptionRules = function () {
        var _this = this;
        var pDone = this.storageArea.get("subscriptions").then(function (result) {
            var rawData = result.hasOwnProperty("subscriptions") ?
                result.subscriptions : undefined;
            _this.subscriptions = subscription_1.UserSubscriptions.create(rawData);
            return _this.loadSubscriptionRulesFromSubInfo(_this.subscriptions.getSubscriptionInfo());
        }).then(function (_a) {
            var failures = _a.failures;
            var serials = {};
            for (var listName in failures) {
                serials[listName] = {};
                for (var subName in failures[listName]) {
                    serials[listName][subName] = -1;
                }
            }
            for (var listName in _this.subscriptionRulesets) {
                for (var subName in _this.subscriptionRulesets[listName]) {
                    if (!serials[listName]) {
                        serials[listName] = {};
                    }
                    var rawRuleset = _this.subscriptionRulesets[listName][subName].rawRuleset;
                    serials[listName][subName] = rawRuleset.metadata.serial;
                }
            }
            _this.subscriptions.update(function (result) {
                _this.log.info("Subscription updates completed: " + result);
            }, serials);
            return;
        });
        pDone.catch(function (e) {
            _this.log.error("loadConfigAndRules():", e);
        });
        return pDone;
    };
    Subscriptions.prototype.loadSubscriptionRulesFromSubInfo = function (subscriptionInfo) {
        var _this = this;
        var failures = {};
        var promises = [];
        var _loop_1 = function (listName) {
            var _loop_2 = function (subName) {
                this_1.log.info("loadSubscriptionRules: " + listName + " / " + subName);
                var pRawRuleset = this_1.rulesetStorage.
                    loadRawRulesetFromFile(subName, listName);
                var pDone = pRawRuleset.then(function (rawRuleset) {
                    if (!rawRuleset)
                        return Promise.reject(RULESET_NOT_EXISTING);
                    if (!_this.subscriptionRulesets[listName]) {
                        _this.subscriptionRulesets[listName] = {};
                    }
                    var list = _this.subscriptionRulesets[listName];
                    list[subName] = {
                        rawRuleset: rawRuleset,
                        ruleset: rawRuleset.toRuleset(subName),
                    };
                    list[subName].ruleset.userRuleset = false;
                    return Promise.resolve();
                }).catch(function (e) {
                    if (e === RULESET_NOT_EXISTING) {
                        _this.log.warn("Ruleset does not exist (yet).");
                    }
                    else {
                        _this.log.error("Error when loading ruleset from file: ", e);
                    }
                    if (!failures[listName]) {
                        failures[listName] = {};
                    }
                    failures[listName][subName] = null;
                });
                promises.push(pDone);
            };
            for (var subName in subscriptionInfo[listName]) {
                _loop_2(subName);
            }
        };
        var this_1 = this;
        for (var listName in subscriptionInfo) {
            _loop_1(listName);
        }
        return Promise.all(promises).then(function () {
            _this.events.listenersMap.onRulesChanged.emit();
            return { failures: failures };
        });
    };
    Subscriptions.prototype.unloadSubscriptionRules = function (subscriptionInfo) {
        var failures = {};
        for (var listName in subscriptionInfo) {
            for (var subName in subscriptionInfo[listName]) {
                this.log.info("unloadSubscriptionRules: " + listName + " / " + subName);
                if (!this.subscriptionRulesets[listName] ||
                    !this.subscriptionRulesets[listName][subName]) {
                    if (!failures[listName]) {
                        failures[listName] = {};
                    }
                    failures[listName][subName] = null;
                    continue;
                }
                var list = this.subscriptionRulesets[listName];
                delete list[subName];
            }
        }
        this.events.listenersMap.onRulesChanged.emit();
        return failures;
    };
    Subscriptions.prototype.getSubscriptions = function () {
        return this.subscriptions;
    };
    Subscriptions.prototype.observe = function (subject, topic, data) {
        var _this = this;
        switch (topic) {
            case subscription_1.SUBSCRIPTION_UPDATED_TOPIC: {
                this.log.log("XXX updated: " + data);
                var subInfo = JSON.parse(data);
                this.loadSubscriptionRulesFromSubInfo(subInfo).
                    catch(this.log.onError("SUBSCRIPTION_UPDATED_TOPIC"));
                break;
            }
            case subscription_1.SUBSCRIPTION_ADDED_TOPIC: {
                this.log.log("XXX added: " + data);
                var subInfo_1 = JSON.parse(data);
                var pLoadSubscriptionRules = this.loadSubscriptionRulesFromSubInfo(subInfo_1);
                pLoadSubscriptionRules.then(function (_a) {
                    var failures = _a.failures;
                    var failed = Object.getOwnPropertyNames(failures).length > 0;
                    if (failed) {
                        var serials = {};
                        for (var listName in subInfo_1) {
                            if (!serials[listName]) {
                                serials[listName] = {};
                            }
                            for (var subName in subInfo_1[listName]) {
                                serials[listName][subName] = -1;
                            }
                        }
                        _this.subscriptions.update(function (result) {
                            _this.log.info("Subscription update completed: " + result);
                        }, serials);
                    }
                    return;
                }).catch(function (e) {
                    _this.log.error("SUBSCRIPTION_ADDED_TOPIC", e);
                });
                break;
            }
            case subscription_1.SUBSCRIPTION_REMOVED_TOPIC: {
                this.log.log("YYY: " + data);
                var subInfo = JSON.parse(data);
                this.unloadSubscriptionRules(subInfo);
                break;
            }
            default:
                console.error("[RequestPolicy] unknown topic observed: " + topic);
        }
    };
    Subscriptions.prototype.startupSelf = function () {
        environment_1.MainEnvironment.obMan.observe([
            subscription_1.SUBSCRIPTION_UPDATED_TOPIC,
            subscription_1.SUBSCRIPTION_ADDED_TOPIC,
            subscription_1.SUBSCRIPTION_REMOVED_TOPIC,
        ], this.observe.bind(this));
        return this.loadSubscriptionRules();
    };
    return Subscriptions;
}(module_1.Module));
exports.Subscriptions = Subscriptions;