"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var module_1 = require("lib/classes/module");
var IDN_SERVICE = Cc["@mozilla.org/network/idn-service;1"]
    .getService(Ci.nsIIDNService);
var HostLevel;
(function (HostLevel) {
    HostLevel[HostLevel["DOMAIN"] = 1] = "DOMAIN";
    HostLevel[HostLevel["HOST"] = 2] = "HOST";
    HostLevel[HostLevel["SOP"] = 3] = "SOP";
})(HostLevel || (HostLevel = {}));
var UriService = (function (_super) {
    __extends(UriService, _super);
    function UriService(log, nameRoot) {
        if (nameRoot === void 0) { nameRoot = "app"; }
        return _super.call(this, nameRoot + ".services.uri", log) || this;
    }
    Object.defineProperty(UriService.prototype, "hostLevels", {
        get: function () { return HostLevel; },
        enumerable: true,
        configurable: true
    });
    UriService.prototype.getIdentifier = function (aUri, aLevel) {
        if (aLevel === void 0) { aLevel = HostLevel.SOP; }
        var identifier;
        var identifierGettingFn;
        switch (aLevel) {
            case HostLevel.DOMAIN:
                identifierGettingFn = function () { return null; };
                break;
            case HostLevel.HOST:
                identifierGettingFn = this.getHost.bind(this);
                break;
            case HostLevel.SOP:
                identifierGettingFn = this.getPrePath.bind(this);
                break;
            default:
                throw new Error("Invalid identifier level specified " +
                    "in getIdentifier(): " + aLevel);
        }
        try {
            identifier = identifierGettingFn(aUri);
        }
        catch (e) {
            identifier = false;
        }
        if (identifier) {
            return identifier;
        }
        else {
            if (aUri.indexOf("file://") === 0) {
                return "file://";
            }
            else if (aUri.indexOf("data:") === 0) {
                identifier = aUri.split(",")[0];
                return identifier.split(";")[0];
            }
            this.log.info("Unable to getIdentifier from uri " +
                aUri + " using identifier level " + aLevel + ".");
            return aUri;
        }
    };
    UriService.prototype.getHostByUriObj = function (aUriObj) {
        try {
            return aUriObj.host;
        }
        catch (e) {
            return null;
        }
    };
    UriService.prototype.getHost = function (aUri) {
        return this.getHostByUriObj(this.getUriObject(aUri));
    };
    UriService.prototype.uriObjHasPort = function (aUriObj) {
        try {
            aUriObj.port;
            return true;
        }
        catch (e) {
            return false;
        }
    };
    UriService.prototype.getUriObject = function (uri) {
        try {
            return Services.io.newURI(uri, null, null);
        }
        catch (e) {
            var msg = "getUriObject() exception on uri <" + uri + ">.";
            this.log.log(msg);
            throw e;
        }
    };
    UriService.prototype.isValidUri = function (uri) {
        try {
            Services.io.newURI(uri, null, null);
            return true;
        }
        catch (e) {
            return false;
        }
    };
    UriService.prototype.getBaseDomain = function (uri) {
        var host = this.getHost(uri);
        if (host === null) {
            return null;
        }
        try {
            var baseDomain = Services.eTLD.getBaseDomainFromHost(host, 0);
            return IDN_SERVICE.convertToDisplayIDN(baseDomain, {});
        }
        catch (e) {
            if (e.name === "NS_ERROR_HOST_IS_IP_ADDRESS") {
                return host;
            }
            else if (e.name === "NS_ERROR_INSUFFICIENT_DOMAIN_LEVELS") {
                return host;
            }
            else {
                throw e;
            }
        }
    };
    UriService.prototype.isIPAddress = function (host) {
        try {
            Services.eTLD.getBaseDomainFromHost(host, 0);
            return false;
        }
        catch (e) {
            switch (e.name) {
                case "NS_ERROR_HOST_IS_IP_ADDRESS":
                    return true;
                case "NS_ERROR_INSUFFICIENT_DOMAIN_LEVELS":
                    return false;
                default:
                    console.error("[RequestPolicy] Unexpected error:", e);
                    return false;
            }
        }
    };
    UriService.prototype.getPath = function (uri) {
        return this.getUriObject(uri).path;
    };
    UriService.prototype.getPrePath = function (uri) {
        return this.getUriObject(uri).prePath;
    };
    UriService.prototype.stripFragment = function (uri) {
        return uri.split("#")[0];
    };
    UriService.prototype.ensureUriHasPath = function (uri) {
        try {
            return this.getUriObject(uri).spec;
        }
        catch (e) {
            return uri;
        }
    };
    UriService.prototype.formatIDNUri = function (uri) {
        return this.getUriObject(uri).spec;
    };
    UriService.prototype.hasStandardPort = function (uri) {
        return uri.port === -1 ||
            uri.scheme !== "http" && uri.scheme !== "https" ||
            uri.port === 80 && uri.scheme === "http" ||
            uri.port === 443 && uri.scheme === "https";
    };
    UriService.prototype.getDefaultPortForScheme = function (scheme) {
        switch (scheme) {
            case "http":
                return 80;
            case "https":
                return 443;
            default:
                this.log.warn("Unknown default port for scheme " + scheme + ".");
                return null;
        }
    };
    return UriService;
}(module_1.Module));
exports.UriService = UriService;