"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var module_1 = require("lib/classes/module");
var JSUtils = require("lib/utils/js-utils");
var VersionInfoService = (function (_super) {
    __extends(VersionInfoService, _super);
    function VersionInfoService(log, versionComparator, cachedSettings, managementApi, runtimeApi) {
        var _this = _super.call(this, "app.services.versionInfo", log) || this;
        _this.versionComparator = versionComparator;
        _this.cachedSettings = cachedSettings;
        _this.managementApi = managementApi;
        _this.runtimeApi = runtimeApi;
        return _this;
    }
    Object.defineProperty(VersionInfoService.prototype, "curAppVersion", {
        get: function () { return this.infos.curAppVersion; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VersionInfoService.prototype, "curRPVersion", {
        get: function () { return this.infos.curRPVersion; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VersionInfoService.prototype, "isRPUpgrade", {
        get: function () { return this.infos.isRPUpgrade; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VersionInfoService.prototype, "lastAppVersion", {
        get: function () { return this.infos.lastAppVersion; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VersionInfoService.prototype, "lastRPVersion", {
        get: function () { return this.infos.lastRPVersion; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(VersionInfoService.prototype, "startupPreconditions", {
        get: function () {
            return [
                this.cachedSettings.whenReady,
            ];
        },
        enumerable: true,
        configurable: true
    });
    VersionInfoService.prototype.startupSelf = function () {
        var _this = this;
        var promises = {};
        var checkPromise = function (aPropName) {
            promises[aPropName].catch(function (e) {
                _this.log.error("Error initializing \"" + aPropName + "\":", e);
            });
        };
        var infos = {};
        promises.lastRPVersion = Promise.resolve(this.cachedSettings.get("lastVersion"));
        checkPromise("lastRPVersion");
        promises.isRPUpgrade =
            promises.lastRPVersion.
                then(function (lastRPVersion) {
                infos.isRPUpgrade = !!lastRPVersion &&
                    _this.versionComparator.compare(lastRPVersion, "1.0.0a8") <= 0;
                return infos.isRPUpgrade;
            });
        checkPromise("isRPUpgrade");
        promises.curRPVersion =
            this.managementApi.getSelf().
                then(function (addon) {
                infos.curRPVersion = addon.version;
                return infos.curRPVersion;
            });
        checkPromise("curRPVersion");
        promises.lastAppVersion = Promise.resolve(this.cachedSettings.get("lastAppVersion"));
        checkPromise("lastAppVersion");
        promises.curAppVersion =
            this.runtimeApi.getBrowserInfo().
                then(function (_a) {
                var version = _a.version;
                infos.curAppVersion = version;
                return version;
            });
        checkPromise("curAppVersion");
        return Promise.all(JSUtils.objectValues(promises)).then(function () {
            _this.infos = infos;
            var curAppVersion = infos.curAppVersion, curRPVersion = infos.curRPVersion;
            return _this.cachedSettings.set({
                lastAppVersion: curAppVersion,
                lastVersion: curRPVersion,
            });
        }).catch(function (e) {
            _this.log.error("Failed to initialize VersionInfoService", e);
        });
    };
    return VersionInfoService;
}(module_1.Module));
exports.VersionInfoService = VersionInfoService;