"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var module_1 = require("lib/classes/module");
var listener_factories_1 = require("lib/utils/listener-factories");
var AsyncSettings = (function (_super) {
    __extends(AsyncSettings, _super);
    function AsyncSettings(log, storageApi, storageArea, defaultSettings, storageReadyPromise) {
        var _this = _super.call(this, "app.storage.asyncSettings", log) || this;
        _this.storageApi = storageApi;
        _this.storageArea = storageArea;
        _this.defaultSettings = defaultSettings;
        _this.storageReadyPromise = storageReadyPromise;
        _this.changeEventListener = _this.onChangedEvent.bind(_this);
        _this.events = listener_factories_1.createListenersMap(["onChanged"]);
        _this.onChanged = _this.events.interfaces.onChanged;
        return _this;
    }
    Object.defineProperty(AsyncSettings.prototype, "startupPreconditions", {
        get: function () {
            return [
                this.storageReadyPromise,
            ];
        },
        enumerable: true,
        configurable: true
    });
    AsyncSettings.prototype.get = function (aKeys) {
        var _this = this;
        this.assertReady();
        if (aKeys === "")
            return Promise.resolve({});
        var keys = typeof aKeys === "string" ? [aKeys] : aKeys;
        var keysWithDefaults = {};
        keys.forEach(function (key) {
            if (!_this.defaultSettings.hasOwnProperty(key)) {
                throw new Error("Setting \"" + key + "\" has no default value!");
            }
            keysWithDefaults[key] = _this.defaultSettings[key];
        });
        return this.storageArea.get(keysWithDefaults);
    };
    AsyncSettings.prototype.set = function (aKeys) {
        return this.storageArea.set(aKeys);
    };
    AsyncSettings.prototype.startupSelf = function () {
        this.storageApi.onChanged.addListener(this.changeEventListener);
        return Promise.resolve();
    };
    AsyncSettings.prototype.shutdownSelf = function () {
        this.storageApi.onChanged.removeListener(this.changeEventListener);
        return Promise.resolve();
    };
    AsyncSettings.prototype.onChangedEvent = function (changes, areaName) {
        var _this = this;
        var changes2 = {};
        Object.keys(changes).forEach(function (key) {
            if (!_this.defaultSettings.hasOwnProperty(key))
                return;
            var change = changes[key];
            if (!change.hasOwnProperty("newValue")) {
                var defaultValue = _this.defaultSettings[key];
                if (change.hasOwnProperty("oldValue") &&
                    change.oldValue === defaultValue)
                    return;
                change.newValue = defaultValue;
            }
            changes2[key] = change;
        });
        if (Object.keys(changes2).length === 0)
            return;
        this.events.listenersMap.onChanged.emit(changes2, areaName);
    };
    return AsyncSettings;
}(module_1.Module));
exports.AsyncSettings = AsyncSettings;