"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var js_utils_1 = require("lib/utils/js-utils");
var I18nUtils = require("./i18n-utils");
var locale_data_1 = require("./locale-data");
var AsyncLocaleData = (function (_super) {
    __extends(AsyncLocaleData, _super);
    function AsyncLocaleData(tryCatchUtils, chromeFileService, mozServices) {
        var _this = _super.call(this) || this;
        _this.tryCatchUtils = tryCatchUtils;
        _this.chromeFileService = chromeFileService;
        _this.mozServices = mozServices;
        _this.dReady = js_utils_1.defer();
        return _this;
    }
    Object.defineProperty(AsyncLocaleData.prototype, "whenReady", {
        get: function () {
            return this.dReady.promise;
        },
        enumerable: true,
        configurable: true
    });
    AsyncLocaleData.prototype.getAppLocale = function () {
        var appLocale = this.tryCatchUtils.getAppLocale(this.mozServices.prefs, this.mozServices.locale);
        return I18nUtils.normalizeToBCP47(appLocale);
    };
    AsyncLocaleData.prototype.getDefaultLocale = function () {
        var manifestUrl = this.chromeFileService.getChromeUrl("bootstrap-data/manifest.json");
        return this.chromeFileService.parseJSON(manifestUrl).then(function (manifest) {
            if (manifest && manifest.default_locale) {
                var bcp47tag = I18nUtils.normalizeToBCP47(manifest.default_locale);
                return Promise.resolve(bcp47tag);
            }
            else {
                return Promise.reject(new Error("'default_locale' key not found in manifest.json"));
            }
        });
    };
    AsyncLocaleData.prototype.getAvailableLocales = function () {
        var localesListJSON = this.chromeFileService.getChromeUrl("bootstrap-data/locales.json");
        return this.chromeFileService.parseJSON(localesListJSON).then(function (localesDirNames) { return Promise.all(localesDirNames.map(function (dirName) { return [
            I18nUtils.normalizeToBCP47(dirName),
            dirName,
        ]; })); }).then(function (mapAsArray) { return new Map(mapAsArray); });
    };
    AsyncLocaleData.prototype.getBestMatches = function (requestedLocales, localesMap) {
        var e_1, _a, e_2, _b;
        var matchesSet = new Set();
        var localesTag = Array.from(localesMap.keys());
        try {
            for (var requestedLocales_1 = __values(requestedLocales), requestedLocales_1_1 = requestedLocales_1.next(); !requestedLocales_1_1.done; requestedLocales_1_1 = requestedLocales_1.next()) {
                var locale = requestedLocales_1_1.value;
                var normTag = I18nUtils.normalizeToBCP47(locale);
                var matchTag = I18nUtils.getBestAvailableLocale(localesTag, normTag);
                if (matchTag) {
                    matchesSet.add(matchTag);
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (requestedLocales_1_1 && !requestedLocales_1_1.done && (_a = requestedLocales_1.return)) _a.call(requestedLocales_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        var result = new Map();
        try {
            for (var matchesSet_1 = __values(matchesSet), matchesSet_1_1 = matchesSet_1.next(); !matchesSet_1_1.done; matchesSet_1_1 = matchesSet_1.next()) {
                var key = matchesSet_1_1.value;
                result.set(key, localesMap.get(key));
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (matchesSet_1_1 && !matchesSet_1_1.done && (_b = matchesSet_1.return)) _b.call(matchesSet_1);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return result;
    };
    AsyncLocaleData.prototype.loadLocalesMessages = function (localesMap) {
        var _this = this;
        return Promise.all(Array.from(localesMap.keys()).map(function (key) {
            var dir = localesMap.get(key);
            var file = "_locales/" + dir + "/messages.json";
            var url = _this.chromeFileService.getChromeUrl(file);
            return _this.chromeFileService.parseJSON(url).then(function (messages) { return Promise.resolve({ locale: key, messages: messages }); });
        }));
    };
    AsyncLocaleData.prototype.updateLocalesPrefs = function (defaultLocale, uiLocale) {
        var _this = this;
        var normDefault = I18nUtils.normalizeToBCP47(defaultLocale);
        var normUi = I18nUtils.normalizeToBCP47(uiLocale);
        var loadedTags = Array.from(this.messages.keys()).
            filter(function (tag) { return tag.toLowerCase() !== _this.BUILTIN.toLowerCase(); }).
            map(function (tag) { return I18nUtils.normalizeToBCP47(tag); });
        var bestDefault = I18nUtils.
            getBestAvailableLocale(loadedTags, normDefault);
        if (!bestDefault) {
            throw new Error("Can't find a locale matching the default locale "
                + ("'" + defaultLocale + "'"));
        }
        var bestUi = I18nUtils.getBestAvailableLocale(loadedTags, normUi);
        if (!bestUi) {
            bestUi = normUi;
        }
        this.defaultLocale = bestDefault;
        this.selectedLocale = bestUi;
        return {
            default: bestDefault,
            selected: bestUi,
        };
    };
    AsyncLocaleData.prototype.startup = function () {
        var _this = this;
        var defaultLocale;
        var uiLocale;
        var p = this.getDefaultLocale().
            then(function (result) { return defaultLocale = result; }).
            then(function () { return uiLocale = _this.getAppLocale(); }).
            then(function () { return _this.getAvailableLocales(); }).
            then(function (AllLocales) { return _this.getBestMatches([defaultLocale, uiLocale], AllLocales); }).
            then(function (bestMatches) { return _this.loadLocalesMessages(bestMatches); }).
            then(function (localesMessages) { return Promise.all(localesMessages.map(function (obj) { return _this.addLocale(obj.locale, obj.messages); })); }).then(function () { return _this.updateLocalesPrefs(defaultLocale, uiLocale); }).
            then(function () {
            _this.dReady.resolve(undefined);
            return;
        });
        p.catch(function (e) {
            console.error("[RequestPolicy] LazyLocaleData.startup()");
            console.dir(e);
            _this.dReady.reject(e);
        });
        return p;
    };
    return AsyncLocaleData;
}(locale_data_1.LocaleData));
exports.AsyncLocaleData = AsyncLocaleData;