"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var case_insensitive_map_1 = require("lib/classes/case-insensitive-map");
var LocaleData = (function () {
    function LocaleData() {
        this.defaultLocale = "";
        this.selectedLocale = "";
        this.messages = new Map();
        this.warnedMissingKeys = new Set();
    }
    Object.defineProperty(LocaleData.prototype, "BUILTIN", {
        get: function () { return LocaleData.BUILTIN; },
        enumerable: true,
        configurable: true
    });
    LocaleData.prototype.serialize = function () {
        return {
            defaultLocale: this.defaultLocale,
            messages: this.messages,
            selectedLocale: this.selectedLocale,
        };
    };
    LocaleData.prototype.has = function (locale) {
        return this.messages.has(locale);
    };
    LocaleData.prototype.localizeMessage = function (message, substitutions, options) {
        var _this = this;
        if (substitutions === void 0) { substitutions = []; }
        if (options === void 0) { options = {}; }
        var e_1, _a;
        var defaultOptions = {
            cloneScope: null,
            defaultValue: "",
        };
        var locales = this.availableLocales;
        if (options.locale) {
            locales = new Set([this.BUILTIN, options.locale, this.defaultLocale]
                .filter(function (locale) { return _this.messages.has(locale); }));
        }
        options = Object.assign(defaultOptions, options);
        try {
            for (var locales_1 = __values(locales), locales_1_1 = locales_1.next(); !locales_1_1.done; locales_1_1 = locales_1.next()) {
                var locale = locales_1_1.value;
                var messages = this.messages.get(locale);
                if (messages.has(message)) {
                    var str = messages.get(message);
                    if (!str.includes("$")) {
                        return str;
                    }
                    if (!Array.isArray(substitutions)) {
                        substitutions = [substitutions];
                    }
                    var replacer = function (matched, index, dollarSigns) {
                        if (index) {
                            var intIndex = parseInt(index, 10) - 1;
                            return intIndex in substitutions ? substitutions[intIndex] : "";
                        }
                        return dollarSigns;
                    };
                    return str.replace(/\$(?:([1-9]\d*)|(\$+))/g, replacer);
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (locales_1_1 && !locales_1_1.done && (_a = locales_1.return)) _a.call(locales_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        if (!this.warnedMissingKeys.has(message)) {
            var error = "Unknown localization message " + message;
            console.error("[RequestPolicy] " + error);
            this.warnedMissingKeys.add(message);
        }
        return options.defaultValue;
    };
    LocaleData.prototype.localize = function (str, locale) {
        var _this = this;
        if (locale === void 0) { locale = this.selectedLocale; }
        if (!str) {
            return str;
        }
        return str.replace(/__MSG_([A-Za-z0-9@_]+?)__/g, function (matched, message) {
            return _this.localizeMessage(message, [], { locale: locale, defaultValue: matched });
        });
    };
    LocaleData.prototype.addLocale = function (locale, messages) {
        var e_2, _a;
        var result = new case_insensitive_map_1.CIMap();
        var isPlainObject = function (obj) { return (obj && typeof obj === "object"); };
        if (!isPlainObject(messages)) {
            console.error("[RequestPolicy] Invalid locale data for " + locale);
            return result;
        }
        var _loop_1 = function (key) {
            var e_3, _a;
            var msg = messages[key];
            if (!isPlainObject(msg) || typeof (msg.message) !== "string") {
                console.error("[RequestPolicy] Invalid locale message data for " + locale + ",\n          message " + JSON.stringify(key));
                return "continue";
            }
            var placeholders = new case_insensitive_map_1.CIMap();
            if (isPlainObject(msg.placeholders)) {
                try {
                    for (var _b = __values(Object.keys(msg.placeholders)), _c = _b.next(); !_c.done; _c = _b.next()) {
                        var key2 = _c.value;
                        placeholders.set(key2, msg.placeholders[key2]);
                    }
                }
                catch (e_3_1) { e_3 = { error: e_3_1 }; }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                    }
                    finally { if (e_3) throw e_3.error; }
                }
            }
            var replacer = function (match, name) {
                var replacement = placeholders.get(name);
                if (isPlainObject(replacement) && "content" in replacement) {
                    return replacement.content;
                }
                return "";
            };
            var value = msg.message.replace(/\$([A-Za-z0-9@_]+)\$/g, replacer);
            result.set(key, value);
        };
        try {
            for (var _b = __values(Object.keys(messages)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var key = _c.value;
                _loop_1(key);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
        this.messages.set(locale, result);
        return result;
    };
    Object.defineProperty(LocaleData.prototype, "availableLocales", {
        get: function () {
            var _this = this;
            return new Set([this.BUILTIN, this.selectedLocale, this.defaultLocale]
                .filter(function (locale) { return _this.messages.has(locale); }));
        },
        enumerable: true,
        configurable: true
    });
    LocaleData.BUILTIN = "@@BUILTIN_MESSAGES";
    return LocaleData;
}());
exports.LocaleData = LocaleData;