"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ChromeFileService = (function () {
    function ChromeFileService(netUtil, http) {
        this.netUtil = netUtil;
        this.http = http;
    }
    ChromeFileService.prototype.getChromeUrl = function (path) {
        var replacer = /^(?:\.\/|\/)?(?:content\/)?(.+)$/mg;
        return path.replace(replacer, "chrome://rpcontinued/content/$1");
    };
    ChromeFileService.prototype.readDirectory = function (aChromeUrl) {
        var chromeUrl = aChromeUrl;
        if (!chromeUrl) {
            return Promise.reject(new Error("Invalid null argument"));
        }
        else if (!chromeUrl.endsWith("/")) {
            chromeUrl = chromeUrl + "/";
        }
        return this.sendHttpGet(chromeUrl).then(function (responseText) {
            var fileList = [];
            var extractor = /^201:\s(\S+)\s+\d+\s+\S+\s+(\S+)\s*$/gm;
            var entry;
            var nextEntry = function () {
                entry = extractor.exec(responseText);
                return entry;
            };
            while (nextEntry()) {
                fileList.push({
                    isDir: entry[2].toLowerCase() === "directory",
                    name: entry[1],
                });
            }
            return Promise.resolve(fileList);
        });
    };
    ChromeFileService.prototype.parseJSON = function (chromeUrl) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                _this.netUtil.asyncFetch(chromeUrl, function (inputStream, status) {
                    try {
                        if (!_this.isSuccessCode(status)) {
                            reject(new Error("Enable to load '" + chromeUrl + "': " + status));
                            return;
                        }
                        var text = _this.netUtil.readInputStreamToString(inputStream, inputStream.available(), { charset: "utf-8" });
                        inputStream.close();
                        resolve(JSON.parse(text));
                    }
                    catch (e) {
                        reject(e);
                    }
                });
            }
            catch (e) {
                reject(e);
            }
        });
    };
    ChromeFileService.prototype.sendHttpGet = function (url) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                var xhrOptions = {
                    method: "GET",
                    onLoad: function (responseText, xhr) {
                        if (xhr && xhr.status === 200) {
                            resolve(responseText);
                        }
                        else {
                            reject(new Error("Invalid status code when reading '" + url + "'"));
                        }
                    },
                    onError: function (error) {
                        reject(new Error("Error reading '" + url + "': " + error));
                    },
                };
                _this.http.httpRequest(url, xhrOptions);
            }
            catch (e) {
                reject(e);
            }
        });
    };
    ChromeFileService.prototype.isSuccessCode = function (returnCode) {
        return (returnCode & 0x80000000) === 0;
    };
    return ChromeFileService;
}());
exports.ChromeFileService = ChromeFileService;