"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var REQUESTPOLICY_DIR = "requestpolicy";
var FileService = (function () {
    function FileService(xpc, mozFileUtils) {
        this.xpc = xpc;
        this.mozFileUtils = mozFileUtils;
    }
    FileService.prototype.getRPFile = function (aPath) {
        return this.getProfileFile(this.getRPPath(aPath));
    };
    FileService.prototype.getRPDir = function (aPath, shouldCreate) {
        return this.getProfileDir(this.getRPPath(aPath), shouldCreate);
    };
    FileService.prototype.getAllRPFiles = function (aDir) {
        var dirPath = aDir ? aDir.path : "";
        var dirFile = aDir ? aDir.file : this.getRPFile(dirPath);
        if (!dirFile.exists())
            return [];
        var entries = dirFile.directoryEntries;
        var files = [];
        while (entries.hasMoreElements()) {
            var file = entries.getNext().QueryInterface(Ci.nsIFile);
            var path = (dirPath ? dirPath + "/" : "") + file.leafName;
            if (file.isFile()) {
                files.push(path);
            }
            else if (file.isDirectory()) {
                files = files.concat(this.getAllRPFiles({ path: path, file: file }));
            }
        }
        return files;
    };
    FileService.prototype.fileToString = function (file) {
        var stream = this.xpc.createFileInputStreamInstance();
        stream.init(file, 0x01, 292, 0);
        stream.QueryInterface(Ci.nsILineInputStream);
        var cstream = this.xpc.createConverterInputStreamInstance();
        cstream.init(stream, "UTF-8", 0, 0);
        var str = "";
        var data = {};
        var read;
        do {
            read = cstream.readString(0xffffffff, data);
            str += data.value;
        } while (read !== 0);
        cstream.close();
        return str;
    };
    FileService.prototype.stringToFile = function (str, file) {
        var stream = this.xpc.createFileOutputStreamInstance();
        stream.init(file, 0x02 | 0x08 | 0x10 | 0x20, -1, 0);
        var cos = this.xpc.createConverterOutputStreamInstance();
        cos.init(stream, "UTF-8", 4096, 0x0000);
        cos.writeString(str);
        cos.close();
        stream.close();
    };
    FileService.prototype.getRPPath = function (aPath) {
        return REQUESTPOLICY_DIR + (aPath ? "/" + aPath : "");
    };
    FileService.prototype.getProfileFile = function (aPath) {
        return this.mozFileUtils.getFile("ProfD", aPath.split("/"));
    };
    FileService.prototype.getProfileDir = function (aPath, shouldCreate) {
        if (shouldCreate === void 0) { shouldCreate = true; }
        return this.mozFileUtils.getDir("ProfD", aPath.split("/"), shouldCreate);
    };
    return FileService;
}());
exports.FileService = FileService;