"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var constants_1 = require("data/constants");
var JsonStorage = (function () {
    function JsonStorage(filesService) {
        this.filesService = filesService;
    }
    JsonStorage.prototype.isJsonStorageKey = function (aKey) {
        return aKey.startsWith("policies/") || aKey === "subscriptions";
    };
    JsonStorage.prototype.getAll = function () {
        var _this = this;
        var allFiles = this.filesService.getAllRPFiles();
        var allPrefs = {};
        allFiles.forEach(function (path) {
            var key = _this.getKeyFromPath(path);
            allPrefs[key] = _this.get(key);
        });
        return allPrefs;
    };
    JsonStorage.prototype.get = function (aKey) {
        this.assertValidKey(aKey);
        var file = this.getFile(aKey);
        if (!file.exists())
            return constants_1.C.UNDEFINED;
        var contents = this.filesService.fileToString(file);
        return JSON.parse(contents);
    };
    JsonStorage.prototype.set = function (aKey, aValue) {
        this.assertValidKey(aKey);
        var file = this.getFile(aKey);
        var newContents = JSON.stringify(aValue);
        this.filesService.stringToFile(newContents, file);
    };
    JsonStorage.prototype.remove = function (aKey) {
        this.assertValidKey(aKey);
        var file = this.getFile(aKey);
        if (!file.exists())
            return;
        file.remove(false);
    };
    JsonStorage.prototype.assertValidKey = function (aKey) {
        if (!this.isJsonStorageKey(aKey)) {
            throw new Error("Invalid key \"" + aKey + "\".");
        }
    };
    JsonStorage.prototype.getKeyFromPath = function (aPath) {
        return aPath.replace(/\.json$/, "");
    };
    JsonStorage.prototype.getFile = function (aKey) {
        return this.filesService.getRPFile(aKey + ".json");
    };
    return JsonStorage;
}());
exports.JsonStorage = JsonStorage;