"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var PrefBranch = (function () {
    function PrefBranch(ci, prefsService, xpcService, branchRoot) {
        this.ci = ci;
        this.prefsService = prefsService;
        this.xpcService = xpcService;
        this.branchRoot = branchRoot;
        this.branch = this.prefsService.getBranch(this.branchRoot).
            QueryInterface(this.ci.nsIPrefBranch2);
    }
    PrefBranch.prototype.getAll = function () {
        var _this = this;
        var allPrefs = {};
        var childList = this.prefsService.getChildList("");
        childList.forEach(function (prefName) {
            allPrefs[prefName] = _this.get(prefName);
        });
        return allPrefs;
    };
    PrefBranch.prototype.get = function (aPrefName) {
        if (!this.isSet(aPrefName))
            return undefined;
        var type = this.branch.getPrefType(aPrefName);
        switch (type) {
            case PrefBranch.PREF_STRING:
                return this.branch.getComplexValue(aPrefName, this.ci.nsISupportsString).data;
            case PrefBranch.PREF_INT:
                return this.branch.getIntPref(aPrefName);
            case PrefBranch.PREF_BOOL:
                return this.branch.getBoolPref(aPrefName);
            case PrefBranch.PREF_INVALID:
            default:
                throw new Error("Unknown type <" + type + "> of pref \"" + aPrefName + "\"");
        }
    };
    PrefBranch.prototype.set = function (aPrefName, aValue) {
        var type = typeof aValue;
        switch (type) {
            case "string":
                var str = this.xpcService.createSupportsStringInstance();
                str.data = aValue;
                this.branch.setComplexValue(aPrefName, this.ci.nsISupportsString, str);
                break;
            case "number":
                this.branch.setIntPref(aPrefName, aValue);
                break;
            case "boolean":
                this.branch.setBoolPref(aPrefName, aValue);
                break;
            default:
                throw new Error("Invalid type <" + type + ">. Value is \"" + aValue + "\".");
        }
    };
    PrefBranch.prototype.reset = function (aPrefName) {
        this.branch.clearUserPref(aPrefName);
    };
    PrefBranch.prototype.isSet = function (aPrefName) {
        return this.branch.prefHasUserValue(aPrefName);
    };
    PrefBranch.prototype.addObserver = function (aDomain, aObserver, aHoldWeak) {
        return this.branch.addObserver(aDomain, aObserver, !!aHoldWeak);
    };
    PrefBranch.prototype.removeObserver = function (aDomain, aObserver) {
        return this.branch.removeObserver(aDomain, aObserver);
    };
    PrefBranch.PREF_INVALID = 0;
    PrefBranch.PREF_STRING = 32;
    PrefBranch.PREF_INT = 64;
    PrefBranch.PREF_BOOL = 128;
    return PrefBranch;
}());
exports.PrefBranch = PrefBranch;