"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var map_of_sets_1 = require("lib/classes/map-of-sets");
var PrefObserver = (function () {
    function PrefObserver(rpPrefBranch) {
        this.rpPrefBranch = rpPrefBranch;
        this.maps = {
            domainsToListeners: new map_of_sets_1.MapOfSets(),
            domainsToObservers: new Map(),
        };
    }
    PrefObserver.prototype.addListener = function (aDomain, aListener) {
        if (false === this.maps.domainsToObservers.has(aDomain)) {
            var observer = {
                observe: this._onObserve.bind(this, aDomain),
            };
            this.rpPrefBranch.addObserver(aDomain, observer);
            this.maps.domainsToObservers.set(aDomain, observer);
        }
        this.maps.domainsToListeners.addToSet(aDomain, aListener);
    };
    PrefObserver.prototype.addListeners = function (aDomains, aListener) {
        var e_1, _a;
        try {
            for (var aDomains_1 = __values(aDomains), aDomains_1_1 = aDomains_1.next(); !aDomains_1_1.done; aDomains_1_1 = aDomains_1.next()) {
                var domain = aDomains_1_1.value;
                this.addListener(domain, aListener);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (aDomains_1_1 && !aDomains_1_1.done && (_a = aDomains_1.return)) _a.call(aDomains_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    PrefObserver.prototype.removeListener = function (aDomain, aListener) {
        this.maps.domainsToListeners.deleteFromSet(aDomain, aListener);
        if (false === this.maps.domainsToListeners.has(aDomain)) {
            var observer = this.maps.domainsToObservers.get(aDomain);
            this.rpPrefBranch.removeObserver(aDomain, observer);
            this.maps.domainsToObservers.delete(aDomain);
        }
    };
    PrefObserver.prototype.removeListeners = function (aDomains, aListener) {
        var e_2, _a;
        try {
            for (var aDomains_2 = __values(aDomains), aDomains_2_1 = aDomains_2.next(); !aDomains_2_1.done; aDomains_2_1 = aDomains_2.next()) {
                var domain = aDomains_2_1.value;
                this.removeListener(domain, aListener);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (aDomains_2_1 && !aDomains_2_1.done && (_a = aDomains_2.return)) _a.call(aDomains_2);
            }
            finally { if (e_2) throw e_2.error; }
        }
    };
    PrefObserver.prototype.removeAllListeners = function () {
        var e_3, _a, e_4, _b;
        try {
            for (var _c = __values(this.maps.domainsToListeners.keys()), _d = _c.next(); !_d.done; _d = _c.next()) {
                var domain = _d.value;
                var listeners = this.maps.domainsToListeners.get(domain);
                try {
                    for (var _e = __values(listeners.values()), _f = _e.next(); !_f.done; _f = _e.next()) {
                        var listener = _f.value;
                        this.removeListener(domain, listener);
                    }
                }
                catch (e_4_1) { e_4 = { error: e_4_1 }; }
                finally {
                    try {
                        if (_f && !_f.done && (_b = _e.return)) _b.call(_e);
                    }
                    finally { if (e_4) throw e_4.error; }
                }
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
            }
            finally { if (e_3) throw e_3.error; }
        }
    };
    PrefObserver.prototype._onObserve = function (aDomain, aSubject, aTopic, aData) {
        var e_5, _a;
        var prefName = aData;
        var listeners = this.maps.domainsToListeners.get(aDomain);
        try {
            for (var _b = __values(listeners.values()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var listener = _c.value;
                listener.call(null, prefName);
            }
        }
        catch (e_5_1) { e_5 = { error: e_5_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_5) throw e_5.error; }
        }
    };
    return PrefObserver;
}());
exports.PrefObserver = PrefObserver;