"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var jsm_service_1 = require("bootstrap/api/services/jsm-service");
var log_1 = require("lib/classes/log");
var tryCatchUtils = require("lib/utils/try-catch-utils");
var api_module_1 = require("./api/api.module");
var extension_1 = require("./api/extension");
var async_locale_data_1 = require("./api/i18n/async-locale-data");
var i18n_module_1 = require("./api/i18n/i18n.module");
var management_1 = require("./api/management");
var manifest_1 = require("./api/manifest");
var misc_infos_1 = require("./api/misc-infos");
var network_prediction_enabled_1 = require("./api/privacy/network-prediction-enabled");
var privacy_module_1 = require("./api/privacy/privacy.module");
var runtime_1 = require("./api/runtime");
var chrome_file_service_1 = require("./api/services/chrome-file-service");
var file_service_1 = require("./api/services/file-service");
var xpconnect_service_1 = require("./api/services/xpconnect-service");
var json_storage_1 = require("./api/storage/json-storage");
var pref_branch_1 = require("./api/storage/pref-branch");
var pref_observer_1 = require("./api/storage/pref-observer");
var storage_module_1 = require("./api/storage/storage.module");
var sync_local_storage_area_1 = require("./api/storage/sync-local-storage-area");
var log = new log_1.Log();
var jsmService = new jsm_service_1.JSMService(Cu);
var mozAddonManager = jsmService.getAddonManager();
var mozNetUtil = jsmService.getNetUtil();
var mozHttp = jsmService.getHttp();
var mozFileUtils = jsmService.getFileUtils();
var mozServices = jsmService.getServices();
var extension = new extension_1.Extension(log);
var xpconnectService = new xpconnect_service_1.XPConnectService();
var fileService = new file_service_1.FileService(xpconnectService, mozFileUtils);
var chromeFileService = new chrome_file_service_1.ChromeFileService(mozNetUtil, mozHttp);
var localeData = new async_locale_data_1.AsyncLocaleData(tryCatchUtils, chromeFileService, mozServices);
var i18n = new i18n_module_1.I18n(log, localeData);
var management = new management_1.Management(log, mozAddonManager);
var manifest = new manifest_1.Manifest(log, chromeFileService);
var runtime = new runtime_1.Runtime(log, Services.appinfo);
var createPrefBranch = function (branchRoot) { return new pref_branch_1.PrefBranch(Ci, Services.prefs, xpconnectService, branchRoot); };
var rootPrefBranch = createPrefBranch("");
var rpPrefBranch = createPrefBranch("extensions.requestpolicy.");
var networkPredictionEnabled = new network_prediction_enabled_1.NetworkPredictionEnabledSetting(log, rootPrefBranch);
var privacy = new privacy_module_1.PrivacyApi(log, networkPredictionEnabled);
var prefObserverFactory = function () { return new pref_observer_1.PrefObserver(rpPrefBranch); };
var jsonPrefs = new json_storage_1.JsonStorage(fileService);
var slsa = new sync_local_storage_area_1.SyncLocalStorageArea(Services.prefs, rpPrefBranch, jsonPrefs);
var storage = new storage_module_1.Storage(log, slsa, rpPrefBranch);
var miscInfos = new misc_infos_1.MiscInfos(Services.appinfo, rpPrefBranch, Services.vc);
exports.api = new api_module_1.Api(log, extension, i18n, management, manifest, privacy, runtime, storage, Services.prefs, miscInfos, rpPrefBranch, prefObserverFactory, tryCatchUtils);