"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log_1 = require("app/log");
var constants_1 = require("data/constants");
var filtered_management_1 = require("lib/classes/filtered-management");
var notifications_1 = require("models/notifications");
var AMO = constants_1.C.AMO;
var ADDON_IDS = Object.freeze(new Set([
    "requestpolicy@requestpolicy.com",
    "rpcontinued@requestpolicy.org",
    AMO ?
        "rpcontinued@non-amo.requestpolicy.org" :
        "rpcontinued@amo.requestpolicy.org",
]));
function isRP(aAddon) {
    return ADDON_IDS.has(aAddon.id);
}
var RPManagement = new filtered_management_1.FilteredManagement(isRP);
function addNotification() {
    notifications_1.Notifications.add(notifications_1.NotificationID.MultipleRPInstallations);
}
function maybeAddNotification(aAddon) {
    if (!isRP(aAddon))
        return;
    addNotification();
}
exports.OtherRPInstallationsController = {
    startup: function () {
        RPManagement.getAll().then(function (addons) {
            var enabledAddons = addons.filter(function (addon) { return addon.enabled; });
            if (enabledAddons.length === 0)
                return;
            addNotification();
        }).catch(function (e) {
            log_1.log.error("Error getting the list of addons! Details:", e);
        });
        browser.management.onEnabled.addListener(maybeAddNotification);
        browser.management.onInstalled.addListener(maybeAddNotification);
    },
};