"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log_1 = require("app/log");
var XULUtils = require("lib/utils/xul-utils");
var CustomizableUI = null;
if (LegacyApi.miscInfos.isAustralis) {
    (CustomizableUI = Cu.import("resource:///modules/CustomizableUI.jsm", {}).CustomizableUI);
}
exports.ToolbarButtonController = (function () {
    var self = {};
    var toolbarButtonId = "rpcontinuedOffAmoToolbarButton";
    var isAustralis = LegacyApi.miscInfos.isAustralis;
    (function () {
        if (!isAustralis) {
            return;
        }
        function removeToolbarButtonFromAustralis() {
            var id = XULUtils.xulTrees.toolbarbutton[0].attributes.id;
            CustomizableUI.destroyWidget(id);
        }
        function addToolbarButtonToAustralis() {
            var _a = XULUtils.xulTrees.toolbarbutton[0].attributes, id = _a.id, label = _a.label, tooltiptext = _a.tooltiptext;
            CustomizableUI.createWidget({
                id: id,
                defaultArea: CustomizableUI.AREA_NAVBAR,
                label: label,
                tooltiptext: tooltiptext,
                onCommand: function (aEvent) {
                    var win = aEvent.target.ownerDocument.defaultView;
                    win.rpcontinued.overlay.toggleMenu();
                },
            });
        }
        self.startup = function () {
            addToolbarButtonToAustralis();
        };
        self.shutdown = function () {
            removeToolbarButtonFromAustralis();
        };
    })();
    (function () {
        if (isAustralis) {
            return;
        }
        function addToolbarButtonToNavBar(win) {
            var doc = win.document;
            var isFirstRun = false;
            if (Services.vc.compare(LegacyApi.miscInfos.lastAppVersion, "0.0") <= 0) {
                log_1.log.info("This is the first run.");
                isFirstRun = true;
            }
            var id = toolbarButtonId;
            var toolbarSelector = "[currentset^='" + id + ",'],[currentset*='," + id + ",'],[currentset$='," + id + "']";
            var toolbar = doc.querySelector(toolbarSelector);
            if (!toolbar) {
                if (isFirstRun) {
                    toolbar = doc.getElementById("nav-bar");
                    toolbar.insertItem(id);
                    toolbar.setAttribute("currentset", toolbar.currentSet);
                    doc.persist(toolbar.id, "currentset");
                }
            }
            else {
                var currentset = toolbar.getAttribute("currentset").split(",");
                var index = currentset.indexOf(id);
                var before = null;
                for (var i = index + 1, len = currentset.length; i < len; i++) {
                    before = doc.getElementById(currentset[i]);
                    if (before) {
                        toolbar.insertItem(id, before);
                        break;
                    }
                }
                if (!before) {
                    toolbar.insertItem(id);
                }
            }
        }
        self.loadIntoWindow = function (win) {
            if (!isAustralis) {
                XULUtils.addTreeElementsToWindow(win, "toolbarbutton");
                addToolbarButtonToNavBar(win);
            }
        };
        self.unloadFromWindow = function (win) {
            XULUtils.removeTreeElementsFromWindow(win, "toolbarbutton");
        };
    })();
    return self;
})();