"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var app_content_1 = require("app/app.content");
var log_1 = require("app/log");
var environment_1 = require("lib/environment");
var html_utils_1 = require("lib/utils/html-utils");
var constants_1 = require("data/constants");
var managers_1 = require("framescripts/managers");
var blocked_content_js_1 = require("framescripts/blocked-content.js");
var uriServices = app_content_1.rp.services.uri;
exports.ManagerForDOMContentLoaded = (function () {
    var self = {};
    var content = cfmm.content;
    function htmlAnchorTagClicked(event) {
        managers_1.overlayComm.run(function () {
            cfmm.sendSyncMessage(constants_1.C.MM_PREFIX + "notifyLinkClicked", { origin: event.currentTarget.ownerDocument.URL,
                dest: event.currentTarget.href });
        });
    }
    function isActiveTopLevelDocument(documentToCheck) {
        return documentToCheck === content.document;
    }
    function onDOMContentLoaded(event) {
        var doc = event.originalTarget;
        if (doc.nodeName !== "#document") {
            return;
        }
        onDocumentLoaded(doc);
        var pBlockedURIs = browser.runtime.sendMessage({
            type: "notifyDocumentLoaded",
            documentURI: doc.documentURI,
        }).then(function (aResponse) {
            if (typeof aResponse !== "object" ||
                typeof aResponse.blockedURIs !== "object") {
                console.error("[RequestPolicy] There seems to be no message " +
                    "listener for \"notifyDocumentLoaded\".");
                return null;
            }
            var blockedURIs = aResponse.blockedURIs;
            return blockedURIs;
        });
        pBlockedURIs.then(function (blockedURIs) {
            if (blockedURIs !== null) {
                blocked_content_js_1.ManagerForBlockedContent.indicateBlockedVisibleObjects(doc, blockedURIs);
            }
            return;
        }).catch(function (e) {
            console.error("[RequestPolicy] " + e);
        });
        pBlockedURIs.then(function (blockedURIs) {
            if (blockedURIs !== null && isActiveTopLevelDocument(doc)) {
                cfmm.sendAsyncMessage(constants_1.C.MM_PREFIX + "notifyTopLevelDocumentLoaded");
            }
            return;
        }).catch(function (e) {
            console.error("[RequestPolicy] " + e);
        });
    }
    function onDOMFrameContentLoaded(event) {
        var iframe = event.target;
        if (iframe.contentDocument === undefined) {
            return;
        }
        if (isActiveTopLevelDocument(iframe.ownerDocument)) {
            managers_1.overlayComm.run(function () {
                cfmm.sendAsyncMessage(constants_1.C.MM_PREFIX + "notifyDOMFrameContentLoaded");
            });
        }
    }
    function onDocumentLoaded(doc) {
        var e_1, _a;
        var DocEnv = new environment_1.Environment(environment_1.MainEnvironment, "DocEnv");
        DocEnv.shutdownOnUnload(doc.defaultView);
        DocEnv.startup();
        var documentURI = doc.documentURI;
        var metaRefreshes = [];
        var metaTags = doc.getElementsByTagName("meta");
        for (var i = 0; i < metaTags.length; i++) {
            var metaTag = metaTags[i];
            if (!metaTag.httpEquiv || metaTag.httpEquiv.toLowerCase() !== "refresh") {
                continue;
            }
            var originalDestURI = null;
            var _b = html_utils_1.parseRefresh(metaTag.content), delay = _b.delay, destURI = _b.destURI;
            if (!uriServices.isValidUri(destURI)) {
                originalDestURI = destURI;
                destURI = doc.documentURIObject.resolve(destURI);
            }
            metaRefreshes.push({ delay: delay, destURI: destURI,
                originalDestURI: originalDestURI });
        }
        if (metaRefreshes.length > 0) {
            log_1.log.info("Number of meta refreshes found: " + metaRefreshes.length);
            var docShell = doc.defaultView.
                QueryInterface(Ci.nsIInterfaceRequestor).
                getInterface(Ci.nsIWebNavigation).
                QueryInterface(Ci.nsIDocShell);
            if (!docShell.allowMetaRedirects) {
                log_1.log.warn("Another extension disabled docShell.allowMetaRedirects.");
            }
            managers_1.overlayComm.run(function () {
                cfmm.sendAsyncMessage(constants_1.C.MM_PREFIX + "handleMetaRefreshes", { documentURI: documentURI, metaRefreshes: metaRefreshes });
            });
        }
        var anchorTags = doc.getElementsByTagName("a");
        try {
            for (var anchorTags_1 = __values(anchorTags), anchorTags_1_1 = anchorTags_1.next(); !anchorTags_1_1.done; anchorTags_1_1 = anchorTags_1.next()) {
                var anchorTag = anchorTags_1_1.value;
                anchorTag.addEventListener("click", htmlAnchorTagClicked, false);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (anchorTags_1_1 && !anchorTags_1_1.done && (_a = anchorTags_1.return)) _a.call(anchorTags_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        DocEnv.addShutdownFunction(environment_1.Level.INTERFACE, function () {
            var e_2, _a;
            try {
                for (var anchorTags_2 = __values(anchorTags), anchorTags_2_1 = anchorTags_2.next(); !anchorTags_2_1.done; anchorTags_2_1 = anchorTags_2.next()) {
                    var anchorTag = anchorTags_2_1.value;
                    anchorTag.removeEventListener("click", htmlAnchorTagClicked, false);
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (anchorTags_2_1 && !anchorTags_2_1.done && (_a = anchorTags_2.return)) _a.call(anchorTags_2);
                }
                finally { if (e_2) throw e_2.error; }
            }
        });
    }
    environment_1.MainEnvironment.elManager.addListener(cfmm, "DOMContentLoaded", onDOMContentLoaded, true);
    environment_1.MainEnvironment.elManager.addListener(cfmm, "DOMFrameContentLoaded", onDOMFrameContentLoaded, true);
    return self;
})();