"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log_1 = require("app/log");
var environment_1 = require("lib/environment");
var constants_1 = require("data/constants");
var managers_1 = require("framescripts/managers");
var States = {
    "WAITING": 0,
    "RUNNING": 1,
    "STOPPED": 2,
};
function FramescriptToOverlayCommunication(aEnv) {
    var self = this;
    self.waitingRunnables = [];
    self.state = States.WAITING;
    self.env = aEnv;
    self.env.addStartupFunction(environment_1.Level.INTERFACE, startCommNowOrLater.bind(null, self));
    self.env.addShutdownFunction(environment_1.Level.INTERFACE, stopCommunication.bind(null, self));
}
exports.FramescriptToOverlayCommunication = FramescriptToOverlayCommunication;
FramescriptToOverlayCommunication.prototype._dump = function (msg) {
    var self = this;
    log_1.log.log(self.env.uid + ": " + msg);
};
function startCommNowOrLater(self) {
    var answers = cfmm.sendSyncMessage(constants_1.C.MM_PREFIX + "isOverlayReady");
    if (answers.length > 0 && answers[0] === true) {
        startCommunication(self);
    }
    else {
        managers_1.mlManager.addListener("overlayIsReady", startCommunication.bind(null, self), undefined, true);
    }
}
function startCommunication(self) {
    if (self.state === States.WAITING) {
        self.state = States.RUNNING;
        while (self.waitingRunnables.length !== 0) {
            var runnable = self.waitingRunnables.shift();
            runnable.call(null);
        }
    }
}
function stopCommunication(self) {
    self.state = States.STOPPED;
}
FramescriptToOverlayCommunication.prototype.run = function (aRunnable) {
    var self = this;
    switch (self.state) {
        case States.RUNNING:
            aRunnable.call(null);
            break;
        case States.WAITING:
            self.waitingRunnables.push(aRunnable);
            break;
        default:
            break;
    }
};