"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var listener_factories_1 = require("lib/utils/listener-factories");
var MANAGEMENT_EVENT_NAMES = [
    "onDisabled",
    "onEnabled",
    "onInstalled",
    "onUninstalled",
];
var FilteredManagement = (function () {
    function FilteredManagement(filter) {
        var _this = this;
        this.evntLsnrsMap = listener_factories_1.createListenersMap(MANAGEMENT_EVENT_NAMES).listenersMap;
        this.filter = filter;
        MANAGEMENT_EVENT_NAMES.forEach(function (eventName) {
            browser.management[eventName].
                addListener(_this.onManagementEvent.bind(_this, eventName));
        });
    }
    Object.defineProperty(FilteredManagement.prototype, "onDisabled", {
        get: function () { return this.evntLsnrsMap.onDisabled.interface; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FilteredManagement.prototype, "onEnabled", {
        get: function () { return this.evntLsnrsMap.onEnabled.interface; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FilteredManagement.prototype, "onInstalled", {
        get: function () { return this.evntLsnrsMap.onInstalled.interface; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FilteredManagement.prototype, "onUninstalled", {
        get: function () {
            return this.evntLsnrsMap.onUninstalled.interface;
        },
        enumerable: true,
        configurable: true
    });
    FilteredManagement.prototype.getAll = function () {
        var _this = this;
        return browser.management.getAll().
            then(function (addons) { return addons.filter(_this.filter); });
    };
    FilteredManagement.prototype.onManagementEvent = function (aEventName, aExtensionInfo) {
        if (!this.filter(aExtensionInfo))
            return;
        this.evntLsnrsMap[aEventName].emit(aExtensionInfo);
    };
    return FilteredManagement;
}());
exports.FilteredManagement = FilteredManagement;