"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var constants_1 = require("data/constants");
function GUILocation(value, properties) {
    this.value = value || null;
    this.properties = properties || null;
}
exports.GUILocation = GUILocation;
GUILocation.prototype.toString = function () {
    return this.value;
};
GUILocation.merge = function (SubclassOfGUILocation, location1, location2) {
    return new SubclassOfGUILocation(location1.value, GUILocationProperties.merge(location1.properties, location2.properties));
};
GUILocation.existsInArray = function (locationString, locations) {
    return GUILocation.indexOfLocationInArray(locationString, locations) !== -1;
};
GUILocation.indexOfLocationInArray = function (aLocationString, locations) {
    var locationString = aLocationString instanceof GUILocation ? aLocationString.value :
        aLocationString;
    for (var i in locations) {
        if (locations[i].value === locationString) {
            return i;
        }
    }
    return -1;
};
GUILocation.sortByNumRequestsCompareFunction = function (a, b) {
    return GUILocation.compareFunction(a, b, "sortByNumRequests");
};
GUILocation.compareFunction = function (a, b, sortType) {
    var aDefault = 0 < a.properties.numDefaultPolicyRequests;
    var bDefault = 0 < b.properties.numDefaultPolicyRequests;
    if (aDefault !== bDefault) {
        if (aDefault === true) {
            return -1;
        }
        else {
            return 1;
        }
    }
    if (sortType === "sortByNumRequests") {
        if (a.properties.numRequests > b.properties.numRequests) {
            return -1;
        }
        if (a.properties.numRequests < b.properties.numRequests) {
            return 1;
        }
    }
    if (a.value > b.value) {
        return 1;
    }
    if (a.value < b.value) {
        return -1;
    }
    return 0;
};
function GUIOrigin(origin, properties) {
    GUILocation.call(this, origin, properties);
}
exports.GUIOrigin = GUIOrigin;
GUIOrigin.prototype = new GUILocation();
GUIOrigin.merge = GUILocation.merge.bind(GUIOrigin, GUIOrigin);
GUIOrigin.indexOfOriginInArray = GUILocation.indexOfLocationInArray;
function GUIDestination(dest, properties) {
    GUILocation.call(this, dest, properties);
}
exports.GUIDestination = GUIDestination;
GUIDestination.prototype = new GUILocation();
GUIDestination.merge = GUILocation.merge.bind(GUIDestination, GUIDestination);
GUIDestination.indexOfDestInArray = GUILocation.indexOfLocationInArray;
function GUILocationProperties(value, properties) {
    this.reset();
}
exports.GUILocationProperties = GUILocationProperties;
GUILocationProperties.prototype.reset = function () {
    this.numRequests = 0;
    this.numDefaultPolicyRequests = 0;
    this.numAllowedRequests = 0;
    this.numBlockedRequests = 0;
};
GUILocationProperties.prototype.accumulate = function (requests, ruleAction) {
    var extractRuleActions = undefined === ruleAction;
    var ruleActionCounter = 0;
    for (var destIdent in requests) {
        var destIdentRequests = requests[destIdent];
        for (var destUri in destIdentRequests) {
            var destUriRequests = destIdentRequests[destUri];
            for (var i in destUriRequests) {
                var request = destUriRequests[i];
                ++this.numRequests;
                if (!extractRuleActions) {
                    ++ruleActionCounter;
                }
                else if (request.isAllowed) {
                    ++this.numAllowedRequests;
                }
                else {
                    ++this.numBlockedRequests;
                }
                if (request.isDefaultPolicyUsed()) {
                    ++this.numDefaultPolicyRequests;
                }
            }
        }
    }
    switch (ruleAction) {
        case constants_1.C.RULE_ACTION_ALLOW:
            this.numAllowedRequests += ruleActionCounter;
            break;
        case constants_1.C.RULE_ACTION_DENY:
            this.numBlockedRequests += ruleActionCounter;
            break;
        default:
            break;
    }
};
GUILocationProperties.requestCountProperties = [
    "numRequests",
    "numDefaultPolicyRequests",
    "numBlockedRequests",
    "numAllowedRequests",
];
GUILocationProperties.merge = function (prop1, prop2) {
    var e_1, _a;
    var requestCountProperties = GUILocationProperties.requestCountProperties;
    var newObj = new GUILocationProperties();
    try {
        for (var requestCountProperties_1 = __values(requestCountProperties), requestCountProperties_1_1 = requestCountProperties_1.next(); !requestCountProperties_1_1.done; requestCountProperties_1_1 = requestCountProperties_1.next()) {
            var propertyName = requestCountProperties_1_1.value;
            newObj[propertyName] += prop1[propertyName] + prop2[propertyName];
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (requestCountProperties_1_1 && !requestCountProperties_1_1.done && (_a = requestCountProperties_1.return)) _a.call(requestCountProperties_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return newObj;
};