"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var app_background_1 = require("app/app.background");
var windows_1 = require("models/windows");
var manager_for_pref_observer_1 = require("lib/manager-for-pref-observer");
var XULUtils = require("lib/utils/xul-utils");
var environment_1 = require("lib/environment");
var cachedSettings = app_background_1.rp.storage.cachedSettings;
function KeyboardShortcut(aID, aDefaultCombo, aCallback, aUserEnabledPrefName, aUserComboPrefName) {
    this._id = aID;
    this._elementID = "rpKey_" + this._id;
    this._defaultCombo = aDefaultCombo;
    this._callback = aCallback;
    this._userEnabledPrefName = aUserEnabledPrefName;
    this._userComboPrefName = aUserComboPrefName;
    this._elementAttributes = {
        disabled: null,
        modifiers: null,
        key: null,
    };
    this._listeners = {
        onWindowLoad: this._loadIntoWindow.bind(this),
        onWindowUnload: this._unloadFromWindow.bind(this),
        onKeyCommand: this._onPress.bind(this),
        onPrefChange: this._onPrefChange.bind(this),
    };
    this._determineElementAttributes();
    windows_1.Windows.forEachOpenWindow(this._loadIntoWindow.bind(this));
    windows_1.Windows.addListener("load", this._listeners.onWindowLoad);
    windows_1.Windows.addListener("unload", this._listeners.onWindowUnload);
    manager_for_pref_observer_1.ManagerForPrefObservers.get(environment_1.MainEnvironment).addListeners([
        this._userEnabledPrefName,
        this._userComboPrefName,
    ], this._listeners.onPrefChange);
}
exports.KeyboardShortcut = KeyboardShortcut;
KeyboardShortcut.prototype.destroy = function () {
    windows_1.Windows.forEachOpenWindow(this._unloadFromWindow.bind(this));
    windows_1.Windows.removeListener("load", this._listeners.onWindowLoad);
    windows_1.Windows.removeListener("unload", this._listeners.onWindowUnload);
    manager_for_pref_observer_1.ManagerForPrefObservers.get(environment_1.MainEnvironment).removeListeners([
        this._userEnabledPrefName,
        this._userComboPrefName,
    ], this._listeners.onPrefChange);
};
KeyboardShortcut.prototype._loadIntoWindow = function (window) {
    this._createElement(window);
    this._setElementAttributes(window);
};
KeyboardShortcut.prototype._unloadFromWindow = function (window) {
    this._removeElement(window);
};
KeyboardShortcut.prototype._onPress = function (event) {
    var window = event.target.ownerDocument.defaultView;
    this._callback.call(null, window);
};
KeyboardShortcut.prototype._onPrefChange = function () {
    var _this = this;
    this._determineElementAttributes();
    windows_1.Windows.forEachOpenWindow(function (window) {
        _this._setElementAttributes(window);
        var keyset = window.document.getElementById("rpcontinuedKeyset");
        keyset.parentNode.appendChild(keyset);
    });
};
KeyboardShortcut.prototype._createElement = function (window) {
    var keyset = window.document.getElementById("rpcontinuedKeyset");
    var element = window.document.createElement("key");
    element.setAttribute("id", this._elementID);
    element.setAttribute("disabled", "true");
    element.setAttribute("oncommand", "void(0);");
    element.addEventListener("command", this._listeners.onKeyCommand, false);
    keyset.appendChild(element);
};
KeyboardShortcut.prototype._setElementAttributes = function (window) {
    var element = window.document.getElementById(this._elementID);
    element.setAttribute("modifiers", this._elementAttributes.modifiers);
    element.setAttribute("key", this._elementAttributes.key);
    element.setAttribute("disabled", this._elementAttributes.disabled);
};
KeyboardShortcut.prototype._removeElement = function (window) {
    var element = window.document.getElementById(this._elementID);
    element.removeEventListener("command", this._listeners.onKeyCommand, false);
    element.remove();
};
Object.defineProperty(KeyboardShortcut.prototype, "userCombo", {
    get: function () {
        return cachedSettings.get(this._userComboPrefName);
    },
});
Object.defineProperty(KeyboardShortcut.prototype, "userEnabled", {
    get: function () {
        return cachedSettings.get(this._userEnabledPrefName);
    },
});
var ELEMENT_ATTRIBUTES_WHEN_DISABLED = {
    disabled: "true",
    modifiers: "",
    key: "",
};
KeyboardShortcut.prototype._determineElementAttributes = function () {
    if (!this.userEnabled) {
        this._elementAttributes = ELEMENT_ATTRIBUTES_WHEN_DISABLED;
        return;
    }
    var combo = this.userCombo;
    if (combo === "default") {
        combo = this._defaultCombo;
    }
    if (combo === "none") {
        this._elementAttributes = ELEMENT_ATTRIBUTES_WHEN_DISABLED;
        return;
    }
    var rv = XULUtils.keyboardShortcuts.getKeyAttributesFromCombo(combo);
    if (false === rv.success) {
        console.error("[RequestPolicy] Error parsing keyboard combination for shortcut " +
            ("\"" + this._id + "\": " + rv.errorMessage));
        this._elementAttributes = ELEMENT_ATTRIBUTES_WHEN_DISABLED;
        return;
    }
    this._elementAttributes = {
        disabled: "false",
        modifiers: rv.modifiers,
        key: rv.key,
    };
};