"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var constants_1 = require("data/constants");
var LogLevel;
(function (LogLevel) {
    LogLevel[LogLevel["OFF"] = Number.MAX_VALUE] = "OFF";
    LogLevel[LogLevel["ERROR"] = 1000] = "ERROR";
    LogLevel[LogLevel["WARNING"] = 900] = "WARNING";
    LogLevel[LogLevel["INFO"] = 800] = "INFO";
    LogLevel[LogLevel["DEBUG"] = 700] = "DEBUG";
    LogLevel[LogLevel["ALL"] = Number.MIN_VALUE] = "ALL";
})(LogLevel = exports.LogLevel || (exports.LogLevel = {}));
var MINIMUM_LOGGING_LEVEL = LogLevel.ERROR;
function dir() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    args.forEach(function (arg) { return console.dir(arg); });
}
var DEFAULT_OPTIONS = Object.freeze({
    enabled: null,
    enabledCondition: null,
    level: null,
    name: null,
    prefix: null,
});
var ROOT_OPTIONS = {
    enabled: true,
    level: MINIMUM_LOGGING_LEVEL,
    prefix: constants_1.C.LOG_PREFIX,
};
var Log = (function () {
    function Log(aOptions, aParent) {
        this.ownInternalOptions = {
            enabled: null,
            level: null,
            prefix: "",
        };
        this.ownDelayedLogs = new Map();
        this.parent = null;
        var options = Object.assign({}, DEFAULT_OPTIONS, aOptions);
        var enabled = options.enabled, name = options.name;
        var enabledCondition = options.enabledCondition, level = options.level, prefix = options.prefix;
        if (enabledCondition !== null && enabled !== null) {
            console.error("[RequestPolicy] Both 'enabledCondition' and 'enabled' specified! " +
                "Ignoring 'enabled'.");
            enabled = null;
        }
        if (enabled !== null)
            this.setEnabled(enabled);
        if (enabledCondition !== null)
            this.setEnabledIf(enabledCondition);
        this.setLevel(level);
        if (name !== null && prefix !== null) {
            console.error("[RequestPolicy] Both 'name' and 'prefix' specified! " +
                "Ignoring 'name'.");
            name = null;
        }
        if (name !== null)
            this.setName(name);
        if (prefix !== null)
            this.setPrefix(name);
        if (aParent)
            this.parent = aParent;
    }
    Object.defineProperty(Log.prototype, "enabled", {
        get: function () {
            return this.getInternalOption("enabled");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Log.prototype, "level", {
        get: function () {
            return this.getInternalOption("level");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Log.prototype, "prefix", {
        get: function () {
            var parentPrefix = this.parent ? this.parent.prefix : ROOT_OPTIONS.prefix;
            return parentPrefix + this.ownInternalOptions.prefix;
        },
        enumerable: true,
        configurable: true
    });
    Log.prototype.cbDir = function (cb) {
        if (!this.shouldLog(LogLevel.DEBUG))
            return;
        var _a = __read(cb()), msg = _a[0], args = _a.slice(1);
        this.dir.apply(this, __spread([msg], args));
    };
    Log.prototype.cbError = function (cb) {
        if (!this.shouldLog(LogLevel.ERROR))
            return;
        var _a = __read(cb(), 2), msg = _a[0], error = _a[1];
        this.error(msg, error);
    };
    Log.prototype.cbLog = function (cb) {
        if (this.shouldLog(LogLevel.DEBUG))
            return;
        var _a = __read(cb()), msg = _a[0], args = _a.slice(1);
        this.dir.apply(this, __spread([msg], args));
    };
    Log.prototype.cbInfo = function (cb) {
        if (this.shouldLog(LogLevel.INFO))
            return;
        var _a = __read(cb()), msg = _a[0], args = _a.slice(1);
        this.dir.apply(this, __spread([msg], args));
    };
    Log.prototype.cbWarn = function (cb) {
        if (this.shouldLog(LogLevel.WARNING))
            return;
        var _a = __read(cb()), msg = _a[0], args = _a.slice(1);
        this.dir.apply(this, __spread([msg], args));
    };
    Log.prototype.dir = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (!this.shouldLog(LogLevel.DEBUG))
            return;
        dir.apply(void 0, __spread(args));
    };
    Log.prototype.error = function (msg) {
        var errorsAndOtherDirArgs = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            errorsAndOtherDirArgs[_i - 1] = arguments[_i];
        }
        this.maybeLogInternal(LogLevel.ERROR, "error", msg, errorsAndOtherDirArgs);
    };
    Log.prototype.log = function (msg) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        this.maybeLogInternal(LogLevel.DEBUG, "log", msg, args);
    };
    Log.prototype.info = function (msg) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        this.maybeLogInternal(LogLevel.INFO, "info", msg, args);
    };
    Log.prototype.warn = function (msg) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        this.maybeLogInternal(LogLevel.WARNING, "warn", msg, args);
    };
    Log.prototype.onError = function (msg) {
        var _this = this;
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        return function (error) {
            _this.error.apply(_this, __spread([msg, error], args));
        };
    };
    Log.prototype.setEnabled = function (enabled) {
        var _this = this;
        if (typeof enabled.then === "function") {
            var pEnabled_1 = enabled;
            var delayedLog_1 = [];
            this.ownDelayedLogs.set(pEnabled_1, delayedLog_1);
            pEnabled_1.then(function (finallyEnabled) {
                if (typeof finallyEnabled !== "boolean") {
                    console.error("[RequestPolicy] [" + _this.ownInternalOptions.prefix + "] promise was " +
                        "resolved with a non-boolean value! Using 'true' instead.");
                    console.dir(finallyEnabled);
                    finallyEnabled = true;
                }
                if (_this.ownInternalOptions.enabled === pEnabled_1) {
                    _this.ownInternalOptions.enabled = finallyEnabled;
                }
                delayedLog_1.forEach(function (_a) {
                    var forceLog = _a.forceLog, logArgs = _a.logArgs;
                    if (finallyEnabled || forceLog) {
                        logArgs[2][0] = "[DELAYED] " + logArgs[2][0];
                        _this.logNowInternal.apply(_this, logArgs);
                    }
                });
                _this.ownDelayedLogs.delete(pEnabled_1);
            }).catch(function (e) {
                console.error("[RequestPolicy] An error occurred:");
                console.dir(e);
            });
        }
        else if (typeof enabled !== "boolean") {
            throw new Error("'" + enabled + "' is not boolean!");
        }
        this.ownInternalOptions.enabled = enabled;
    };
    Log.prototype.setEnabledIf = function (condition) {
        var conditionFulfilled;
        switch (typeof condition) {
            case "object":
                switch (condition.type) {
                    case "C":
                        if (typeof condition.C !== "string") {
                            console.error("[RequestPolicy] Log: condition.C is not a string!");
                            return;
                        }
                        if (!(condition.C in constants_1.C)) {
                            console.error("[RequestPolicy] Log: Constant \"" + condition.C + "\" does not exist!");
                            return;
                        }
                        if (typeof constants_1.C[condition.C] !== "boolean") {
                            console.warn("[RequestPolicy] Log: C[\"" + condition.C + "\"] is not a bool.");
                        }
                        conditionFulfilled = !!constants_1.C[condition.C];
                        break;
                    default:
                        console.error("[RequestPolicy] Log: Invalid condition type");
                        return;
                }
                break;
            default:
                conditionFulfilled = !!condition;
                break;
        }
        this.setEnabled(conditionFulfilled);
    };
    Log.prototype.setLevel = function (level) {
        if (level === "all")
            level = LogLevel.ALL;
        this.ownInternalOptions.level = level;
    };
    Log.prototype.setName = function (name) {
        this.ownInternalOptions.prefix = name ? "[" + name + "] " : "";
    };
    Log.prototype.setPrefix = function (prefix) {
        this.ownInternalOptions.prefix = prefix || "";
    };
    Log.prototype.extend = function (options) {
        return new Log(options, this);
    };
    Log.prototype.getInternalOption = function (aOption) {
        if (this.ownInternalOptions[aOption] !== null) {
            return this.ownInternalOptions[aOption];
        }
        if (this.parent !== null) {
            return this.parent[aOption];
        }
        return ROOT_OPTIONS[aOption];
    };
    Log.prototype.getDelayedLog = function (aEnabledPromise) {
        if (this.ownDelayedLogs.has(aEnabledPromise)) {
            return this.ownDelayedLogs.get(aEnabledPromise);
        }
        if (this.parent !== null) {
            return this.parent.getDelayedLog(aEnabledPromise);
        }
        throw new Error("Promise not found: " + aEnabledPromise);
    };
    Log.prototype.shouldLog = function (aLevel) {
        if (aLevel >= MINIMUM_LOGGING_LEVEL) {
            return true;
        }
        if (constants_1.C.UI_TESTING && aLevel >= LogLevel.WARNING) {
            return true;
        }
        if (aLevel >= this.level) {
            var isPromise = typeof this.enabled !== "boolean";
            if (isPromise) {
                return this.enabled;
            }
            else if (this.enabled) {
                return true;
            }
        }
        return false;
    };
    Log.prototype.maybeLogInternal = function (aLevel, aFnName, aMsg, aDirArgs) {
        var shouldLog = this.shouldLog(aLevel);
        if (shouldLog === false)
            return;
        var msgs = typeof aMsg === "object" ? aMsg : [aMsg];
        var isPromise = typeof this.enabled !== "boolean";
        if (!isPromise) {
            return this.logNowInternal(aLevel, aFnName, msgs, aDirArgs);
        }
        var forceLog = shouldLog === true;
        var delayedLog = this.getDelayedLog(this.enabled);
        if (!delayedLog) {
            console.error("[RequestPolicy] delayedLog for " + this.enabled + " is undefined");
            console.dir(this.enabled);
            return;
        }
        delayedLog.push({
            forceLog: forceLog,
            logArgs: [aLevel, aFnName, msgs, aDirArgs],
        });
    };
    Log.prototype.logNowInternal = function (aLevel, aFnName, aMsgs, aDirArgs) {
        var _a = __read(aMsgs), firstMsg = _a[0], otherMsgs = _a.slice(1);
        console[aFnName].apply(console, __spread([this.prefix + firstMsg], otherMsgs));
        dir.apply(void 0, __spread(aDirArgs));
    };
    return Log;
}());
exports.Log = Log;