"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var js_utils_1 = require("lib/utils/js-utils");
var MaybePromise = (function () {
    function MaybePromise(spec) {
        this.status = spec.status;
        switch (spec.status) {
            case "PROMISE":
                this.promise = spec.promise;
                break;
            case "fulfilled":
                this.resolutionValue = spec.resolutionValue;
                break;
            case "rejected":
                this.rejectionValue = spec.rejectionValue;
                break;
        }
    }
    MaybePromise.resolve = function (aVal) {
        if (aVal instanceof MaybePromise)
            return aVal;
        if (js_utils_1.isThenable(aVal)) {
            return new MaybePromise({
                promise: aVal,
                status: "PROMISE",
            });
        }
        else {
            return new MaybePromise({
                resolutionValue: aVal,
                status: "fulfilled",
            });
        }
    };
    MaybePromise.reject = function (aRejectValue) {
        return new MaybePromise({
            rejectionValue: aRejectValue,
            status: "rejected",
        });
    };
    MaybePromise.prototype.then = function (aThen, aCatch) {
        if (this.status === "PROMISE") {
            return MaybePromise.resolve(this.promise.then(aThen, aCatch));
        }
        if (this.status === "rejected") {
            if (aCatch) {
                try {
                    return MaybePromise.resolve(aCatch(this.rejectionValue));
                }
                catch (e) {
                    return MaybePromise.reject(e);
                }
            }
            return MaybePromise.reject(this.rejectionValue);
        }
        if (aThen) {
            try {
                return MaybePromise.resolve(aThen(this.resolutionValue));
            }
            catch (e) {
                return MaybePromise.reject(e);
            }
        }
        return MaybePromise.resolve(this.resolutionValue);
    };
    MaybePromise.prototype.catch = function (aCatch) {
        return this.then(undefined, aCatch);
    };
    MaybePromise.prototype.getStatus = function () { return this.status; };
    MaybePromise.prototype.isPromiseWrapper = function () { return this.status === "PROMISE"; };
    MaybePromise.prototype.isFulfilled = function () {
        this.assertIsNoPromiseWrapper();
        return this.status === "fulfilled";
    };
    MaybePromise.prototype.isRejected = function () {
        this.assertIsNoPromiseWrapper();
        return this.status === "rejected";
    };
    MaybePromise.prototype.toPromise = function () {
        if (this.status === "PROMISE")
            return this.promise;
        if (this.status === "fulfilled") {
            return Promise.resolve(this.resolutionValue);
        }
        return Promise.reject(this.rejectionValue);
    };
    MaybePromise.prototype.getPromise = function () {
        this.assertIsPromiseWrapper();
        return this.promise;
    };
    MaybePromise.prototype.getValue = function () {
        this.assertIsNoPromiseWrapper();
        if (this.status === "rejected") {
            throw new Error("The MaybePromise has been rejected!");
        }
        return this.resolutionValue;
    };
    MaybePromise.prototype.getRejectionValue = function () {
        this.assertIsNoPromiseWrapper();
        if (this.status === "fulfilled") {
            throw new Error("The MaybePromise has been fulfilled!");
        }
        return this.rejectionValue;
    };
    MaybePromise.prototype.assertIsPromiseWrapper = function () {
        if (this.status !== "PROMISE") {
            throw new Error("The MaybePromise is NOT a Promise wrapper!");
        }
    };
    MaybePromise.prototype.assertIsNoPromiseWrapper = function () {
        if (this.status === "PROMISE") {
            throw new Error("The MaybePromise is a Promise wrapper!");
        }
    };
    return MaybePromise;
}());
exports.MaybePromise = MaybePromise;