"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var log_1 = require("lib/classes/log");
var manager_for_event_listeners_1 = require("lib/manager-for-event-listeners");
var observer_manager_1 = require("lib/observer-manager");
var State;
(function (State) {
    State[State["NOT_STARTED"] = 0] = "NOT_STARTED";
    State[State["STARTING_UP"] = 1] = "STARTING_UP";
    State[State["STARTUP_DONE"] = 2] = "STARTUP_DONE";
    State[State["SHUTTING_DOWN"] = 3] = "SHUTTING_DOWN";
    State[State["SHUT_DOWN"] = 4] = "SHUT_DOWN";
})(State = exports.State || (exports.State = {}));
var Level;
(function (Level) {
    Level[Level["ESSENTIAL"] = 1] = "ESSENTIAL";
    Level[Level["BACKEND"] = 2] = "BACKEND";
    Level[Level["INTERFACE"] = 3] = "INTERFACE";
    Level[Level["UI"] = 4] = "UI";
})(Level = exports.Level || (exports.Level = {}));
var LevelState;
(function (LevelState) {
    LevelState[LevelState["NOT_ENTERED"] = 0] = "NOT_ENTERED";
    LevelState[LevelState["PROCESSING"] = 1] = "PROCESSING";
    LevelState[LevelState["FINISHED_PROCESSING"] = 2] = "FINISHED_PROCESSING";
})(LevelState = exports.LevelState || (exports.LevelState = {}));
var BaseEnvironment = (function () {
    function BaseEnvironment(aOuterEnv, aName) {
        if (aName === void 0) { aName = "anonymous"; }
        this.log = new log_1.Log({
            enabledCondition: { type: "C", C: "LOG_ENVIRONMENT" },
            name: "Environment",
        });
        this.isMainEnvironment = false;
        this.uid = Math.random().toString(36).substr(2, 5);
        this.envState = State.NOT_STARTED;
        this.innerEnvs = new Set();
        this.levels = {
            shutdown: BaseEnvironment.generateLevelObjects(),
            startup: BaseEnvironment.generateLevelObjects(),
        };
        this.envState = State.NOT_STARTED;
        this.name = aName;
        this.outerEnv = aOuterEnv instanceof BaseEnvironment ? aOuterEnv : null;
        this.innerEnvs = new Set();
        this.levels = {
            shutdown: BaseEnvironment.generateLevelObjects(),
            startup: BaseEnvironment.generateLevelObjects(),
        };
        this.log.log("created new Environment \"" + this.name + "\"");
    }
    BaseEnvironment.getBootstrapMetadata = function (startupOrShutdown) {
        return BaseEnvironment.BOOTSTRAP[startupOrShutdown];
    };
    BaseEnvironment.generateLevelObjects = function () {
        var obj = {};
        for (var levelName in Level) {
            var level = Level[levelName];
            obj[level] = {
                functions: [],
                levelState: LevelState.NOT_ENTERED,
            };
        }
        return obj;
    };
    BaseEnvironment.prototype.isShuttingDownOrShutDown = function () {
        return this.envState >= State.SHUTTING_DOWN;
    };
    BaseEnvironment.prototype.register = function () {
        if (this.outerEnv) {
            this.outerEnv.registerInnerEnvironment(this);
        }
    };
    BaseEnvironment.prototype.unregister = function () {
        if (this.outerEnv) {
            this.outerEnv.unregisterInnerEnvironment(this);
        }
    };
    BaseEnvironment.prototype.registerInnerEnvironment = function (aEnv) {
        if (this.envState === State.NOT_STARTED) {
            this.log.warn("registerInnerEnvironment() has been called but " +
                "the outer environment hasn't started up yet. " +
                "Starting up now.");
            this.startup();
        }
        this.log.log("registering inner environment");
        this.innerEnvs.add(aEnv);
    };
    BaseEnvironment.prototype.unregisterInnerEnvironment = function (aEnv) {
        if (this.innerEnvs.has(aEnv) === false) {
            console.error("[RequestPolicy] it seems like an inner Environment did not register.");
        }
        else {
            this.innerEnvs.delete(aEnv);
        }
    };
    BaseEnvironment.prototype.addStartupFunction = function (aLevel, f) {
        if (this.isShuttingDownOrShutDown()) {
            return;
        }
        if (this.levels.startup[aLevel].levelState >= LevelState.PROCESSING) {
            f();
        }
        else {
            this.levels.startup[aLevel].functions.push(f);
        }
    };
    BaseEnvironment.prototype.addShutdownFunction = function (aLevel, f) {
        var _this = this;
        if (this.levels.shutdown[aLevel].levelState >= LevelState.PROCESSING) {
            f();
            this.log.cbLog(function () { return [
                "calling shutdown function immediately: " +
                    ("\"" + (f.name || "anonymous") + "\" (" + _this.name + ")"),
            ]; });
        }
        else {
            this.levels.shutdown[aLevel].functions.push(f);
        }
    };
    BaseEnvironment.prototype.startup = function (aBootstrapArgs, aUntilLevel) {
        this.log.log("starting up: " + this.name);
        this._bootstrap("startup", aBootstrapArgs, aUntilLevel);
    };
    BaseEnvironment.prototype.shutdown = function (aBootstrapArgs, aUntilLevel) {
        this.log.log("shutting down: " + this.name);
        this._bootstrap("shutdown", aBootstrapArgs, aUntilLevel);
    };
    BaseEnvironment.prototype._iterateLevels = function (aStartupOrShutdown, aFn, aUntilLevel) {
        if (aUntilLevel === void 0) { aUntilLevel = null; }
        var sequence = BaseEnvironment.
            BOOTSTRAP[aStartupOrShutdown].levelSequence;
        for (var i = 0, len = sequence.length; i < len; ++i) {
            var level = sequence[i];
            aFn(level);
            if (level === aUntilLevel) {
                break;
            }
        }
    };
    BaseEnvironment.prototype._callFunctions = function (aFunctions, aBootstrapArgs) {
        if (aBootstrapArgs === void 0) { aBootstrapArgs = []; }
        while (aFunctions.length > 0) {
            var f = aFunctions.pop();
            try {
                f.apply(void 0, __spread(aBootstrapArgs));
            }
            catch (e) {
                console.error("[RequestPolicy] Error in bootstrap function! Details:");
                console.dir(e);
            }
            this.log.log("function called! (" + aFunctions.length +
                " functions left)");
        }
    };
    BaseEnvironment.prototype._processLevel = function (aStartupOrShutdown, aLevel, aBootstrapArgs) {
        var levelObj = this.levels[aStartupOrShutdown][aLevel];
        if (levelObj.levelState === LevelState.NOT_ENTERED) {
            levelObj.levelState = LevelState.PROCESSING;
            this.log.log("processing level " + aLevel + " of startup (" +
                this.uid + ")");
            if (aStartupOrShutdown === "shutdown") {
                this.innerEnvs.forEach(function (innerEnv) {
                    innerEnv.shutdown(aBootstrapArgs, aLevel);
                });
            }
            this._callFunctions(levelObj.functions, aBootstrapArgs);
            levelObj.levelState = LevelState.FINISHED_PROCESSING;
            this.log.log("processing level " + aLevel + " of startup " +
                "(" + this.uid + ") finished");
        }
    };
    BaseEnvironment.prototype._processLevels = function (aStartupOrShutdown, aBootstrapArgs, aUntilLevel) {
        var _this = this;
        this._iterateLevels(aStartupOrShutdown, function (level) {
            _this._processLevel(aStartupOrShutdown, level, aBootstrapArgs);
        }, aUntilLevel);
    };
    Object.defineProperty(BaseEnvironment.prototype, "envInfo", {
        get: function () {
            return "'" + this.name + "' (" + this.uid + ")";
        },
        enumerable: true,
        configurable: true
    });
    BaseEnvironment.prototype._logStartupOrShutdown = function (aStartupOrShutdown) {
        var _this = this;
        this.log.cbLog(function () { return [
            aStartupOrShutdown + ": " + _this.envInfo + ". " +
                (_this.outerEnv ?
                    " OuterEnv is " + _this.outerEnv.envInfo + "." :
                    " No OuterEnv."),
        ]; });
    };
    BaseEnvironment.prototype._bootstrap = function (aStartupOrShutdown, aBootstrapArgs, aUntilLevel) {
        if (aBootstrapArgs === void 0) { aBootstrapArgs = []; }
        if (aUntilLevel === void 0) { aUntilLevel = BaseEnvironment.BOOTSTRAP[aStartupOrShutdown].lastLevel; }
        var _a = BaseEnvironment.getBootstrapMetadata(aStartupOrShutdown), lastLevel = _a.lastLevel, envStates = _a.envStates, functions = _a.functions;
        if (this.envState === envStates.beforeProcessing) {
            this._logStartupOrShutdown(aStartupOrShutdown);
            functions.beforeProcessing.call(this);
            this.envState = envStates.duringProcessing;
        }
        if (this.envState === envStates.duringProcessing) {
            this._processLevels(aStartupOrShutdown, aBootstrapArgs, aUntilLevel);
            if (aUntilLevel === lastLevel) {
                this.envState = envStates.afterProcessing;
                functions.afterProcessing.call(this);
            }
        }
    };
    BaseEnvironment.BOOTSTRAP = {
        shutdown: {
            envStates: {
                afterProcessing: State.SHUT_DOWN,
                beforeProcessing: State.STARTUP_DONE,
                duringProcessing: State.SHUTTING_DOWN,
            },
            functions: {
                beforeProcessing: function () { },
                afterProcessing: function () {
                    var self = this;
                    self.innerEnvs.length = 0;
                    self.unregister();
                },
            },
            lastLevel: Level.ESSENTIAL,
            levelSequence: [
                Level.UI,
                Level.INTERFACE,
                Level.BACKEND,
                Level.ESSENTIAL,
            ],
        },
        startup: {
            envStates: {
                afterProcessing: State.STARTUP_DONE,
                beforeProcessing: State.NOT_STARTED,
                duringProcessing: State.STARTING_UP,
            },
            functions: {
                beforeProcessing: function () {
                    var self = this;
                    self.register();
                },
                afterProcessing: function () { },
            },
            lastLevel: Level.UI,
            levelSequence: [
                Level.ESSENTIAL,
                Level.BACKEND,
                Level.INTERFACE,
                Level.UI,
            ],
        },
    };
    return BaseEnvironment;
}());
var Environment = (function (_super) {
    __extends(Environment, _super);
    function Environment() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(Environment.prototype, "elManager", {
        get: function () {
            if (!this.lazyElManager) {
                this.lazyElManager = new manager_for_event_listeners_1.ManagerForEventListeners(this);
            }
            return this.lazyElManager;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Environment.prototype, "obMan", {
        get: function () {
            if (!this.lazyObMan) {
                this.lazyObMan = new observer_manager_1.ObserverManager(this);
            }
            return this.lazyObMan;
        },
        enumerable: true,
        configurable: true
    });
    Environment.prototype.shutdownOnUnload = function (aEventTarget) {
        var _this = this;
        this.elManager.addListener(aEventTarget, "unload", function () {
            _this.log.log("an EventTarget's `unload` function " +
                "has been called. Going to shut down Environment " +
                ("\"" + _this.name));
            _this.shutdown();
        });
    };
    return Environment;
}(BaseEnvironment));
exports.Environment = Environment;
exports.MainEnvironment = new Environment(null, "Main Environment");
exports.MainEnvironment.isMainEnvironment =
    typeof browser.extension.getBackgroundPage === "function";