"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var environment_1 = require("lib/environment");
var constants_1 = require("data/constants");
var LOG_EVENT_LISTENERS = constants_1.C.LOG_EVENT_LISTENERS;
function addEvLis(listener) {
    listener.target.addEventListener(listener.eventType, listener.callback, listener.useCapture);
    listener.listening = true;
}
var ManagerForEventListeners = (function () {
    function ManagerForEventListeners(aEnv) {
        var _this = this;
        this.listeners = [];
        this.addNewListenersImmediately = false;
        this.environment = aEnv;
        if (aEnv) {
            this.log = aEnv.log;
            this.environment.addStartupFunction(environment_1.Level.INTERFACE, function () {
                if (LOG_EVENT_LISTENERS) {
                    _this.log.log("" + "From now on new event listeners will be " +
                        "added immediately. Environment: \"" + _this.environment.name + "\"");
                }
                _this.addNewListenersImmediately = true;
                _this.addAllListeners();
            });
            this.environment.addShutdownFunction(environment_1.Level.INTERFACE, function () {
                _this.removeAllListeners();
            });
        }
        else {
            console.error("[RequestPolicy] No Environment was specified for a new " +
                "ManagerForEventListeners! " +
                "This means that the listeners won't be removed!");
        }
    }
    ManagerForEventListeners.prototype.addListener = function (aEventTarget, aEventType, aCallback, aUseCapture) {
        var self = this;
        if (typeof aCallback !== "function") {
            console.error("[RequestPolicy] The callback for an event listener" +
                ("must be a function! Event type was \"" + aEventType + "\""));
            return;
        }
        var listener = {
            target: aEventTarget,
            eventType: aEventType,
            callback: aCallback,
            useCapture: !!aUseCapture,
            listening: false,
        };
        if (self.addNewListenersImmediately) {
            if (LOG_EVENT_LISTENERS) {
                this.log.log("Immediately adding event listener for \"" + listener.eventType + "\". Environment: \"" + self.environment.name + "\"");
            }
            addEvLis(listener);
        }
        self.listeners.push(listener);
    };
    ManagerForEventListeners.prototype.addAllListeners = function () {
        var e_1, _a;
        var self = this;
        try {
            for (var _b = __values(self.listeners), _c = _b.next(); !_c.done; _c = _b.next()) {
                var listener = _c.value;
                if (listener.listening === false) {
                    if (LOG_EVENT_LISTENERS) {
                        this.log.log("Lazily adding event listener for \"" + listener.eventType + "\". Environment: \"" + self.environment.name + "\"");
                    }
                    addEvLis(listener);
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    ManagerForEventListeners.prototype.removeAllListeners = function () {
        var self = this;
        while (self.listeners.length > 0) {
            var listener = self.listeners.pop();
            if (LOG_EVENT_LISTENERS) {
                this.log.log("Removing event listener for \"" + listener.eventType + "\". Environment: \"" + self.environment.name + "\"");
            }
            listener.target.removeEventListener(listener.eventType, listener.callback, listener.useCapture);
        }
    };
    return ManagerForEventListeners;
}());
exports.ManagerForEventListeners = ManagerForEventListeners;