"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var environment_1 = require("lib/environment");
var constants_1 = require("data/constants");
var LOG_MESSAGE_LISTENERS = constants_1.C.LOG_MESSAGE_LISTENERS;
function ManagerForMessageListeners(aEnv, aMM) {
    var self = this;
    self.listeners = [];
    self.addNewListenersImmediately = false;
    self.environment = aEnv;
    if (aEnv) {
        self.log = aEnv.log;
        self.environment.addStartupFunction(environment_1.Level.INTERFACE, function () {
            if (LOG_MESSAGE_LISTENERS) {
                self.log.log("" + "From now on new message listeners will be added " +
                    "immediately. Environment: \"" + self.environment.name + "\"");
            }
            self.addNewListenersImmediately = true;
            self.addAllListeners();
        });
        self.environment.addShutdownFunction(environment_1.Level.INTERFACE, function () {
            self.removeAllListeners();
        });
    }
    else {
        console.error("[RequestPolicy] No Environment was specified for a new " +
            "ManagerForMessageListeners! This means that the listeners " +
            "won't be unregistered!");
    }
    self.mm = aMM;
    if (!self.mm) {
        self.log.warn("No Message Manager was specified " +
            "for a new ManagerForMessageListeners!");
    }
}
exports.ManagerForMessageListeners = ManagerForMessageListeners;
ManagerForMessageListeners.prototype.addListener = function (aMessageName, aCallback, aCallbackOnShutdown, aAddImmediately) {
    var self = this;
    if (typeof aCallback !== "function") {
        console.error("[RequestPolicy] " + "The callback for a message listener" +
            "must be a function! The message name was \"" + aMessageName + "\"");
        return;
    }
    if (aMessageName.indexOf(constants_1.C.MM_PREFIX) === 0) {
        self.log.warn("The message name that has been passed to " +
            "`addListener()` contains the MM Prefix. " +
            "Extracting the message name.");
        aMessageName = aMessageName.substr(constants_1.C.MM_PREFIX.length);
    }
    var listener = {
        messageName: aMessageName,
        messageID: constants_1.C.MM_PREFIX + aMessageName,
        callback: function (aMessage) {
            if (self.environment.envState === environment_1.State.SHUTTING_DOWN) {
                console.log("[RequestPolicy] [RequestPolicy] Listener for " + aMessageName + " has been called, but RP is already shutting down.");
                if (typeof aCallbackOnShutdown === "function") {
                    return aCallbackOnShutdown(aMessage);
                }
                return;
            }
            return aCallback(aMessage);
        },
        listening: false,
    };
    if (aAddImmediately === true || self.addNewListenersImmediately) {
        if (LOG_MESSAGE_LISTENERS) {
            self.log.log("Immediately adding message listener for \"" + listener.messageName + "\". Environment: \"" + self.environment.name + "\"");
        }
        self.mm.addMessageListener(listener.messageID, listener.callback);
        listener.listening = true;
    }
    self.listeners.push(listener);
};
ManagerForMessageListeners.prototype.addAllListeners = function () {
    var e_1, _a;
    var self = this;
    try {
        for (var _b = __values(self.listeners), _c = _b.next(); !_c.done; _c = _b.next()) {
            var listener = _c.value;
            if (listener.listening === false) {
                if (LOG_MESSAGE_LISTENERS) {
                    self.log.log("Lazily adding message listener for \"" + listener.messageName + "\". Environment: \"" + self.environment.name + "\"");
                }
                self.mm.addMessageListener(listener.messageID, listener.callback);
                listener.listening = true;
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
        }
        finally { if (e_1) throw e_1.error; }
    }
};
ManagerForMessageListeners.prototype.removeAllListeners = function () {
    var self = this;
    while (self.listeners.length > 0) {
        var listener = self.listeners.pop();
        if (LOG_MESSAGE_LISTENERS) {
            self.log.log("Removing message listener for \"" + listener.messageName + "\".");
        }
        self.mm.removeMessageListener(listener.messageID, listener.callback);
    }
};