"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var xpcom_observer_1 = require("lib/classes/xpcom-observer");
var environment_1 = require("lib/environment");
var ObserverManager = (function () {
    function ObserverManager(aEnv) {
        var _this = this;
        this.environment = aEnv;
        if (aEnv) {
            this.log = aEnv.log;
            this.environment.addShutdownFunction(environment_1.Level.INTERFACE, function () {
                _this.unregisterAllObservers();
            });
        }
        else {
            console.error("[RequestPolicy] No Environment was specified for " +
                "a new ObserverManager! This means that the observers " +
                "won't be unregistered!");
        }
        this.observers = [];
    }
    ObserverManager.prototype.observeSingleTopic = function (aTopic, aCallback) {
        var self = this;
        self.observers.push(new xpcom_observer_1.XPCOMObserver(aTopic, aCallback));
    };
    ObserverManager.prototype.observe = function (aTopics, aCallback) {
        var self = this;
        aTopics.forEach(function (topic) {
            self.observeSingleTopic(topic, aCallback);
        });
    };
    ObserverManager.prototype.unregisterAllObservers = function () {
        var self = this;
        while (self.observers.length > 0) {
            var observer = self.observers.pop();
            observer.unregister();
        }
    };
    return ObserverManager;
}());
exports.ObserverManager = ObserverManager;