"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var request_1 = require("lib/request");
var request_result_1 = require("lib/request-result");
var environment_1 = require("lib/environment");
var Utils = require("lib/utils/misc-utils");
var compatibility_rules_1 = require("models/compatibility-rules");
var metadata_1 = require("models/metadata");
var requests_1 = require("models/requests");
var log_1 = require("app/log");
var content_policy_1 = require("main/content-policy");
var try_catch_utils_1 = require("lib/utils/try-catch-utils");
var app_background_1 = require("app/app.background");
var uriService = app_background_1.rp.services.uri;
var cachedSettings = app_background_1.rp.storage.cachedSettings;
var logRequests = log_1.log.extend({
    enabledCondition: { type: "C", C: "LOG_REQUESTS" },
    level: "all",
    name: "Requests",
});
var CP_OK = Ci.nsIContentPolicy.ACCEPT;
var CP_REJECT = Ci.nsIContentPolicy.REJECT_SERVER;
var CP_MAPPEDDESTINATION = 0x178c40bf;
var HTTPS_EVERYWHERE_REWRITE_TOPIC = "https-everywhere-uri-rewrite";
var lastShouldLoadCheckTimeout = 200;
var lastShouldLoadCheck = {
    "origin": null,
    "destination": null,
    "time": 0,
    "result": null,
};
var historyRequests = {};
function reject(reason, request) {
    logRequests.log("** BLOCKED ** reason: " + reason + ". " + request.detailsToString());
    if (cachedSettings.alias.isBlockingDisabled()) {
        return CP_OK;
    }
    if (request.aContext) {
        request.aContext.rpcontinuedBlocked = true;
    }
    cacheShouldLoadResult(CP_REJECT, request.originURI, request.destURI);
    requests_1.Requests.notifyNewRequest({
        isAllowed: false,
        isInsert: false,
        requestResult: request.requestResult,
        originUri: request.originURI,
        destUri: request.destURI,
    });
    if (Ci.nsIContentPolicy.TYPE_DOCUMENT === request.aContentType) {
        var browser = request.getBrowser();
        var window = request.getChromeWindow();
        if (!browser || !window || typeof window.rpcontinued === "undefined") {
            log_1.log.warn("The user could not be notified " +
                "about the blocked top-level document request!");
        }
        else {
            window.rpcontinued.overlay.observeBlockedTopLevelDocRequest(browser, request.originURI, request.destURIWithRef);
        }
    }
    return CP_REJECT;
}
function accept(reason, request, unforbidable) {
    logRequests.log("** ALLOWED ** reason: " + reason + ". " + request.detailsToString());
    cacheShouldLoadResult(CP_OK, request.originURI, request.destURI);
    requests_1.Requests.notifyNewRequest({
        isAllowed: true,
        isInsert: false,
        requestResult: request.requestResult,
        originUri: request.originURI,
        destUri: request.destURI,
        unforbidable: unforbidable,
    });
    return CP_OK;
}
function cacheShouldLoadResult(result, originUri, destUri) {
    var date = new Date();
    lastShouldLoadCheck.time = date.getTime();
    lastShouldLoadCheck.destination = destUri;
    lastShouldLoadCheck.origin = originUri;
    lastShouldLoadCheck.result = result;
}
function isDuplicateRequest(request) {
    if (lastShouldLoadCheck.origin === request.originURI &&
        lastShouldLoadCheck.destination === request.destURI) {
        var date = new Date();
        if (date.getTime() - lastShouldLoadCheck.time <
            lastShouldLoadCheckTimeout) {
            logRequests.log("Using cached shouldLoad() result of " + lastShouldLoadCheck.result + " for request to <" + request.destURI + "> from <" + request.originURI + ">.");
            return true;
        }
        else {
            logRequests.log("shouldLoad() cache expired for result of " + lastShouldLoadCheck.result + " for request to <" + request.destURI + "> from <" + request.originURI + ">.");
        }
    }
    return false;
}
function getDomNodeFromRequestContext(context) {
    var result = try_catch_utils_1.queryInterface(context, Ci.nsIDOMNode);
    if (!result.error) {
        return result.value;
    }
    var e = result.error;
    if (e.result !== Cr.NS_ERROR_NO_INTERFACE) {
        throw e;
    }
}
function process(request) {
    var e_1, _a, e_2, _b, e_3, _c, e_4, _d, e_5, _e;
    try {
        if (request.isInternal()) {
            logRequests.log("" + "Allowing a request that seems to be internal. " +
                "Origin: " + request.originURI + ", Dest: " + request.destURI);
            return CP_OK;
        }
        var originURI = request.originURI;
        var destURI = request.destURI;
        if (request.aRequestOrigin.scheme === "moz-nullprincipal") {
            if (request.aRequestPrincipal) {
                logRequests.log("" + "Allowing request that appears to be a URL entered in the " +
                    "location bar or some other good explanation: " + destURI);
                requests_1.Requests._removeSavedRequestsByOriginURI(destURI);
                return CP_OK;
            }
        }
        if (request.aRequestOrigin.scheme === "view-source") {
            var newOriginURI = originURI.split(":").slice(1).join(":");
            logRequests.log("Considering view-source origin <" + originURI + "> " +
                ("to be origin <" + newOriginURI + ">"));
            originURI = newOriginURI;
            request.setOriginURI(originURI);
        }
        if (request.aContentLocation.scheme === "view-source") {
            var newDestURI = destURI.split(":").slice(1).join(":");
            if (newDestURI.indexOf("data:text/html") === 0) {
                logRequests.log("Allowing \"data:text/html\" view-source destination" +
                    " (Selection Source)");
                return CP_OK;
            }
            else {
                logRequests.log("Considering view-source destination <" + destURI + "> " +
                    ("to be destination <" + newDestURI + ">"));
                destURI = newDestURI;
                request.setDestURI(destURI);
            }
        }
        if (originURI === "about:blank" && request.aContext) {
            var domNode = getDomNodeFromRequestContext(request.aContext);
            if (domNode && domNode.nodeType === Ci.nsIDOMNode.DOCUMENT_NODE) {
                var newOriginURI = void 0;
                if (request.aContext.documentURI &&
                    request.aContext.documentURI !== "about:blank") {
                    newOriginURI = request.aContext.documentURI;
                }
                else if (request.aContext.ownerDocument &&
                    request.aContext.ownerDocument.documentURI &&
                    request.aContext.ownerDocument.documentURI !== "about:blank") {
                    newOriginURI = request.aContext.ownerDocument.documentURI;
                }
                if (newOriginURI) {
                    newOriginURI = uriService.stripFragment(newOriginURI);
                    log_1.log.log("Considering origin <" + originURI + "> to be origin <" + newOriginURI + ">");
                    originURI = newOriginURI;
                    request.setOriginURI(originURI);
                }
            }
        }
        if (isDuplicateRequest(request)) {
            return lastShouldLoadCheck.result;
        }
        if (originURI === destURI) {
            logRequests.log("" + "Allowing (but not recording) request " +
                "where origin is the same as the destination: " + originURI);
            return CP_OK;
        }
        if (request.aContext) {
            var domNode = getDomNodeFromRequestContext(request.aContext);
            if (domNode && domNode.nodeName === "LINK" &&
                (domNode.rel === "icon" || domNode.rel === "shortcut icon")) {
                metadata_1.FaviconRequests[destURI] = true;
            }
        }
        if (metadata_1.ClickedLinks[originURI] &&
            metadata_1.ClickedLinks[originURI][destURI]) {
            request.requestResult = new request_result_1.RequestResult(true, request_result_1.RequestReason.LinkClick);
            return accept("User-initiated request by link click", request);
        }
        else if (metadata_1.SubmittedForms[originURI] &&
            metadata_1.SubmittedForms[originURI][destURI.split("?")[0]]) {
            request.requestResult = new request_result_1.RequestResult(true, request_result_1.RequestReason.FormSubmission);
            return accept("User-initiated request by form submission", request);
        }
        else if (historyRequests[destURI]) {
            delete historyRequests[destURI];
            request.requestResult = new request_result_1.RequestResult(true, request_result_1.RequestReason.HistoryRequest);
            return accept("History request", request, true);
        }
        else if (metadata_1.UserAllowedRedirects[originURI] &&
            metadata_1.UserAllowedRedirects[originURI][destURI]) {
            request.requestResult = new request_result_1.RequestResult(true, request_result_1.RequestReason.UserAllowedRedirect);
            return accept("User-allowed redirect", request, true);
        }
        var originHost = uriService.getHostByUriObj(request.aRequestOrigin);
        if (request.aRequestOrigin.scheme === "chrome") {
            if (originHost === "browser") {
                request.requestResult = new request_result_1.RequestResult(true, request_result_1.RequestReason.UserAction);
                return accept("User action (e.g. address entered in address bar) " +
                    "or other good explanation (e.g. new window/tab opened)", request);
            }
            else {
                request.requestResult = new request_result_1.RequestResult(true, request_result_1.RequestReason.UserAction);
                return accept("User action (e.g. address entered in address bar) " +
                    "or other good explanation (e.g. new window/tab opened)", request);
            }
        }
        if (request.aContext) {
            var domNode = getDomNodeFromRequestContext(request.aContext);
            if (domNode && domNode.nodeName === "xul:browser" &&
                domNode.currentURI && domNode.currentURI.spec === "about:blank") {
                request.requestResult = new request_result_1.RequestResult(true, request_result_1.RequestReason.NewWindow);
                return accept("New window (should probably only be an allowed " +
                    "popup's initial request)", request, true);
            }
        }
        var originIdent = uriService.getIdentifier(originURI);
        var destIdent = uriService.getIdentifier(destURI);
        if (originIdent === destIdent &&
            originIdent !== null && destIdent !== null) {
            request.requestResult = new request_result_1.RequestResult(true, request_result_1.RequestReason.IdenticalIdentifier);
            return accept("" + "Allowing request where origin protocol, host, and port are the" +
                " same as the destination: " + originIdent, request);
        }
        request.requestResult = app_background_1.rp.policy.checkRequestAgainstUserRules(request.aRequestOrigin, request.aContentLocation);
        try {
            for (var _f = __values(request.requestResult.matchedDenyRules), _g = _f.next(); !_g.done; _g = _f.next()) {
                var matchedDenyRule = _g.value;
                log_1.log.log("Matched deny rules");
                log_1.log.dir(matchedDenyRule);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_g && !_g.done && (_a = _f.return)) _a.call(_f);
            }
            finally { if (e_1) throw e_1.error; }
        }
        try {
            for (var _h = __values(request.requestResult.matchedAllowRules), _j = _h.next(); !_j.done; _j = _h.next()) {
                var matchedAllowRule = _j.value;
                log_1.log.log("Matched allow rules");
                log_1.log.dir(matchedAllowRule);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_j && !_j.done && (_b = _h.return)) _b.call(_h);
            }
            finally { if (e_2) throw e_2.error; }
        }
        if (request.requestResult.allowRulesExist() &&
            request.requestResult.denyRulesExist()) {
            var _k = request.requestResult.resolveConflict(), conflictCanBeResolved = _k.conflictCanBeResolved, shouldAllow = _k.shouldAllow;
            if (conflictCanBeResolved) {
                request.requestResult.resultReason = request_result_1.RequestReason.UserPolicy;
                request.requestResult.isAllowed = shouldAllow;
                if (shouldAllow) {
                    return accept("Allowed by user policy", request);
                }
                else {
                    return reject("Blocked by user policy", request);
                }
            }
            request.requestResult.resultReason =
                request_result_1.RequestReason.DefaultPolicyInconsistentRules;
            if (cachedSettings.alias.isDefaultAllow()) {
                request.requestResult.isAllowed = true;
                return accept("User policy indicates both allow and block. " +
                    "Using default allow policy", request);
            }
            else {
                request.requestResult.isAllowed = false;
                return reject("User policy indicates both allow and block. " +
                    "Using default block policy", request);
            }
        }
        if (request.requestResult.allowRulesExist()) {
            request.requestResult.resultReason = request_result_1.RequestReason.UserPolicy;
            request.requestResult.isAllowed = true;
            return accept("Allowed by user policy", request);
        }
        if (request.requestResult.denyRulesExist()) {
            request.requestResult.resultReason = request_result_1.RequestReason.UserPolicy;
            request.requestResult.isAllowed = false;
            return reject("Blocked by user policy", request);
        }
        request.requestResult = app_background_1.rp.policy.
            checkRequestAgainstSubscriptionRules(request.aRequestOrigin, request.aContentLocation);
        try {
            for (var _l = __values(request.requestResult.matchedDenyRules), _m = _l.next(); !_m.done; _m = _l.next()) {
                var matchedDenyRule = _m.value;
                log_1.log.log("Matched deny rules");
                log_1.log.dir(matchedDenyRule);
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_m && !_m.done && (_c = _l.return)) _c.call(_l);
            }
            finally { if (e_3) throw e_3.error; }
        }
        try {
            for (var _o = __values(request.requestResult.matchedAllowRules), _p = _o.next(); !_p.done; _p = _o.next()) {
                var matchedAllowRule = _p.value;
                log_1.log.log("Matched allow rules");
                log_1.log.dir(matchedAllowRule);
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (_p && !_p.done && (_d = _o.return)) _d.call(_o);
            }
            finally { if (e_4) throw e_4.error; }
        }
        if (request.requestResult.allowRulesExist() &&
            request.requestResult.denyRulesExist()) {
            var _q = request.requestResult.resolveConflict(), conflictCanBeResolved = _q.conflictCanBeResolved, shouldAllow = _q.shouldAllow;
            if (conflictCanBeResolved) {
                request.requestResult.resultReason =
                    request_result_1.RequestReason.SubscriptionPolicy;
                request.requestResult.isAllowed = shouldAllow;
                if (shouldAllow) {
                    return accept("Allowed by subscription policy", request);
                }
                else {
                    return reject("Blocked by subscription policy", request);
                }
            }
            request.requestResult.resultReason =
                request_result_1.RequestReason.DefaultPolicyInconsistentRules;
            if (cachedSettings.alias.isDefaultAllow()) {
                request.requestResult.isAllowed = true;
                return accept("Subscription rules indicate both allow and block. " +
                    "Using default allow policy", request);
            }
            else {
                request.requestResult.isAllowed = false;
                return reject("Subscription rules indicate both allow and block. " +
                    "Using default block policy", request);
            }
        }
        if (request.requestResult.denyRulesExist()) {
            request.requestResult.resultReason =
                request_result_1.RequestReason.SubscriptionPolicy;
            request.requestResult.isAllowed = false;
            return reject("Blocked by subscription policy", request);
        }
        if (request.requestResult.allowRulesExist()) {
            request.requestResult.resultReason =
                request_result_1.RequestReason.SubscriptionPolicy;
            request.requestResult.isAllowed = true;
            return accept("Allowed by subscription policy", request);
        }
        try {
            for (var CompatibilityRules_1 = __values(compatibility_rules_1.CompatibilityRules), CompatibilityRules_1_1 = CompatibilityRules_1.next(); !CompatibilityRules_1_1.done; CompatibilityRules_1_1 = CompatibilityRules_1.next()) {
                var rule = CompatibilityRules_1_1.value;
                var allowOrigin = rule.origin ? originURI.startsWith(rule.origin) :
                    true;
                var allowDest = rule.dest ? destURI.startsWith(rule.dest) : true;
                if (allowOrigin && allowDest) {
                    request.requestResult = new request_result_1.RequestResult(true, request_result_1.RequestReason.Compatibility);
                    return accept("Extension/application compatibility rule matched [" + rule.info + "]", request, true);
                }
            }
        }
        catch (e_5_1) { e_5 = { error: e_5_1 }; }
        finally {
            try {
                if (CompatibilityRules_1_1 && !CompatibilityRules_1_1.done && (_e = CompatibilityRules_1.return)) _e.call(CompatibilityRules_1);
            }
            finally { if (e_5) throw e_5.error; }
        }
        if (request.aContext) {
            var info = compatibility_rules_1.CompatibilityRules.checkBaseUriWhitelist(request.aContext.baseURI);
            if (info.isWhitelisted) {
                request.requestResult = new request_result_1.RequestResult(true, request_result_1.RequestReason.Compatibility);
                return accept("Extension/application compatibility rule matched [" + info.addonName + "]", request, true);
            }
        }
        if (request.aExtra !== CP_MAPPEDDESTINATION &&
            metadata_1.MappedDestinations[destURI]) {
            for (var mappedDest in metadata_1.MappedDestinations[destURI]) {
                var mappedDestUriObj = metadata_1.MappedDestinations[destURI][mappedDest];
                logRequests.log("Checking mapped destination: " + mappedDest);
                var mappedResult = content_policy_1.default.shouldLoad(request.aContentType, mappedDestUriObj, request.aRequestOrigin, request.aContext, request.aMimeTypeGuess, CP_MAPPEDDESTINATION);
                if (mappedResult === CP_OK) {
                    return CP_OK;
                }
            }
        }
        request.requestResult = request.checkByDefaultPolicy();
        if (request.requestResult.isAllowed) {
            return accept("Allowed by default policy", request);
        }
        else {
            return request.aExtra === CP_MAPPEDDESTINATION ?
                CP_REJECT :
                reject("Denied by default policy", request);
        }
    }
    catch (e) {
        console.error("[RequestPolicy] Fatal Error:");
        console.dir(e);
        if (cachedSettings.alias.isBlockingDisabled()) {
            log_1.log.warn("Allowing request due to internal error.");
            return CP_OK;
        }
        log_1.log.warn("Rejecting request due to internal error.");
        return CP_REJECT;
    }
}
exports.process = process;
var examineHttpRequest = function (aSubject) {
    var httpChannel = aSubject.QueryInterface(Ci.nsIHttpChannel);
    var result = try_catch_utils_1.getRequestHeaderFromHttpChannel(httpChannel, "X-moz");
    if (result.value === "prefetch") {
        log_1.log.warn("Discovered prefetch request being sent to: " + httpChannel.name);
    }
};
environment_1.MainEnvironment.obMan.observe(["http-on-modify-request"], examineHttpRequest);
function registerHistoryRequest(destinationUrl) {
    destinationUrl = uriService.ensureUriHasPath(uriService.stripFragment(destinationUrl));
    historyRequests[destinationUrl] = true;
    log_1.log.info("History item requested: <" + destinationUrl + ">.");
}
exports.registerHistoryRequest = registerHistoryRequest;
function registerFormSubmitted(originUrl, destinationUrl) {
    originUrl = uriService.ensureUriHasPath(uriService.stripFragment(originUrl));
    destinationUrl = uriService.ensureUriHasPath(uriService.stripFragment(destinationUrl));
    log_1.log.info("Form submitted from <" + originUrl + "> to <" + destinationUrl + ">.");
    destinationUrl = destinationUrl.split("?")[0];
    if (metadata_1.SubmittedForms[originUrl] === undefined) {
        metadata_1.SubmittedForms[originUrl] = {};
    }
    if (metadata_1.SubmittedForms[originUrl][destinationUrl] === undefined) {
        metadata_1.SubmittedForms[originUrl][destinationUrl] = true;
    }
    if (metadata_1.SubmittedFormsReverse[destinationUrl] === undefined) {
        metadata_1.SubmittedFormsReverse[destinationUrl] = {};
    }
    if (metadata_1.SubmittedFormsReverse[destinationUrl][originUrl] === undefined) {
        metadata_1.SubmittedFormsReverse[destinationUrl][originUrl] = true;
    }
}
exports.registerFormSubmitted = registerFormSubmitted;
function registerLinkClicked(originUrl, destinationUrl) {
    originUrl = uriService.ensureUriHasPath(uriService.stripFragment(originUrl));
    destinationUrl = uriService.ensureUriHasPath(uriService.stripFragment(destinationUrl));
    log_1.log.info("Link clicked from <" + originUrl + "> to <" + destinationUrl + ">.");
    if (metadata_1.ClickedLinks[originUrl] === undefined) {
        metadata_1.ClickedLinks[originUrl] = {};
    }
    if (metadata_1.ClickedLinks[originUrl][destinationUrl] === undefined) {
        metadata_1.ClickedLinks[originUrl][destinationUrl] = true;
    }
    if (metadata_1.ClickedLinksReverse[destinationUrl] === undefined) {
        metadata_1.ClickedLinksReverse[destinationUrl] = {};
    }
    if (metadata_1.ClickedLinksReverse[destinationUrl][originUrl] === undefined) {
        metadata_1.ClickedLinksReverse[destinationUrl][originUrl] = true;
    }
}
exports.registerLinkClicked = registerLinkClicked;
function registerAllowedRedirect(originUrl, destinationUrl) {
    originUrl = uriService.ensureUriHasPath(uriService.stripFragment(originUrl));
    destinationUrl = uriService.ensureUriHasPath(uriService.stripFragment(destinationUrl));
    log_1.log.info("User-allowed redirect from <" + originUrl + "> to <" + destinationUrl + ">.");
    if (metadata_1.UserAllowedRedirects[originUrl] === undefined) {
        metadata_1.UserAllowedRedirects[originUrl] = {};
    }
    if (metadata_1.UserAllowedRedirects[originUrl][destinationUrl] === undefined) {
        metadata_1.UserAllowedRedirects[originUrl][destinationUrl] = true;
    }
}
exports.registerAllowedRedirect = registerAllowedRedirect;
environment_1.MainEnvironment.obMan.observe([HTTPS_EVERYWHERE_REWRITE_TOPIC], function (subject, topic, data) {
    handleHttpsEverywhereUriRewrite(subject, data);
});
function mapDestinations(origDestUri, newDestUri) {
    origDestUri = uriService.stripFragment(origDestUri);
    newDestUri = uriService.stripFragment(newDestUri);
    log_1.log.info("Mapping destination <" + origDestUri + "> to <" + newDestUri + ">.");
    if (!metadata_1.MappedDestinations[newDestUri]) {
        metadata_1.MappedDestinations[newDestUri] = {};
    }
    metadata_1.MappedDestinations[newDestUri][origDestUri] =
        uriService.getUriObject(origDestUri);
}
function handleHttpsEverywhereUriRewrite(oldURI, newSpec) {
    oldURI = oldURI.QueryInterface(Ci.nsIURI);
    mapDestinations(oldURI.spec, newSpec);
}
function checkRedirect(request) {
    var e_6, _a;
    var originURI = request.originURI;
    var destURI = request.destURI;
    var originURIObj = uriService.getUriObject(originURI);
    var destURIObj = uriService.getUriObject(destURI);
    {
        var result_1 = app_background_1.rp.policy.checkRequestAgainstUserRules(originURIObj, destURIObj);
        if (result_1.denyRulesExist() && result_1.allowRulesExist()) {
            var _b = result_1.resolveConflict(), conflictCanBeResolved = _b.conflictCanBeResolved, shouldAllow = _b.shouldAllow;
            result_1.isAllowed = conflictCanBeResolved ? shouldAllow :
                cachedSettings.alias.isDefaultAllow();
            return result_1;
        }
        if (result_1.denyRulesExist()) {
            result_1.isAllowed = false;
            return result_1;
        }
        if (result_1.allowRulesExist()) {
            result_1.isAllowed = true;
            return result_1;
        }
    }
    {
        var result_2 = app_background_1.rp.policy.checkRequestAgainstSubscriptionRules(originURIObj, destURIObj);
        if (result_2.denyRulesExist() && result_2.allowRulesExist()) {
            var _c = result_2.resolveConflict(), conflictCanBeResolved = _c.conflictCanBeResolved, shouldAllow = _c.shouldAllow;
            result_2.isAllowed = conflictCanBeResolved ? shouldAllow :
                cachedSettings.alias.isDefaultAllow();
            return result_2;
        }
        if (result_2.denyRulesExist()) {
            result_2.isAllowed = false;
            return result_2;
        }
        if (result_2.allowRulesExist()) {
            result_2.isAllowed = true;
            return result_2;
        }
    }
    if (destURI[0] && destURI[0] === "/" || destURI.indexOf(":") === -1) {
        return new request_result_1.RequestResult(true, request_result_1.RequestReason.RelativeUrl);
    }
    try {
        for (var CompatibilityRules_2 = __values(compatibility_rules_1.CompatibilityRules), CompatibilityRules_2_1 = CompatibilityRules_2.next(); !CompatibilityRules_2_1.done; CompatibilityRules_2_1 = CompatibilityRules_2.next()) {
            var rule = CompatibilityRules_2_1.value;
            var allowOrigin = rule.origin ? originURI.startsWith(rule.origin) : true;
            var allowDest = rule.dest ? destURI.startsWith(rule.dest) : true;
            if (allowOrigin && allowDest) {
                return new request_result_1.RequestResult(true, request_result_1.RequestReason.Compatibility);
            }
        }
    }
    catch (e_6_1) { e_6 = { error: e_6_1 }; }
    finally {
        try {
            if (CompatibilityRules_2_1 && !CompatibilityRules_2_1.done && (_a = CompatibilityRules_2.return)) _a.call(CompatibilityRules_2);
        }
        finally { if (e_6) throw e_6.error; }
    }
    var result = request.checkByDefaultPolicy();
    return result;
}
function isAllowedRedirect(originURI, destURI) {
    var request = new request_1.Request(originURI, destURI);
    return true === checkRedirect(request).isAllowed;
}
exports.isAllowedRedirect = isAllowedRedirect;
function processUrlRedirection(request) {
    if (request.isInternal()) {
        logRequests.log("" + "Allowing a redirection that seems to be internal. " +
            "Origin: " + request.originURI + ", Dest: " + request.destURI);
        return CP_OK;
    }
    var originURI = request.originURI, destURI = request.destURI;
    if (!isContentRequest(request)) {
        var originPath = request.originUriObj.path;
        if (originPath === "/favicon.ico" ||
            metadata_1.FaviconRequests[originURI]) {
            metadata_1.FaviconRequests[destURI] = true;
            log_1.log.info("Redirection from <" + originURI + "> to <" + destURI + "> " +
                "appears to be a redirected favicon request. " +
                "This will be treated as a content request.");
        }
        else {
            logRequests.warn("** ALLOWED ** redirection from <" + originURI + "> " +
                ("to <" + destURI + ">. ") +
                "Original request is from privileged code.");
            return CP_OK;
        }
    }
    if (request.destUriObj.schemeIs("javascript")) {
        log_1.log.warn("Ignoring redirect to javascript URI <" + destURI + ">");
        return CP_OK;
    }
    request.requestResult = checkRedirect(request);
    if (true === request.requestResult.isAllowed) {
        logRequests.warn("** ALLOWED ** redirection from <" + originURI + "> " +
            ("to <" + destURI + ">. ") +
            "Same hosts or allowed origin/destination.");
        requests_1.Requests.notifyNewRequest({
            isAllowed: true,
            isInsert: false,
            requestResult: request.requestResult,
            originUri: originURI,
            destUri: destURI,
        });
        metadata_1.AllowedRedirectsReverse[destURI] = originURI;
        if (request.oldChannel.referrer) {
            var realOrigin = request.oldChannel.referrer.spec;
            if (metadata_1.ClickedLinks[realOrigin] &&
                metadata_1.ClickedLinks[realOrigin][originURI]) {
                logRequests.log("" + "This redirect was from a link click." +
                    " Registering an additional click to <" + destURI + "> " +
                    ("from <" + realOrigin + ">"));
                registerLinkClicked(realOrigin, destURI);
            }
            else if (metadata_1.SubmittedForms[realOrigin] &&
                metadata_1.SubmittedForms[realOrigin][originURI.split("?")[0]]) {
                logRequests.log("" + "This redirect was from a form submission." +
                    " Registering an additional form submission to <" + destURI + "> " + ("from <" + realOrigin + ">"));
                registerFormSubmitted(realOrigin, destURI);
            }
        }
        return CP_OK;
    }
    try {
        if (cachedSettings.alias.isBlockingDisabled()) {
            return CP_OK;
        }
        maybeShowRedirectNotification(request);
        {
            var initialOrigin = getOriginOfInitialRedirect(request);
            if (metadata_1.ClickedLinksReverse.hasOwnProperty(initialOrigin)) {
                var linkClickDest = initialOrigin;
                var linkClickOrigin = void 0;
                for (var i in metadata_1.ClickedLinksReverse[linkClickDest]) {
                    if (metadata_1.ClickedLinksReverse[linkClickDest].
                        hasOwnProperty(i)) {
                        linkClickOrigin = i;
                    }
                }
                requests_1.Requests.notifyNewRequest({
                    isAllowed: true,
                    isInsert: true,
                    originUri: linkClickOrigin,
                    destUri: linkClickDest,
                    requestResult: request.requestResult,
                });
            }
        }
        requests_1.Requests.notifyNewRequest({
            isAllowed: false,
            isInsert: false,
            requestResult: request.requestResult,
            originUri: request.originURI,
            destUri: request.destURI,
        });
        logRequests.warn("** BLOCKED ** redirection from <" + originURI + "> " +
            ("to <" + destURI + ">."));
        return CP_REJECT;
    }
    catch (e) {
        console.error("[RequestPolicy] Fatal Error:");
        console.dir(e);
        if (cachedSettings.alias.isBlockingDisabled()) {
            log_1.log.warn("Allowing request due to internal error.");
            return CP_OK;
        }
        log_1.log.warn("Rejecting request due to internal error.");
        return CP_REJECT;
    }
}
exports.processUrlRedirection = processUrlRedirection;
function showRedirectNotification(request) {
    var browser = request.browser;
    if (browser === null) {
        return false;
    }
    var window = browser.ownerGlobal;
    Utils.tryMultipleTimes(function () {
        var showNotification = Utils.getObjectPath(window, "rpcontinued", "overlay", "_showRedirectNotification");
        if (!showNotification) {
            return false;
        }
        return showNotification(browser, request.destURIWithRef, 0, request.originURI, true);
    });
    return true;
}
function maybeShowRedirectNotification(aRequest) {
    {
        var loadFlags = aRequest.loadFlags;
        var topLevelDocFlag = Ci.nsIChannel.LOAD_INITIAL_DOCUMENT_URI;
        if ((loadFlags & topLevelDocFlag) !== topLevelDocFlag) {
            return;
        }
    }
    var rv = showRedirectNotification(aRequest);
    if (true !== rv) {
        log_1.log.warn("" + "A redirection of a top-level document has been observed, " +
            "but it was not possible to notify the user! The redirection " +
            "was from page <" + aRequest.originURI + "> " +
            ("to <" + aRequest.destURI + ">."));
    }
}
function getOriginOfInitialRedirect(aRequest) {
    var initialOrigin = aRequest.originURI;
    var initialDest = aRequest.destURI;
    var ASSUME_REDIRECT_LOOP = 100;
    for (var i = 0; i < ASSUME_REDIRECT_LOOP; ++i) {
        if (!metadata_1.AllowedRedirectsReverse.hasOwnProperty(initialOrigin)) {
            break;
        }
        initialDest = initialOrigin;
        initialOrigin = metadata_1.AllowedRedirectsReverse[initialOrigin];
    }
    return initialOrigin;
}
function isContentRequest(request) {
    var loadContext = request.oldChannel.loadContext;
    if (loadContext === null) {
        return false;
    }
    return !!loadContext.isContent;
}
exports.whenReady = Promise.resolve();