"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var app_background_1 = require("app/app.background");
var log_1 = require("app/log");
var request_result_1 = require("lib/request-result");
var uriService = app_background_1.rp.services.uri;
function getUriIdentifier(uri) {
    try {
        return uriService.getIdentifier(uri, uriService.hostLevels.SOP);
    }
    catch (e) {
        var msg = "getUriIdentifier exception on uri <" + uri + "> " +
            (". Exception was: " + e);
        throw new Error(msg);
    }
}
var RequestSet = (function () {
    function RequestSet() {
        this._origins = {};
    }
    RequestSet.prototype.print = function (name, printFn) {
        if (printFn === void 0) { printFn = log_1.log.log.bind(log_1.log); }
        printFn("-------------------------------------------------");
        printFn("== Request Set <" + name + "> ==");
        var origins = this._origins;
        for (var oUri in origins) {
            printFn("" + "      " + "Origin uri: <" + oUri + ">");
            for (var dBase in origins[oUri]) {
                var dests = origins[oUri];
                printFn("" + "        " + "Dest base domain: <" + dBase + ">");
                for (var dIdent in dests[dBase]) {
                    printFn("" + "          " + "Dest identifier: <" + dIdent + ">");
                    for (var dUri in dests[dBase][dIdent]) {
                        var n = dests[dBase][dIdent][dUri].length;
                        printFn("" + "            " + "Dest uri: (" + n + " requests) <" + dUri + ">");
                    }
                }
            }
        }
        printFn("-------------------------------------------------");
    };
    RequestSet.prototype.getAll = function () {
        return this._origins;
    };
    RequestSet.prototype.getAllMergedOrigins = function () {
        var result = {};
        for (var originUri in this._origins) {
            var dests = this._origins[originUri];
            for (var destBase in dests) {
                if (!result[destBase]) {
                    result[destBase] = {};
                }
                for (var destIdent in dests[destBase]) {
                    if (!result[destBase][destIdent]) {
                        result[destBase][destIdent] = {};
                    }
                    for (var destUri in dests[destBase][destIdent]) {
                        if (!result[destBase][destIdent][destUri]) {
                            result[destBase][destIdent][destUri] =
                                dests[destBase][destIdent][destUri];
                        }
                        else {
                            result[destBase][destIdent][destUri] =
                                result[destBase][destIdent][destUri].
                                    concat(dests[destBase][destIdent][destUri]);
                        }
                    }
                }
            }
        }
        return result;
    };
    RequestSet.prototype.getOriginUri = function (originUri) {
        return this._origins[originUri] || {};
    };
    RequestSet.prototype.addRequest = function (originUri, destUri, aRequestResult) {
        var requestResult = aRequestResult;
        if (requestResult === undefined) {
            log_1.log.warn("addRequest() was called without a requestResult object!" +
                " Creating a new one. -- " +
                ("\"origin: <" + originUri + ">, destination: <" + destUri + ">"));
            requestResult = new request_result_1.RequestResult();
        }
        if (!this._origins[originUri]) {
            this._origins[originUri] = {};
        }
        var dests = this._origins[originUri];
        var destBase = uriService.getBaseDomain(destUri);
        if (!dests[destBase]) {
            dests[destBase] = {};
        }
        var destIdent = getUriIdentifier(destUri);
        if (!dests[destBase][destIdent]) {
            dests[destBase][destIdent] = {};
        }
        if (!dests[destBase][destIdent][destUri]) {
            dests[destBase][destIdent][destUri] = [];
        }
        if (requestResult instanceof Array) {
            dests[destBase][destIdent][destUri] =
                dests[destBase][destIdent][destUri].
                    concat(requestResult);
        }
        else {
            dests[destBase][destIdent][destUri].push(requestResult);
        }
    };
    RequestSet.prototype.removeRequest = function (originUri, destUri) {
        if (!this._origins[originUri]) {
            return;
        }
        var dests = this._origins[originUri];
        var destBase = uriService.getBaseDomain(destUri);
        if (!dests[destBase]) {
            return;
        }
        var destIdent = getUriIdentifier(destUri);
        if (!dests[destBase][destIdent]) {
            return;
        }
        if (!dests[destBase][destIdent][destUri]) {
            return;
        }
        delete dests[destBase][destIdent][destUri];
        if (Object.getOwnPropertyNames(dests[destBase][destIdent]).length > 0) {
            return;
        }
        delete dests[destBase][destIdent];
        if (Object.getOwnPropertyNames(dests[destBase]).length > 0) {
            return;
        }
        delete dests[destBase];
        if (Object.getOwnPropertyNames(dests).length > 0) {
            return;
        }
        delete this._origins[originUri];
    };
    RequestSet.prototype.removeOriginUri = function (originUri) {
        delete this._origins[originUri];
    };
    RequestSet.prototype.containsBlockedRequests = function () {
        var e_1, _a;
        var origins = this._origins;
        for (var originURI in origins) {
            var originUriRequests = origins[originURI];
            for (var destBase in originUriRequests) {
                var destBaseRequests = originUriRequests[destBase];
                for (var destIdent in destBaseRequests) {
                    var destIdentRequests = destBaseRequests[destIdent];
                    for (var destURI in destIdentRequests) {
                        var destUriRequests = destIdentRequests[destURI];
                        try {
                            for (var destUriRequests_1 = __values(destUriRequests), destUriRequests_1_1 = destUriRequests_1.next(); !destUriRequests_1_1.done; destUriRequests_1_1 = destUriRequests_1.next()) {
                                var request = destUriRequests_1_1.value;
                                if (true !== request.isAllowed) {
                                    return true;
                                }
                            }
                        }
                        catch (e_1_1) { e_1 = { error: e_1_1 }; }
                        finally {
                            try {
                                if (destUriRequests_1_1 && !destUriRequests_1_1.done && (_a = destUriRequests_1.return)) _a.call(destUriRequests_1);
                            }
                            finally { if (e_1) throw e_1.error; }
                        }
                    }
                }
            }
        }
        return false;
    };
    return RequestSet;
}());
exports.RequestSet = RequestSet;