"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var app_background_1 = require("app/app.background");
var log_1 = require("app/log");
var http_channel_wrapper_1 = require("lib/http-channel-wrapper");
var request_result_1 = require("lib/request-result");
var try_catch_utils_1 = require("lib/utils/try-catch-utils");
var WindowUtils = require("lib/utils/window-utils");
var logRequests = log_1.log.extend({
    enabledCondition: { type: "C", C: "LOG_REQUESTS" },
    level: "all",
    name: "Requests",
});
var uriService = app_background_1.rp.services.uri;
var INTERNAL_SCHEMES = new Set([
    "resource",
    "about",
    "chrome",
    "moz-extension",
    "moz-icon",
    "moz-filedata",
]);
var WHITELISTED_DESTINATION_SCHEMES = new Set([
    "data",
    "blob",
    "wyciwyg",
    "javascript",
]);
var DEFAULT_ALLOWED_SCHEMES = new Set([
    "moz-extension",
]);
var DEFAULT_ALLOWED_DESTINATION_RESOURCE_URIS = new Set([
    "resource://gre/res/ImageDocument.css",
    "resource://gre/res/TopLevelImageDocument.css",
    "resource://gre-resources/plaintext.css",
    "resource://gre/res/TopLevelVideoDocument.css",
]);
var profileUri = (function () {
    var fileHandler = Services.io.getProtocolHandler("file").
        QueryInterface(Ci.nsIFileProtocolHandler);
    var profileDir = Services.dirsvc.get("ProfD", Ci.nsIFile);
    return fileHandler.getURLSpecFromDir(profileDir);
})();
var WHITELISTED_DESTINATION_JAR_PATH_STARTS = [
    profileUri + "extensions/",
];
var Request = (function () {
    function Request(originURI, destURI) {
        this.originURI = originURI;
        this.destURI = destURI;
    }
    Object.defineProperty(Request.prototype, "originUriObj", {
        get: function () {
            if (!this.originURI)
                return null;
            return Services.io.newURI(this.originURI, null, null);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Request.prototype, "destUriObj", {
        get: function () {
            return Services.io.newURI(this.destURI, null, null);
        },
        enumerable: true,
        configurable: true
    });
    Request.prototype.setOriginURI = function (originURI) {
        this.originURI = originURI;
    };
    Request.prototype.setDestURI = function (destURI) {
        this.destURI = destURI;
    };
    Request.prototype.detailsToString = function () {
        return "destination: " + this.destURI + ", origin: " + this.originURI;
    };
    Request.prototype.isTopLevel = function () {
        return this.getContentPolicyType() === Ci.nsIContentPolicy.TYPE_DOCUMENT;
    };
    Request.prototype.isInternal = function () {
        var e_1, _a;
        var origin = this.originUriObj;
        var dest = this.destUriObj;
        if (origin === undefined || origin === null) {
            logRequests.log("Allowing request without an origin.");
            return true;
        }
        if (origin.spec === "") {
            logRequests.log("Allowing request with empty origin spec!");
            return true;
        }
        if (INTERNAL_SCHEMES.has(dest.scheme) &&
            (INTERNAL_SCHEMES.has(origin.scheme) ||
                origin.spec.startsWith("data:application/vnd.mozilla.xul+xml"))) {
            logRequests.log("Allowing internal request.");
            return true;
        }
        if (WHITELISTED_DESTINATION_SCHEMES.has(dest.scheme)) {
            logRequests.log("Allowing request with a semi-internal destination.");
            return true;
        }
        var destHost = uriService.getHostByUriObj(dest);
        if (destHost === "global" || destHost === "browser") {
            return true;
        }
        if (origin.scheme === "view-source" &&
            dest.spec === "resource://gre-resources/viewsource.css") {
            return true;
        }
        if (dest.scheme === "jar") {
            var path = dest.path;
            try {
                for (var WHITELISTED_DESTINATION_JAR_PATH_STARTS_1 = __values(WHITELISTED_DESTINATION_JAR_PATH_STARTS), WHITELISTED_DESTINATION_JAR_PATH_STARTS_1_1 = WHITELISTED_DESTINATION_JAR_PATH_STARTS_1.next(); !WHITELISTED_DESTINATION_JAR_PATH_STARTS_1_1.done; WHITELISTED_DESTINATION_JAR_PATH_STARTS_1_1 = WHITELISTED_DESTINATION_JAR_PATH_STARTS_1.next()) {
                    var pathStart = WHITELISTED_DESTINATION_JAR_PATH_STARTS_1_1.value;
                    if (path.startsWith(pathStart))
                        return true;
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (WHITELISTED_DESTINATION_JAR_PATH_STARTS_1_1 && !WHITELISTED_DESTINATION_JAR_PATH_STARTS_1_1.done && (_a = WHITELISTED_DESTINATION_JAR_PATH_STARTS_1.return)) _a.call(WHITELISTED_DESTINATION_JAR_PATH_STARTS_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
        }
        if (dest.spec === "about:blank") {
            return true;
        }
        if (origin.scheme === "about" &&
            origin.spec.indexOf("about:neterror?") === 0) {
            return true;
        }
        return false;
    };
    Request.prototype.isAllowedByDefault = function () {
        var origin = this.originUriObj;
        var dest = this.destUriObj;
        if (origin && DEFAULT_ALLOWED_SCHEMES.has(origin.scheme) ||
            DEFAULT_ALLOWED_SCHEMES.has(dest.scheme))
            return true;
        if (dest.scheme === "chrome") {
            if (dest.path.startsWith("/skin/"))
                return true;
            if (dest.spec === "chrome://pluginproblem/content/pluginProblem.xml") {
                return true;
            }
        }
        var destHost = uriService.getHostByUriObj(dest);
        if (dest.scheme === "resource" && (destHost && destHost.startsWith("noscript_") ||
            DEFAULT_ALLOWED_DESTINATION_RESOURCE_URIS.has(dest.spec)))
            return true;
        return false;
    };
    Request.prototype.checkByDefaultPolicy = function () {
        var cachedSettings = app_background_1.rp.storage.cachedSettings;
        if (this.isAllowedByDefault() ||
            cachedSettings.alias.isDefaultAllow() ||
            cachedSettings.alias.isDefaultAllowTopLevel() && this.isTopLevel())
            return new request_result_1.RequestResult(true, request_result_1.RequestReason.DefaultPolicy);
        var originUri = this.originURI;
        var destUri = this.destURI;
        if (cachedSettings.alias.isDefaultAllowSameDomain()) {
            var originDomain = originUri ?
                uriService.getBaseDomain(originUri) : null;
            var destDomain = uriService.getBaseDomain(destUri);
            if (originDomain !== null && destDomain !== null) {
                return new request_result_1.RequestResult(originDomain === destDomain, request_result_1.RequestReason.DefaultSameDomain);
            }
        }
        var originObj = this.originUriObj;
        var destObj = this.destUriObj;
        if (originObj.scheme === "http" && destObj.scheme === "https" &&
            originObj.port === -1 && destObj.port === -1 &&
            originObj.host === destObj.host) {
            return new request_result_1.RequestResult(true, request_result_1.RequestReason.DefaultSameDomain);
        }
        var LEVEL_SOP = uriService.hostLevels.SOP;
        var originIdent = originUri ?
            uriService.getIdentifier(originUri, LEVEL_SOP) : null;
        var destIdent = uriService.getIdentifier(destUri, LEVEL_SOP);
        return new request_result_1.RequestResult(originIdent === destIdent, request_result_1.RequestReason.DefaultSameDomain);
    };
    Request.prototype.getContentPolicyType = function () {
        throw new Error("Request.getContentPolicyType() not implemented");
    };
    return Request;
}());
exports.Request = Request;
var NormalRequest = (function (_super) {
    __extends(NormalRequest, _super);
    function NormalRequest(aContentType, aContentLocation, aRequestOrigin, aContext, aMimeTypeGuess, aExtra, aRequestPrincipal) {
        var _this = _super.call(this, aRequestOrigin ? aRequestOrigin.specIgnoringRef : undefined, aContentLocation.specIgnoringRef) || this;
        _this.aContentType = aContentType;
        _this.aContentLocation = aContentLocation;
        _this.aRequestOrigin = aRequestOrigin;
        _this.aContext = aContext;
        _this.aMimeTypeGuess = aMimeTypeGuess;
        _this.aExtra = aExtra;
        _this.aRequestPrincipal = aRequestPrincipal;
        return _this;
    }
    Object.defineProperty(NormalRequest.prototype, "originUriObj", {
        get: function () {
            return this.aRequestOrigin;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NormalRequest.prototype, "destUriObj", {
        get: function () {
            return this.aContentLocation;
        },
        enumerable: true,
        configurable: true
    });
    NormalRequest.prototype.setOriginURI = function (originURI) {
        this.originURI = originURI;
        this.aRequestOrigin = uriService.getUriObject(originURI);
    };
    NormalRequest.prototype.setDestURI = function (destURI) {
        this.destURI = destURI;
        this.aContentLocation = uriService.getUriObject(destURI);
    };
    Object.defineProperty(NormalRequest.prototype, "destURIWithRef", {
        get: function () {
            return this.aContentLocation.spec;
        },
        enumerable: true,
        configurable: true
    });
    NormalRequest.prototype.getContentPolicyType = function () {
        return this.aContentType;
    };
    NormalRequest.prototype.detailsToString = function () {
        var context = this.aContext instanceof Ci.nsIDOMHTMLElement ?
            "<HTML Element>" : this.aContext;
        return "type: " + this.aContentType +
            ", destination: " + this.destURI +
            ", origin: " + this.originURI +
            ", context: " + context +
            ", mime: " + this.aMimeTypeGuess +
            ", " + this.aExtra;
    };
    NormalRequest.prototype.isAllowedByDefault = function () {
        if (this.aExtra &&
            this.aExtra instanceof Ci.nsISupportsString &&
            this.aExtra.data === "conPolCheckFromDocShell")
            return true;
        if (this.aRequestPrincipal &&
            Services.scriptSecurityManager.isSystemPrincipal(this.aRequestPrincipal))
            return true;
        return _super.prototype.isAllowedByDefault.call(this);
    };
    NormalRequest.prototype.isInternal = function () {
        var rv = _super.prototype.isInternal.call(this);
        if (rv === true) {
            return true;
        }
        if (this.aContentLocation.scheme === "file" &&
            this.aContentType === Ci.nsIContentPolicy.TYPE_DTD) {
            return true;
        }
        return false;
    };
    NormalRequest.prototype.getContentWindow = function () {
        var context = this.aContext;
        if (!context) {
            return null;
        }
        if (context instanceof Ci.nsIDOMXULElement &&
            context.localName === "browser") {
            return context.contentWindow;
        }
        {
            var result = try_catch_utils_1.queryInterface(context, Ci.nsIDOMWindow);
            if (!result.error)
                return result.value;
        }
        var doc;
        {
            var result = try_catch_utils_1.queryInterface(context, Ci.nsIDOMDocument);
            if (!result.error)
                doc = result.value;
        }
        if (!doc) {
            var result = try_catch_utils_1.queryInterface(context, Ci.nsIDOMNode);
            if (!result.error) {
                var node = result.value;
                doc = node.ownerDocument;
            }
        }
        if (!doc)
            return null;
        return doc.defaultView;
    };
    NormalRequest.prototype.getChromeWindow = function () {
        var contentWindow = this.getContentWindow();
        if (contentWindow) {
            return WindowUtils.getChromeWindow(contentWindow);
        }
        else {
            return null;
        }
    };
    NormalRequest.prototype.getBrowser = function () {
        var context = this.aContext;
        if (context instanceof Ci.nsIDOMXULElement &&
            context.localName === "browser") {
            return context;
        }
        else {
            return WindowUtils.getBrowserForWindow(this.getContentWindow());
        }
    };
    return NormalRequest;
}(Request));
exports.NormalRequest = NormalRequest;
var RedirectRequest = (function (_super) {
    __extends(RedirectRequest, _super);
    function RedirectRequest(aOldChannel, aNewChannel, aFlags) {
        var _this = this;
        var oldChannel = new http_channel_wrapper_1.HttpChannelWrapper(aOldChannel);
        var newChannel = new http_channel_wrapper_1.HttpChannelWrapper(aNewChannel);
        _this = _super.call(this, oldChannel.uri.specIgnoringRef, newChannel.uri.specIgnoringRef) || this;
        _this.oldChannel = oldChannel;
        _this.newChannel = newChannel;
        return _this;
    }
    Object.defineProperty(RedirectRequest.prototype, "browser", {
        get: function () {
            return this.oldChannel.browser;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RedirectRequest.prototype, "loadFlags", {
        get: function () {
            return this.oldChannel._httpChannel.loadFlags;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RedirectRequest.prototype, "originUriObj", {
        get: function () {
            return this.oldChannel.uri;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RedirectRequest.prototype, "destUriObj", {
        get: function () {
            return this.newChannel.uri;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RedirectRequest.prototype, "destURIWithRef", {
        get: function () {
            return this.newChannel.uri.spec;
        },
        enumerable: true,
        configurable: true
    });
    RedirectRequest.prototype.getContentPolicyType = function () {
        var loadInfo = this.oldChannel._httpChannel.loadInfo;
        if (!loadInfo)
            return Ci.nsIContentPolicy.TYPE_OTHER;
        if (typeof loadInfo.contentPolicyType !== "undefined") {
            return loadInfo.contentPolicyType;
        }
        return loadInfo.externalContentPolicyType;
    };
    return RedirectRequest;
}(Request));
exports.RedirectRequest = RedirectRequest;