"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var app_background_1 = require("app/app.background");
var log_1 = require("app/log");
var environment_1 = require("lib/environment");
var ruleset_1 = require("lib/ruleset");
var log = log_1.log.extend({
    name: "Subscriptions",
});
exports.SUBSCRIPTION_UPDATED_TOPIC = "rpcontinued-subscription-policy-updated";
exports.SUBSCRIPTION_ADDED_TOPIC = "rpcontinued-subscription-policy-added";
exports.SUBSCRIPTION_REMOVED_TOPIC = "rpcontinued-subscription-policy-removed";
var DEFAULT_SUBSCRIPTION_LIST_URL_BASE = "https://raw.githubusercontent.com/" +
    "RequestPolicyContinued/subscriptions/master/";
var SubscriptionUpdateResult;
(function (SubscriptionUpdateResult) {
    SubscriptionUpdateResult["SUBSCRIPTION_UPDATE_SUCCESS"] = "SUCCESS";
    SubscriptionUpdateResult["SUBSCRIPTION_UPDATE_NOT_NEEDED"] = "NOT_NEEDED";
    SubscriptionUpdateResult["SUBSCRIPTION_UPDATE_FAILURE"] = "FAILURE";
})(SubscriptionUpdateResult || (SubscriptionUpdateResult = {}));
exports.SUBSCRIPTION_UPDATE_SUCCESS = SubscriptionUpdateResult.SUBSCRIPTION_UPDATE_SUCCESS, exports.SUBSCRIPTION_UPDATE_NOT_NEEDED = SubscriptionUpdateResult.SUBSCRIPTION_UPDATE_NOT_NEEDED, exports.SUBSCRIPTION_UPDATE_FAILURE = SubscriptionUpdateResult.SUBSCRIPTION_UPDATE_FAILURE;
function maybeCallback(aCallback) {
    return function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var ok = true;
        try {
            ok = environment_1.MainEnvironment.isShuttingDownOrShutDown() === false;
        }
        catch (e) {
            ok = false;
            log.error("Catched error:", e);
        }
        if (!ok) {
            log.log("Did not call callback function");
            return;
        }
        aCallback.apply(void 0, __spread(args));
    };
}
var UserSubscriptions = (function () {
    function UserSubscriptions(data) {
        this.data = data;
    }
    UserSubscriptions.create = function (aData) {
        var data;
        if (!aData) {
            data = {};
        }
        else {
            var checkResult = UserSubscriptions.checkDataObj(aData);
            if (!checkResult.valid) {
                throw new Error("Invalid subscription data: " + checkResult.error);
            }
            data = aData;
        }
        if (!data.lists) {
            data.lists = {
                official: {
                    subscriptions: {
                        allow_embedded: {},
                        allow_extensions: {},
                        allow_functionality: {},
                        allow_mozilla: {},
                        allow_sameorg: {},
                        deny_trackers: {},
                    },
                },
            };
        }
        return new UserSubscriptions(data);
    };
    UserSubscriptions.checkDataObj = function (dataObj) {
        if (!dataObj || typeof dataObj !== "object") {
            return { valid: false, error: "not an object" };
        }
        if ("lists" in dataObj) {
        }
        return { valid: true };
    };
    Object.defineProperty(UserSubscriptions.prototype, "lists", {
        get: function () {
            return this.data.lists;
        },
        enumerable: true,
        configurable: true
    });
    UserSubscriptions.prototype.toString = function () {
        return "[UserSubscriptions]";
    };
    UserSubscriptions.prototype.save = function () {
        browser.storage.local.set({
            subscriptions: this.data,
        }).catch(function (e) {
            log.error("UserSubscriptions.save():", e);
        });
    };
    UserSubscriptions.prototype.getSubscriptionInfo = function () {
        var lists = this.data.lists;
        var result = {};
        for (var listName in lists) {
            if (!lists[listName].subscriptions) {
                continue;
            }
            result[listName] = {};
            for (var subName in lists[listName].subscriptions) {
                result[listName][subName] = null;
            }
        }
        return result;
    };
    UserSubscriptions.prototype.addSubscription = function (listName, subName) {
        var lists = this.data.lists;
        if (!lists[listName]) {
            lists[listName] = { subscriptions: {} };
        }
        if (!lists[listName].subscriptions) {
            lists[listName].subscriptions = {};
        }
        lists[listName].subscriptions[subName] = {};
        this.save();
    };
    UserSubscriptions.prototype.removeSubscription = function (listName, subName) {
        var lists = this.data.lists;
        if (lists[listName] && lists[listName].subscriptions &&
            lists[listName].subscriptions[subName]) {
            delete lists[listName].subscriptions[subName];
        }
        this.save();
    };
    UserSubscriptions.prototype.update = function (callback, serials) {
        var updatingLists = {};
        var updateResults = {};
        var recordDone = function (listName, subName, result) {
            log.info("Recording done: " + listName + " " + subName);
            if (subName) {
                var updateResultsList = updateResults[listName];
                updateResultsList[subName] = result;
                var list = updatingLists[listName];
                delete list[subName];
                for (var i in list) {
                    i;
                    return;
                }
            }
            delete updatingLists[listName];
            for (var i in updatingLists) {
                i;
                return;
            }
            setTimeout(function () { return callback(updateResults); }, 0);
        };
        var listCount = 0;
        var _loop_1 = function (listName) {
            if (!this_1.lists[listName] || !this_1.lists[listName].subscriptions) {
                log.info("Skipping update of unsubscribed list: " + listName);
                return "continue";
            }
            var updateSubs = {};
            var subCount = 0;
            for (var subName in serials[listName]) {
                if (!this_1.lists[listName].subscriptions[subName]) {
                    log.info("Skipping update of unsubscribed subscription: " + listName +
                        " " + subName);
                    continue;
                }
                updateSubs[subName] = { serial: serials[listName][subName] };
                subCount++;
            }
            if (subCount === 0) {
                log.info("Skipping list with no subscriptions: " + listName);
                return "continue";
            }
            var url = this_1.lists[listName].url;
            if (!url) {
                url = DEFAULT_SUBSCRIPTION_LIST_URL_BASE + listName + ".json";
            }
            if (!url) {
                log.info("Skipping list without url: " + listName);
                return "continue";
            }
            var list = new SubscriptionList(listName, url);
            updatingLists[listName] = {};
            for (var subName in updateSubs) {
                log.info("Will update subscription: " + listName + " " + subName);
                updatingLists[listName][subName] = true;
            }
            updateResults[listName] = {};
            var metadataSuccess = function () {
                var subSuccess = function (sub, status) {
                    log.info("Successfully updated subscription " + sub.toString());
                    recordDone(list.name, sub.name, status);
                };
                var subError = function (sub, error) {
                    log.info("Failed to update subscription " + sub.toString() + ": " +
                        error);
                    recordDone(list.name, sub.name, exports.SUBSCRIPTION_UPDATE_FAILURE);
                };
                log.info("Successfully updated list " + list.toString());
                list.updateSubscriptions(updateSubs, subSuccess, subError);
            };
            var metadataError = function (error) {
                log.info("Failed to update list: " + list.toString() + ": " + error);
                updateResults[listName] = false;
                recordDone(list.name);
            };
            listCount++;
            log.info("Will update list: " + listName);
            list.updateMetadata(metadataSuccess, metadataError);
        };
        var this_1 = this;
        for (var listName in serials) {
            _loop_1(listName);
        }
        if (listCount === 0) {
            log.info("No lists to update.");
            setTimeout(function () { return callback(updateResults); }, 0);
        }
    };
    return UserSubscriptions;
}());
exports.UserSubscriptions = UserSubscriptions;
var SubscriptionList = (function () {
    function SubscriptionList(name, url) {
        this.name = name;
        this.url = url;
    }
    SubscriptionList.prototype.toString = function () {
        return "[SubscriptionList " + this.name + " " + this.url + "]";
    };
    SubscriptionList.prototype.updateMetadata = function (successCallback, errorCallback) {
        var _this = this;
        log.info("Updating " + this.toString());
        var req = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"]
            .createInstance(Ci.nsIXMLHttpRequest);
        req.onload = maybeCallback(function (event) {
            try {
                _this.data = JSON.parse(req.responseText);
                setTimeout(function () { return successCallback(); }, 0);
            }
            catch (e) {
                setTimeout(function () { return errorCallback(e.toString()); }, 0);
            }
        });
        req.onerror = maybeCallback(function (event) {
            setTimeout(function () { return errorCallback(req.statusText); }, 0);
        });
        req.open("GET", this.url);
        req.send(null);
    };
    SubscriptionList.prototype.updateSubscriptions = function (userSubs, successCallback, errorCallback) {
        var _loop_2 = function (subName) {
            var serial = this_2.getSubscriptionSerial(subName);
            if (!serial) {
                return "continue";
            }
            log.info("Current serial for " + this_2.name + " " + subName + ": " +
                userSubs[subName].serial);
            log.info("Available serial for " + this_2.name + " " + subName + ": " +
                serial);
            var subUrl = this_2.getSubscriptionUrl(subName);
            if (!subUrl) {
                return "continue";
            }
            var sub = new Subscription(this_2.name, subName, subUrl);
            try {
                if (serial > userSubs[subName].serial) {
                    sub.update(successCallback, errorCallback);
                }
                else {
                    log.info("No update needed for " + this_2.name + " " + subName);
                    var curSub_1 = sub;
                    setTimeout(function () {
                        successCallback(curSub_1, exports.SUBSCRIPTION_UPDATE_NOT_NEEDED);
                    }, 0);
                }
            }
            catch (e) {
                var curSub_2 = sub;
                setTimeout(function () { return errorCallback(curSub_2, e.toString()); }, 0);
            }
        };
        var this_2 = this;
        for (var subName in userSubs) {
            _loop_2(subName);
        }
    };
    SubscriptionList.prototype.getSubscriptionSerial = function (subName) {
        if (!(subName in this.data.subscriptions)) {
            return null;
        }
        return this.data.subscriptions[subName].serial;
    };
    SubscriptionList.prototype.getSubscriptionUrl = function (subName) {
        if (!(subName in this.data.subscriptions)) {
            return null;
        }
        return this.data.subscriptions[subName].url;
    };
    return SubscriptionList;
}());
var Subscription = (function () {
    function Subscription(listName, subName, subUrl) {
        this.list = listName;
        this.name = subName;
        this.url = subUrl;
    }
    Subscription.prototype.toString = function () {
        return "[Subscription " + this.list + " " + this.name + " " +
            this.url + "]";
    };
    Subscription.prototype.update = function (successCallback, errorCallback) {
        var _this = this;
        log.info("Updating " + this.toString());
        var req = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].
            createInstance(Ci.nsIXMLHttpRequest);
        req.onload = maybeCallback(function (event) {
            try {
                var rawData = req.responseText;
                if (!rawData) {
                    var error_1 = "Empty response when requesting subscription file";
                    setTimeout(function () { return errorCallback(_this, error_1); }, 0);
                    return;
                }
                _this.data = JSON.parse(rawData);
                var serial = void 0;
                if (_this.data && _this.data.metadata && _this.data.metadata.serial) {
                    serial = _this.data.metadata.serial;
                }
                else {
                    var error_2 = "Ruleset has no serial number";
                    setTimeout(function () { return errorCallback(_this, error_2); }, 0);
                    return;
                }
                if (typeof serial !== "number" || serial % 1 !== 0) {
                    var error_3 = "Ruleset has invalid serial number: " + serial;
                    setTimeout(function () { return errorCallback(_this, error_3); }, 0);
                    return;
                }
                try {
                    var rawRuleset = ruleset_1.RawRuleset.create(_this.data);
                    app_background_1.rp.policy.rulesetStorage.saveRawRulesetToFile(rawRuleset, _this.name, _this.list);
                }
                catch (e) {
                    setTimeout(function () { return errorCallback(_this, e.toString()); }, 0);
                    return;
                }
                var subInfo = {};
                subInfo[_this.list] = {};
                subInfo[_this.list][_this.name] = true;
                Services.obs.notifyObservers(null, exports.SUBSCRIPTION_UPDATED_TOPIC, JSON.stringify(subInfo));
                setTimeout(function () {
                    successCallback(_this, exports.SUBSCRIPTION_UPDATE_SUCCESS);
                }, 0);
            }
            catch (e) {
                setTimeout(function () { return errorCallback(_this, e.toString()); }, 0);
            }
        });
        req.onerror = maybeCallback(function (event) {
            setTimeout(function () { return errorCallback(_this, req.statusText); }, 0);
        });
        req.open("GET", this.url);
        req.send(null);
    };
    return Subscription;
}());