"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function runAsync(callback, thisPtr) {
    var params = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        params[_i - 2] = arguments[_i];
    }
    var runnable = {
        run: function () {
            try {
                callback.apply(thisPtr, params);
            }
            catch (e) {
                console.error("[RequestPolicy] Asynchronous callback failed unexpectly. Details:");
                console.dir(e);
                throw e;
            }
        },
    };
    Services.tm.currentThread.dispatch(runnable, Ci.nsIEventTarget.DISPATCH_NORMAL);
}
exports.runAsync = runAsync;
function tryMultipleTimes(aFunction, aTries) {
    if (aTries === void 0) { aTries = 10; }
    if (aTries <= 0) {
        return;
    }
    var triesLeft = aTries - 1;
    runAsync(function () {
        if (aFunction.call(null, triesLeft) !== true) {
            tryMultipleTimes(aFunction, triesLeft);
        }
    });
}
exports.tryMultipleTimes = tryMultipleTimes;
function getObjectPath(object) {
    var properties = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        properties[_i - 1] = arguments[_i];
    }
    return properties.reduce(getObjectProperty, object);
}
exports.getObjectPath = getObjectPath;
function getObjectProperty(object, property) {
    if (!!object && object.hasOwnProperty(property)) {
        return object[property];
    }
    return undefined;
}
exports.getObjectProperty = getObjectProperty;
function createModuleInternal(aModuleScope) {
    var internal = {};
    var sealed = false;
    aModuleScope.getInternal = function () {
        if (sealed) {
            return undefined;
        }
        return internal;
    };
    aModuleScope.sealInternal = function () {
        sealed = true;
    };
    return internal;
}
exports.createModuleInternal = createModuleInternal;
function wrapFunction(aOwnerObject, aFunctionName, aErrorCallback, aBeforeFunction, aAfterFunction) {
    if (aBeforeFunction === void 0) { aBeforeFunction = null; }
    if (aAfterFunction === void 0) { aAfterFunction = null; }
    initWrapperFunction(aOwnerObject, aFunctionName);
    var fnMetadata = aOwnerObject.rpcontinuedWrappedFunctions[aFunctionName];
    fnMetadata.before = aBeforeFunction;
    fnMetadata.after = aAfterFunction;
}
exports.wrapFunction = wrapFunction;
function unwrapFunction(aOwnerObject, aFunctionName) {
    wrapFunction(aOwnerObject, aFunctionName, null, null, null);
}
exports.unwrapFunction = unwrapFunction;
function initWrapperFunction(aOwnerObject, aFunctionName, aErrorCallback) {
    function onError(aError, aWhen) {
        if (typeof aErrorCallback === "function") {
            aErrorCallback("The \"" + aWhen + "\" function of the `" + aFunctionName + "()` wrapper " +
                "has thrown an error.", aError);
        }
    }
    if (!aOwnerObject.hasOwnProperty("rpcontinuedWrappedFunctions")) {
        aOwnerObject.rpcontinuedWrappedFunctions = {};
    }
    var metadata = aOwnerObject.rpcontinuedWrappedFunctions;
    if (metadata.hasOwnProperty(aFunctionName)) {
        return;
    }
    metadata[aFunctionName] = {
        main: aOwnerObject[aFunctionName],
        before: null,
        after: null,
    };
    aOwnerObject[aFunctionName] = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var _a = metadata[aFunctionName], main = _a.main, before = _a.before, after = _a.after;
        try {
            if (before) {
                before.apply(aOwnerObject, args);
            }
        }
        catch (e) {
            onError(e, "before");
        }
        var rv = main.apply(aOwnerObject, args);
        try {
            if (after) {
                after.apply(aOwnerObject, args);
            }
        }
        catch (e) {
            onError(e, "after");
        }
        return rv;
    };
}