"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var JSUtils = require("lib/utils/js-utils");
var constants_1 = require("data/constants");
exports.xulTrees = {};
(function importXulTrees() {
    var xulTreesScope = {
        "exports": exports.xulTrees,
        "C": constants_1.C,
        "appID": Services.appinfo.ID,
    };
    Services.scriptloader.loadSubScript("chrome://rpcontinued/content/ui/xul-trees.js", xulTreesScope);
    var nextID = 1;
    for (var treeName in exports.xulTrees) {
        recursivelyGetAllElementSpecs(exports.xulTrees[treeName]).forEach(ensureIntegrity);
    }
    function ensureIntegrity(aElementSpec) {
        if (!aElementSpec.hasOwnProperty("attributes")) {
            aElementSpec.attributes = {};
        }
        if (!aElementSpec.attributes.hasOwnProperty("id")) {
            aElementSpec.attributes.id = "rpc-autoid-" + nextID++;
        }
    }
})();
function recursivelyGetAllElementSpecs(aElementSpecList) {
    var e_1, _a;
    var allElementSpecs = [];
    try {
        for (var aElementSpecList_1 = __values(aElementSpecList), aElementSpecList_1_1 = aElementSpecList_1.next(); !aElementSpecList_1_1.done; aElementSpecList_1_1 = aElementSpecList_1.next()) {
            var elementSpec = aElementSpecList_1_1.value;
            if (!elementSpec) {
                console.error("[RequestPolicy] An element spec is null!");
                continue;
            }
            allElementSpecs.push(elementSpec);
            if (elementSpec.hasOwnProperty("children")) {
                var allChildrenSpecs = recursivelyGetAllElementSpecs(elementSpec.children);
                allElementSpecs = allElementSpecs.concat(allChildrenSpecs);
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (aElementSpecList_1_1 && !aElementSpecList_1_1.done && (_a = aElementSpecList_1.return)) _a.call(aElementSpecList_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return allElementSpecs;
}
function getParentElementOfSubobject(aDocument, aElementSpec) {
    var subobjectTree = aElementSpec.parent.special.tree;
    var parentElement = aDocument.getElementById(aElementSpec.parent.special.id);
    for (var i = 0, len = subobjectTree.length; i < len; ++i) {
        parentElement = parentElement[subobjectTree[i]];
    }
    return parentElement;
}
function getParentElement(aDocument, aElementSpec) {
    if (!aElementSpec.parent) {
        return false;
    }
    else {
        if (aElementSpec.parent.id) {
            return aDocument.getElementById(aElementSpec.parent.id);
        }
        else if (aElementSpec.parent.special) {
            switch (aElementSpec.parent.special.type) {
                case "__window__":
                    return aDocument.querySelector("window");
                case "subobject":
                    return getParentElementOfSubobject(aDocument, aElementSpec);
                default:
                    return false;
            }
        }
        else {
            return false;
        }
    }
}
function getLocalizedValue(aRawValue) {
    if (!LegacyApi.i18n.matchKeyPattern(aRawValue)) {
        return aRawValue;
    }
    return LegacyApi.i18n.updateString(aRawValue);
}
function setAttributes(aElement, aElementSpec) {
    if (!aElementSpec.hasOwnProperty("attributes")) {
        return;
    }
    for (var attributeName in aElementSpec.attributes) {
        var value = getLocalizedValue(aElementSpec.attributes[attributeName]);
        if (value) {
            aElement.setAttribute(attributeName, value);
        }
    }
}
var _a = (function () {
    function getEventListener(aRootObject, aListenerSpec) {
        var e_2, _a;
        var object = aRootObject;
        try {
            for (var aListenerSpec_1 = __values(aListenerSpec), aListenerSpec_1_1 = aListenerSpec_1.next(); !aListenerSpec_1_1.done; aListenerSpec_1_1 = aListenerSpec_1.next()) {
                var propertyName = aListenerSpec_1_1.value;
                if (!object.hasOwnProperty(propertyName)) {
                    return null;
                }
                object = object[propertyName];
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (aListenerSpec_1_1 && !aListenerSpec_1_1.done && (_a = aListenerSpec_1.return)) _a.call(aListenerSpec_1);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return object;
    }
    function getEventInfoList(aEventTarget, aEventList) {
        if (!aEventList) {
            return [];
        }
        var rootObject = aEventTarget.ownerDocument.defaultView.rpcontinued;
        return Object.keys(aEventList).map(function (eventName) {
            return [
                eventName,
                getEventListener(rootObject, aEventList[eventName]),
            ];
        });
    }
    function addEventListeners(aEventTarget, _a) {
        var events = _a.events;
        var listeners = getEventInfoList(aEventTarget, events);
        listeners.forEach(function (_a) {
            var _b = __read(_a, 2), eventName = _b[0], listener = _b[1];
            aEventTarget.addEventListener(eventName, listener, false);
        });
    }
    function removeEventListeners(aEventTarget, _a) {
        var events = _a.events;
        var listeners = getEventInfoList(aEventTarget, events);
        listeners.forEach(function (_a) {
            var _b = __read(_a, 2), eventName = _b[0], listener = _b[1];
            aEventTarget.removeEventListener(eventName, listener, false);
        });
    }
    return {
        addEventListeners: addEventListeners,
        removeEventListeners: removeEventListeners,
    };
})(), addEventListeners = _a.addEventListeners, removeEventListeners = _a.removeEventListeners;
function recursivelyAddXULElements(aDocument, aElementSpecList, aParentElement) {
    if (aParentElement === void 0) { aParentElement = null; }
    var e_3, _a;
    try {
        for (var aElementSpecList_2 = __values(aElementSpecList), aElementSpecList_2_1 = aElementSpecList_2.next(); !aElementSpecList_2_1.done; aElementSpecList_2_1 = aElementSpecList_2.next()) {
            var elementSpec = aElementSpecList_2_1.value;
            if (!elementSpec || !elementSpec.tag) {
                console.error("[RequestPolicy] Element spec incomplete!");
                continue;
            }
            var parentElement = aParentElement ? aParentElement :
                getParentElement(aDocument, elementSpec);
            if (false === parentElement) {
                console.error("[RequestPolicy] " + "The parent element could not " +
                    "be determined. Tag: " + elementSpec.tag + "; " +
                    ("ID: " + elementSpec.attributes.id));
                continue;
            }
            if (parentElement === null) {
                console.error("[RequestPolicy] parentElement of '" + elementSpec.attributes.id + "' is null!");
                continue;
            }
            var newElement = aDocument.createElement(elementSpec.tag);
            setAttributes(newElement, elementSpec);
            addEventListeners(newElement, elementSpec);
            if (elementSpec.children) {
                recursivelyAddXULElements(aDocument, elementSpec.children, newElement);
            }
            parentElement.appendChild(newElement);
        }
    }
    catch (e_3_1) { e_3 = { error: e_3_1 }; }
    finally {
        try {
            if (aElementSpecList_2_1 && !aElementSpecList_2_1.done && (_a = aElementSpecList_2.return)) _a.call(aElementSpecList_2);
        }
        finally { if (e_3) throw e_3.error; }
    }
}
function addTreeElementsToWindow(aWin, aTreeName) {
    if (exports.xulTrees.hasOwnProperty(aTreeName)) {
        recursivelyAddXULElements(aWin.document, exports.xulTrees[aTreeName]);
    }
}
exports.addTreeElementsToWindow = addTreeElementsToWindow;
function getRootElementIDs(aTreeName) {
    var ids = exports.xulTrees[aTreeName].map(function (aElementSpec) {
        return aElementSpec.attributes.id;
    });
    return ids;
}
function removeTreeElementsFromWindow(aWin, aTreeName) {
    var e_4, _a, e_5, _b;
    if (!exports.xulTrees.hasOwnProperty(aTreeName)) {
        console.error("[RequestPolicy] There's no tree with name '" + aTreeName + "'.");
        return;
    }
    var $id = aWin.document.getElementById.bind(aWin.document);
    try {
        for (var _c = __values(recursivelyGetAllElementSpecs(exports.xulTrees[aTreeName])), _d = _c.next(); !_d.done; _d = _c.next()) {
            var elementSpec = _d.value;
            var id = elementSpec.attributes.id;
            var eventTarget = $id(id);
            if (!eventTarget) {
                console.error("[RequestPolicy] Could not find element with ID \"" + id + "\".");
                continue;
            }
            removeEventListeners(eventTarget, elementSpec);
        }
    }
    catch (e_4_1) { e_4 = { error: e_4_1 }; }
    finally {
        try {
            if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
        }
        finally { if (e_4) throw e_4.error; }
    }
    try {
        for (var _e = __values(getRootElementIDs(aTreeName)), _f = _e.next(); !_f.done; _f = _e.next()) {
            var id = _f.value;
            var node = $id(id);
            if (node && node.parentNode) {
                node.parentNode.removeChild(node);
            }
        }
    }
    catch (e_5_1) { e_5 = { error: e_5_1 }; }
    finally {
        try {
            if (_f && !_f.done && (_b = _e.return)) _b.call(_e);
        }
        finally { if (e_5) throw e_5.error; }
    }
}
exports.removeTreeElementsFromWindow = removeTreeElementsFromWindow;
exports.keyboardShortcuts = (function () {
    var self = {};
    var VALID_MODIFIERS = [
        "shift",
        "alt",
        "meta",
        "control",
        "accel",
    ];
    function error(msg) {
        return { success: false, errorMessage: msg };
    }
    function success(returnValue) {
        returnValue.success = true;
        return returnValue;
    }
    function isValidModifier(aModifier) {
        return JSUtils.arrayIncludes(VALID_MODIFIERS, aModifier);
    }
    var keyRegEx = /^[a-z]$/;
    function _getKeyAttributesFromCombo(aCombo) {
        var e_6, _a;
        if (typeof aCombo !== "string") {
            return error("Not a string!");
        }
        if (aCombo === "") {
            return error("The string must not be empty.");
        }
        var parts = aCombo.split(" ");
        var key = parts.slice(-1)[0];
        var modifiers = parts.slice(0, -1);
        modifiers = __spread(new Set(modifiers));
        try {
            for (var modifiers_1 = __values(modifiers), modifiers_1_1 = modifiers_1.next(); !modifiers_1_1.done; modifiers_1_1 = modifiers_1.next()) {
                var modifier = modifiers_1_1.value;
                if (false === isValidModifier(modifier)) {
                    return error("Invalid modifier: \"" + modifier + "\"");
                }
            }
        }
        catch (e_6_1) { e_6 = { error: e_6_1 }; }
        finally {
            try {
                if (modifiers_1_1 && !modifiers_1_1.done && (_a = modifiers_1.return)) _a.call(modifiers_1);
            }
            finally { if (e_6) throw e_6.error; }
        }
        if (!keyRegEx.test(key)) {
            return error("Invalid key: \"" + key + "\"");
        }
        return success({
            modifiers: modifiers.join(" "),
            key: key,
        });
    }
    self.getKeyAttributesFromCombo = function (aCombo) {
        return _getKeyAttributesFromCombo(aCombo);
    };
    return self;
})();