"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var controllers_1 = require("lib/classes/controllers");
var constants_1 = require("data/constants");
var commonjs_unload_subject_1 = require("legacy/lib/commonjs-unload-subject");
var environment_1 = require("lib/environment");
var app_background_1 = require("app/app.background");
require("main/about-uri");
require("main/content-policy");
require("main/window-manager");
var keyboard_shortcut_controller_1 = require("controllers/keyboard-shortcut-controller");
var notification_controller_1 = require("controllers/notification-controller");
var other_rp_installations_controller_1 = require("controllers/other-rp-installations-controller");
var allControllers = [];
var controllersToBeStartedUp = [
    keyboard_shortcut_controller_1.KeyboardShortcutController,
    other_rp_installations_controller_1.OtherRPInstallationsController,
    notification_controller_1.NotificationsController,
];
allControllers = allControllers.concat(controllersToBeStartedUp);
var log_1 = require("app/log");
var background_page_1 = require("models/background-page");
exports.BackgroundPage = background_page_1.BackgroundPage;
_setBackgroundPage(background_page_1.BackgroundPage);
function logSevereError(aMessage, aError) {
    console.error("[RequestPolicy] [SEVERE] " + aMessage + " - Details:");
    console.dir(aError);
}
var shutdownMessage = constants_1.C.MM_PREFIX + "shutdown";
function shutdown(aShutdownArgs) {
    (new controllers_1.Controllers(allControllers)).shutdown();
    environment_1.MainEnvironment.shutdown(aShutdownArgs);
    app_background_1.rp.shutdown().catch(log_1.log.onError("main shutdown"));
}
function broadcastShutdownMessage() {
    Services.mm.broadcastAsyncMessage(shutdownMessage);
}
environment_1.MainEnvironment.addShutdownFunction(environment_1.Level.BACKEND, broadcastShutdownMessage);
var observer = {
    observe: function (subject, topic, reason) {
        if (subject.wrappedJSObject === commonjs_unload_subject_1.COMMONJS_UNLOAD_SUBJECT) {
            try {
                shutdown([null, reason]);
            }
            catch (e) {
                logSevereError("shutdown() failed!", e);
            }
        }
    },
};
Services.obs.addObserver(observer, "sdk:loader:destroy", false);
(function startup() {
    app_background_1.rp.startup().catch(log_1.log.onError("main startup"));
    try {
        environment_1.MainEnvironment.startup([null, null]);
    }
    catch (e) {
        logSevereError("startup() failed!", e);
    }
    (new controllers_1.Controllers(controllersToBeStartedUp)).startup();
})();