"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var environment_1 = require("lib/environment");
var Utils = require("lib/utils/misc-utils");
var FILENAMES = {
    "basicprefs": "basicprefs.html",
    "advancedprefs": "advancedprefs.html",
    "yourpolicy": "yourpolicy.html",
    "defaultpolicy": "defaultpolicy.html",
    "subscriptions": "subscriptions.html",
    "oldrules": "oldrules.html",
    "setup": "setup.html",
    "experimental": "experimental.html",
};
function getURI(aURI) {
    var id;
    var index = aURI.path.indexOf("?");
    if (index >= 0 && aURI.path.length > index) {
        id = aURI.path.substr(index + 1);
    }
    if (!id || !(id in FILENAMES)) {
        id = "basicprefs";
    }
    var spec = "chrome://rpcontinued/content/settings/" + FILENAMES[id];
    return Services.io.newURI(spec, null, null);
}
exports.AboutRequestPolicy = (function () {
    var self = {};
    self.classDescription = "about:requestpolicy";
    self.contractID = "@mozilla.org/network/protocol/about;1?what=requestpolicy";
    self.classID = ComponentsID("{77d4be21-6a28-4b91-9886-15ccd83795e8}");
    self.QueryInterface = XPCOMUtils.generateQI([Ci.nsIAboutModule]);
    self.getURIFlags = function (aURI) {
        return Ci.nsIAboutModule.ALLOW_SCRIPT;
    };
    self.newChannel = function (aURI, aLoadInfo) {
        var uri = getURI(aURI);
        var channel;
        if (LegacyApi.miscInfos.isGeckoVersionAtLeast("48.0a1")) {
            channel = Services.io.newChannelFromURIWithLoadInfo(uri, aLoadInfo);
        }
        else {
            channel = Services.io.newChannelFromURI(uri);
        }
        channel.originalURI = aURI;
        return channel;
    };
    self.createInstance = function (outer, iid) {
        if (outer) {
            throw Cr.NS_ERROR_NO_AGGREGATION;
        }
        return self.QueryInterface(iid);
    };
    function registerFactory() {
        Cm.QueryInterface(Ci.nsIComponentRegistrar).
            registerFactory(self.classID, self.classDescription, self.contractID, self);
    }
    environment_1.MainEnvironment.addStartupFunction(environment_1.Level.INTERFACE, function () {
        try {
            registerFactory();
        }
        catch (e) {
            if (e.result === Cr.NS_ERROR_FACTORY_EXISTS) {
                Utils.runAsync(registerFactory);
            }
            else {
                console.error("[RequestPolicy] Failed to register factory! Details:");
                console.dir(e);
            }
        }
    });
    function unregisterFactory() {
        var registrar = Cm.QueryInterface(Ci.nsIComponentRegistrar);
        Utils.runAsync(function () {
            registrar.unregisterFactory(self.classID, self);
        });
    }
    environment_1.MainEnvironment.addShutdownFunction(environment_1.Level.INTERFACE, unregisterFactory);
    return self;
})();