"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var log_1 = require("app/log");
var request_1 = require("lib/request");
var Utils = require("lib/utils/misc-utils");
var RequestProcessor = require("lib/request-processor");
var environment_1 = require("lib/environment");
var catMan = Cc["@mozilla.org/categorymanager;1"].
    getService(Ci.nsICategoryManager);
var CP_OK = Ci.nsIContentPolicy.ACCEPT;
var CP_REJECT = Ci.nsIContentPolicy.REJECT_SERVER;
exports.RPContentPolicy = (function () {
    var self = {};
    var XPCOM_CATEGORIES = ["content-policy"];
    self.classDescription = "RequestPolicy ContentPolicy Implementation";
    self.classID = ComponentsID("{d734b30a-996c-4805-be24-25a0738249fe}");
    self.contractID = "@requestpolicy.org/content-policy;1";
    function register() {
        var e_1, _a;
        Cm.QueryInterface(Ci.nsIComponentRegistrar).
            registerFactory(self.classID, self.classDescription, self.contractID, self);
        try {
            for (var XPCOM_CATEGORIES_1 = __values(XPCOM_CATEGORIES), XPCOM_CATEGORIES_1_1 = XPCOM_CATEGORIES_1.next(); !XPCOM_CATEGORIES_1_1.done; XPCOM_CATEGORIES_1_1 = XPCOM_CATEGORIES_1.next()) {
                var category = XPCOM_CATEGORIES_1_1.value;
                catMan.addCategoryEntry(category, self.contractID, self.contractID, false, true);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (XPCOM_CATEGORIES_1_1 && !XPCOM_CATEGORIES_1_1.done && (_a = XPCOM_CATEGORIES_1.return)) _a.call(XPCOM_CATEGORIES_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    }
    environment_1.MainEnvironment.addStartupFunction(environment_1.Level.INTERFACE, function () {
        try {
            register();
        }
        catch (e) {
            if (e.result === Cr.NS_ERROR_FACTORY_EXISTS) {
                Utils.runAsync(register);
            }
            else {
                console.error("[RequestPolicy] Failed to register factory! Details:");
                console.dir(e);
            }
        }
    });
    var unregister = (function () {
        return function () {
            var e_2, _a;
            log_1.log.log("shutting down RPContentPolicy...");
            var finalReturnValue = CP_OK;
            self.shouldLoad = function () { return finalReturnValue; };
            var registrar = Cm.QueryInterface(Ci.nsIComponentRegistrar);
            try {
                for (var XPCOM_CATEGORIES_2 = __values(XPCOM_CATEGORIES), XPCOM_CATEGORIES_2_1 = XPCOM_CATEGORIES_2.next(); !XPCOM_CATEGORIES_2_1.done; XPCOM_CATEGORIES_2_1 = XPCOM_CATEGORIES_2.next()) {
                    var category = XPCOM_CATEGORIES_2_1.value;
                    catMan.deleteCategoryEntry(category, self.contractID, false);
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (XPCOM_CATEGORIES_2_1 && !XPCOM_CATEGORIES_2_1.done && (_a = XPCOM_CATEGORIES_2.return)) _a.call(XPCOM_CATEGORIES_2);
                }
                finally { if (e_2) throw e_2.error; }
            }
            Utils.runAsync(function () {
                registrar.unregisterFactory(self.classID, self);
            });
        };
    })();
    environment_1.MainEnvironment.addShutdownFunction(environment_1.Level.INTERFACE, unregister);
    self.QueryInterface = XPCOMUtils.generateQI([Ci.nsIContentPolicy,
        Ci.nsIObserver,
        Ci.nsIFactory,
        Ci.nsISupportsWeakReference]);
    self.shouldLoad = function (aContentType, aContentLocation, aRequestOrigin, aContext, aMimeTypeGuess, aExtra, aRequestPrincipal) {
        var request = new request_1.NormalRequest(aContentType, aContentLocation, aRequestOrigin, aContext, aMimeTypeGuess, aExtra, aRequestPrincipal);
        return RequestProcessor.process(request);
    };
    self.shouldProcess = function () { return CP_OK; };
    self.createInstance = function (outer, iid) {
        if (outer) {
            throw Cr.NS_ERROR_NO_AGGREGATION;
        }
        return self.QueryInterface(iid);
    };
    return self;
})();
exports.RPChannelEventSink = (function () {
    var self = {};
    var XPCOM_CATEGORIES = ["net-channel-event-sinks"];
    self.classDescription = "RequestPolicy ChannelEventSink Implementation";
    self.classID = ComponentsID("{dc6e2000-2ff0-11e6-bdf4-0800200c9a66}");
    self.contractID = "@requestpolicy.org/net-channel-event-sinks;1";
    function register() {
        var e_3, _a;
        Cm.QueryInterface(Ci.nsIComponentRegistrar).
            registerFactory(self.classID, self.classDescription, self.contractID, self);
        try {
            for (var XPCOM_CATEGORIES_3 = __values(XPCOM_CATEGORIES), XPCOM_CATEGORIES_3_1 = XPCOM_CATEGORIES_3.next(); !XPCOM_CATEGORIES_3_1.done; XPCOM_CATEGORIES_3_1 = XPCOM_CATEGORIES_3.next()) {
                var category = XPCOM_CATEGORIES_3_1.value;
                catMan.addCategoryEntry(category, self.contractID, self.contractID, false, true);
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (XPCOM_CATEGORIES_3_1 && !XPCOM_CATEGORIES_3_1.done && (_a = XPCOM_CATEGORIES_3.return)) _a.call(XPCOM_CATEGORIES_3);
            }
            finally { if (e_3) throw e_3.error; }
        }
    }
    environment_1.MainEnvironment.addStartupFunction(environment_1.Level.INTERFACE, function () {
        try {
            register();
        }
        catch (e) {
            if (e.result === Cr.NS_ERROR_FACTORY_EXISTS) {
                Utils.runAsync(register);
            }
            else {
                console.error("[RequestPolicy] Failed to register factory! Details:");
                console.dir(e);
            }
        }
    });
    var unregister = (function () {
        return function unregister() {
            var e_4, _a;
            log_1.log.log("shutting down RPChannelEventSink...");
            self.asyncOnChannelRedirect = function () { };
            var registrar = Cm.QueryInterface(Ci.nsIComponentRegistrar);
            try {
                for (var XPCOM_CATEGORIES_4 = __values(XPCOM_CATEGORIES), XPCOM_CATEGORIES_4_1 = XPCOM_CATEGORIES_4.next(); !XPCOM_CATEGORIES_4_1.done; XPCOM_CATEGORIES_4_1 = XPCOM_CATEGORIES_4.next()) {
                    var category = XPCOM_CATEGORIES_4_1.value;
                    catMan.deleteCategoryEntry(category, self.contractID, false);
                }
            }
            catch (e_4_1) { e_4 = { error: e_4_1 }; }
            finally {
                try {
                    if (XPCOM_CATEGORIES_4_1 && !XPCOM_CATEGORIES_4_1.done && (_a = XPCOM_CATEGORIES_4.return)) _a.call(XPCOM_CATEGORIES_4);
                }
                finally { if (e_4) throw e_4.error; }
            }
            Utils.runAsync(function () {
                registrar.unregisterFactory(self.classID, self);
            });
        };
    })();
    environment_1.MainEnvironment.addShutdownFunction(environment_1.Level.INTERFACE, unregister);
    self.QueryInterface = XPCOMUtils.generateQI([Ci.nsIChannelEventSink,
        Ci.nsIObserver,
        Ci.nsIFactory,
        Ci.nsISupportsWeakReference]);
    var CES_ACCEPT = Cr.NS_OK;
    var CES_REJECT = Cr.NS_ERROR_FAILURE;
    self.asyncOnChannelRedirect = function (aOldChannel, aNewChannel, aFlags, aCallback) {
        var request = new request_1.RedirectRequest(aOldChannel, aNewChannel, aFlags);
        var rv = RequestProcessor.processUrlRedirection(request);
        var result = rv === CP_REJECT ? CES_REJECT : CES_ACCEPT;
        aCallback.onRedirectVerifyCallback(result);
    };
    self.createInstance = function (outer, iid) {
        if (outer) {
            throw Cr.NS_ERROR_NO_AGGREGATION;
        }
        return self.QueryInterface(iid);
    };
    return self;
})();