"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var environment_1 = require("lib/environment");
var windows_1 = require("models/windows");
var windows_overlay_1 = require("controllers/windows.overlay");
var windows_toolbarbutton_1 = require("controllers/windows.toolbarbutton");
var windows_style_sheets_1 = require("controllers/windows.style-sheets");
var framescript_services_1 = require("main/framescript-services");
var ui_startup_1 = require("models/ui-startup");
var js_utils_1 = require("lib/utils/js-utils");
var app_background_1 = require("app/app.background");
var WindowSubControllers = (function () {
    var self = {};
    var SUBCONTROLLERS = Object.freeze([
        windows_overlay_1.OverlayController,
        windows_toolbarbutton_1.ToolbarButtonController,
        windows_style_sheets_1.StyleSheetsController,
    ]);
    var SUBCONTROLLERS_REVERSE = Object.freeze(SUBCONTROLLERS.slice().reverse());
    function callForEachController(fnName, reverse) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        var controllers = reverse ? SUBCONTROLLERS_REVERSE : SUBCONTROLLERS;
        controllers.forEach(function (controller) {
            if (typeof controller[fnName] === "function") {
                controller[fnName].apply(null, args);
            }
        });
    }
    self.startup = callForEachController.bind(null, "startup", false);
    self.shutdown = callForEachController.bind(null, "shutdown", true);
    self.loadIntoWindow = callForEachController.bind(null, "loadIntoWindow", false);
    self.unloadFromWindow = callForEachController.bind(null, "unloadFromWindow", true);
    return self;
})();
exports.rpWindowManager = (function () {
    var self = {};
    var dStartup = js_utils_1.defer();
    self.pStartup = dStartup.promise;
    var frameScriptURI = "chrome://rpcontinued/content/bootstrap-environments/framescript.js?" + Math.random();
    function loadIntoWindow(window) {
        WindowSubControllers.loadIntoWindow(window);
    }
    function unloadFromWindow(window) {
        WindowSubControllers.unloadFromWindow(window);
    }
    environment_1.MainEnvironment.addStartupFunction(environment_1.Level.INTERFACE, function () {
        Promise.all([
            ui_startup_1.pWindowsAvailable,
            app_background_1.rp.storage.cachedSettings.whenReady,
        ]).then(function () {
            WindowSubControllers.startup();
            windows_1.Windows.forEachOpenWindow(loadIntoWindow);
            windows_1.Windows.addListener("load", loadIntoWindow);
            windows_1.Windows.addListener("unload", unloadFromWindow);
            windows_1.Windows._startListening();
            framescript_services_1.FramescriptServices.init();
            Services.mm.loadFrameScript(frameScriptURI, true);
            dStartup.resolve();
            return;
        }).catch(function (e) {
            console.error("[RequestPolicy] rpWindowManager startup error:");
            console.dir(e);
        });
    });
    environment_1.MainEnvironment.addShutdownFunction(environment_1.Level.INTERFACE, function () {
        Services.mm.removeDelayedFrameScript(frameScriptURI);
        windows_1.Windows._stopListening();
        windows_1.Windows.forEachOpenWindow(unloadFromWindow);
        WindowSubControllers.shutdown();
    });
    return self;
})();