"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var js_utils_1 = require("lib/utils/js-utils");
var ALLOWED_NAMES = new Set([
    "browser:windows-available",
]);
var dataMap = new Map();
function getData(aName) {
    if (!dataMap.has(aName)) {
        dataMap.set(aName, {
            deferred: js_utils_1.defer(),
            registered: false,
        });
    }
    return dataMap.get(aName);
}
function registerAsyncStartup(aName, aPromise) {
    if (!ALLOWED_NAMES.has(aName)) {
        console.error("[RequestPolicy] async startup: Unknown name \"" + aName + "\"");
    }
    var data = getData(aName);
    if (data.registered) {
        console.error("[RequestPolicy] async startup: \"" + aName + "\" is already registered.");
    }
    data.registered = true;
    var _a = data.deferred, resolve = _a.resolve, reject = _a.reject;
    if (aPromise) {
        aPromise.then(resolve).catch(function (e) {
            console.error("[RequestPolicy] async startup: \"" + aName + "\" has been rejected!", e);
            reject(e);
        });
        return;
    }
    return { resolve: resolve, reject: reject };
}
exports.registerAsyncStartup = registerAsyncStartup;
function whenReady() {
    var aDependencies = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        aDependencies[_i] = arguments[_i];
    }
    var e_1, _a;
    var promises = [];
    try {
        for (var aDependencies_1 = __values(aDependencies), aDependencies_1_1 = aDependencies_1.next(); !aDependencies_1_1.done; aDependencies_1_1 = aDependencies_1.next()) {
            var name = aDependencies_1_1.value;
            if (!ALLOWED_NAMES.has(name)) {
                var e = new Error("async startup: \"" + name + "\" is not an allowed name!");
                console.error("[RequestPolicy] " + e);
                return Promise.reject(e);
            }
            promises.push(getData(name).deferred.promise);
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (aDependencies_1_1 && !aDependencies_1_1.done && (_a = aDependencies_1.return)) _a.call(aDependencies_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return Promise.all(promises);
}
exports.whenReady = whenReady;