"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var map_of_sets_1 = require("lib/classes/map-of-sets");
function LoadAndUnloadListener(callback) {
    var windows = new Set();
    function _onEvent(event) {
        var document = event.target;
        var window = document.defaultView;
        var windowtype = document.documentElement.getAttribute("windowtype");
        if (windowtype !== "navigator:browser") {
            _removeEventListener("load", window);
            _removeEventListener("unload", window);
            windows.delete(window);
            return;
        }
        switch (event.type) {
            case "load": {
                _removeEventListener("load", window);
                _addEventListener("unload", window);
                callback.call(null, "load", window);
                break;
            }
            case "unload": {
                _removeEventListener("unload", window);
                windows.delete(window);
                callback.call(null, "unload", window);
                break;
            }
            default:
                break;
        }
    }
    function _addEventListener(eventName, window) {
        window.addEventListener(eventName, _onEvent, false);
    }
    function _removeEventListener(eventName, window) {
        window.removeEventListener(eventName, _onEvent);
    }
    this.listenTo = function (eventName, window) {
        if (eventName !== "load" && eventName !== "unload") {
            throw "incorrect event type!";
        }
        windows.add(window);
        _addEventListener(eventName, window);
    };
    this.stopListeningOnAllWindows = function () {
        var e_1, _a;
        try {
            for (var _b = __values(windows.values()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var window = _c.value;
                _removeEventListener("load", window);
                _removeEventListener("unload", window);
                windows.delete(window);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        windows.clear();
    };
}
exports.Windows = (function () {
    var self = {};
    self.forEachOpenWindow = function (aCallback, aThisArg) {
        if (aThisArg === void 0) { aThisArg = null; }
        var windows = Services.wm.getEnumerator("navigator:browser");
        while (windows.hasMoreElements()) {
            var window = windows.getNext().QueryInterface(Ci.nsIDOMWindow);
            aCallback.call(aThisArg, window);
        }
    };
    return self;
})();
(function (self) {
    var topicsToListeners = new map_of_sets_1.MapOfSets();
    function onEvent(eventType, window) {
        var e_2, _a;
        if (topicsToListeners.has(eventType)) {
            var listeners = __spread(topicsToListeners.get(eventType));
            if (eventType === "unload") {
                listeners.reverse();
            }
            try {
                for (var listeners_1 = __values(listeners), listeners_1_1 = listeners_1.next(); !listeners_1_1.done; listeners_1_1 = listeners_1.next()) {
                    var listener = listeners_1_1.value;
                    listener.call(null, window);
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (listeners_1_1 && !listeners_1_1.done && (_a = listeners_1.return)) _a.call(listeners_1);
                }
                finally { if (e_2) throw e_2.error; }
            }
        }
    }
    var loadAndUnloadListener = new LoadAndUnloadListener(onEvent);
    var windowMediatorListener = {
        onOpenWindow: function (xulWindow) {
            var window = xulWindow.QueryInterface(Ci.nsIInterfaceRequestor).
                getInterface(Ci.nsIDOMWindow);
            loadAndUnloadListener.listenTo("load", window);
        },
        onCloseWindow: function (xulWindow) { },
        onWindowTitleChange: function (xulWindow, newTitle) { },
    };
    self.addListener = function (aEventType, aListener) {
        if (aEventType !== "load" && aEventType !== "unload") {
            throw "incorrect event type!";
        }
        topicsToListeners.addToSet(aEventType, aListener);
    };
    self.removeListener = function (aEventType, aListener) {
        topicsToListeners.deleteFromSet(aEventType, aListener);
    };
    var listening = false;
    self._startListening = function () {
        if (listening === false) {
            Services.wm.addListener(windowMediatorListener);
            listening = true;
            self.forEachOpenWindow(function (window) {
                loadAndUnloadListener.listenTo("unload", window);
            });
        }
    };
    self._stopListening = function () {
        if (listening === true) {
            Services.wm.removeListener(windowMediatorListener);
            listening = false;
            loadAndUnloadListener.stopListeningOnAllWindows();
        }
    };
    return self;
})(exports.Windows);