"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var common_1 = require("./common");
(function () {
    var _a = browser.extension.getBackgroundPage(), rp = _a.rp, RuleUtils = _a.RuleUtils;
    var rules;
    function populateRuleTable() {
        var table = common_1.$id("rules");
        var prefStrings = rp.services.rules.v0.getRPV0PrefStrings();
        rules = rp.services.rules.v0.parse(prefStrings);
        rp.policy.addAllowRules(rules);
        for (var i = 0; i < rules.length; i++) {
            var entry = rules[i];
            var origin = entry.o ?
                RuleUtils.endpointSpecToDisplayString(entry.o) : "";
            var dest = entry.d ?
                RuleUtils.endpointSpecToDisplayString(entry.d) : "";
            addRulesTableRow(table, "allow", origin, dest, entry);
        }
    }
    function addRulesTableRow(table, ruleAction, origin, dest, ruleData) {
        var actionClass;
        var action;
        if (ruleAction === "allow") {
            actionClass = "allow";
            action = browser.i18n.getMessage("allow");
        }
        else {
            actionClass = "block";
            action = browser.i18n.getMessage("block");
        }
        var row = $("<tr>").addClass(actionClass).appendTo(table);
        row.append($("<td>").text(action), $("<td>").text(origin), $("<td>").text(dest));
    }
    window.deleteOldRules = function () {
        rp.services.rules.v0.deleteOldRules();
        $("#doimport").hide();
        $("#deletedone").show();
        $("#showReimportOptions").hide();
        $("#reimportOldRules").hide();
        $("#deleteOldRules").hide();
    };
    window.showReimportOptions = function () {
        $("#showReimportOptions").hide();
        $("#reimportOldRules").show();
    };
    window.importOldRules = function () {
        if (!rules || rules.length === 0) {
            throw new Error("rules is undefined or empty");
        }
        rp.policy.addAllowRules(rules);
        $("#doimport").hide();
        $("#policy").hide();
        $("#importoptions").hide();
        $("#importdone").show();
    };
    window.onload = function () {
        var oldRulesExist = rp.services.rules.v0.oldRulesExist();
        if (!oldRulesExist) {
            $("#hasrules").hide();
            $("#norules").show();
            return;
        }
        populateRuleTable();
    };
})();