"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var common_1 = require("./common");
(function () {
    var _a = browser.extension.getBackgroundPage(), SUBSCRIPTION_ADDED_TOPIC = _a.SUBSCRIPTION_ADDED_TOPIC, SUBSCRIPTION_REMOVED_TOPIC = _a.SUBSCRIPTION_REMOVED_TOPIC, log = _a.log, rp = _a.rp;
    var cachedSettings = rp.storage.cachedSettings;
    function showConfigure() {
        $("#welcome").css("display", "none");
        $("#configure").css("display", "block");
    }
    function handleDefaultPolicyChange() {
        cachedSettings.set({
            "defaultPolicy.allow": common_1.$id("defaultallow").checked,
        }).catch(log.onError("handleDefaultPolicyChange"));
        setAllowSameDomainBlockDisplay();
    }
    function handleAllowSameDomainChange() {
        cachedSettings.set({
            "defaultPolicy.allowSameDomain": common_1.$id("allowsamedomain").checked,
        }).catch(log.onError("handleAllowSameDomainChange"));
    }
    function setAllowSameDomainBlockDisplay() {
        if (common_1.$id("defaultallow").checked) {
            $("#allowsamedomainblock").css("display", "none");
        }
        else {
            $("#allowsamedomainblock").css("display", "block");
        }
    }
    function handleSubscriptionsChange() {
        var enableSubs = common_1.$id("enablesubs").checked;
        var subs = {
            allow_embedded: {},
            allow_extensions: {},
            allow_functionality: {},
            allow_mozilla: {},
            allow_sameorg: {},
            deny_trackers: {},
        };
        var userSubs = rp.policy.subscriptions.getSubscriptions();
        for (var subName in subs) {
            var subInfo = {};
            subInfo.official = {};
            subInfo.official[subName] = true;
            if (enableSubs) {
                userSubs.addSubscription("official", subName);
                Services.obs.notifyObservers(null, SUBSCRIPTION_ADDED_TOPIC, JSON.stringify(subInfo));
            }
            else {
                userSubs.removeSubscription("official", subName);
                Services.obs.notifyObservers(null, SUBSCRIPTION_REMOVED_TOPIC, JSON.stringify(subInfo));
            }
        }
    }
    window.onload = function () {
        if (rp.services.versionInfo.isRPUpgrade) {
            showConfigure();
        }
        var defaultAllow = cachedSettings.get("defaultPolicy.allow");
        common_1.$id("defaultallow").checked = defaultAllow;
        common_1.$id("defaultdeny").checked = !defaultAllow;
        if (!defaultAllow) {
            $("#allowsamedomainblock").css("display", "block");
        }
        common_1.$id("allowsamedomain").checked =
            cachedSettings.get("defaultPolicy.allowSameDomain");
        $("#showconfigure").click(showConfigure);
        $("input[name=defaultpolicy]").change(handleDefaultPolicyChange);
        $("input[name=subscriptions]").change(handleSubscriptionsChange);
        $("#allowsamedomain").change(handleAllowSameDomainChange);
    };
})();