"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var common_1 = require("./common");
(function () {
    var _a = browser.extension.getBackgroundPage(), SUBSCRIPTION_ADDED_TOPIC = _a.SUBSCRIPTION_ADDED_TOPIC, SUBSCRIPTION_REMOVED_TOPIC = _a.SUBSCRIPTION_REMOVED_TOPIC, log = _a.log, rp = _a.rp;
    function getInputElement(subName) {
        var elements = document.body.querySelectorAll("input[name=" + subName + "]");
        if (elements.length <= 0) {
            return null;
        }
        return elements[0];
    }
    function getAllSubscriptionElements() {
        var divs = document.getElementsByClassName("subscription");
        var elements = [];
        for (var i = 0, len = divs.length; i < len; ++i) {
            var div = divs[i];
            elements.push({
                div: div,
                id: div.id,
                input: getInputElement(div.id),
            });
        }
        return elements;
    }
    function updateDisplay() {
        var userSubs = rp.policy.subscriptions.getSubscriptions();
        var subsInfo = userSubs.getSubscriptionInfo();
        var allSubElements = getAllSubscriptionElements();
        for (var i = 0, len = allSubElements.length; i < len; ++i) {
            var element = allSubElements[i];
            element.input.checked = element.id in subsInfo.official;
        }
    }
    function handleSubscriptionCheckboxChange(event) {
        var userSubs = rp.policy.subscriptions.getSubscriptions();
        var subName = event.target.name;
        var enabled = event.target.checked;
        var subInfo = {};
        subInfo.official = {};
        subInfo.official[subName] = true;
        if (enabled) {
            userSubs.addSubscription("official", subName);
            Services.obs.notifyObservers(null, SUBSCRIPTION_ADDED_TOPIC, JSON.stringify(subInfo));
        }
        else {
            userSubs.removeSubscription("official", subName);
            Services.obs.notifyObservers(null, SUBSCRIPTION_REMOVED_TOPIC, JSON.stringify(subInfo));
        }
    }
    window.onload = function () {
        updateDisplay();
        var available = {
            allow_embedded: {},
            allow_extensions: {},
            allow_functionality: {},
            allow_mozilla: {},
            allow_sameorg: {},
            deny_trackers: {},
        };
        for (var subName in available) {
            var el = getInputElement(subName);
            if (!el) {
                log.log("Skipping unexpected official subName: " + subName);
                continue;
            }
            common_1.elManager.addListener(el, "change", handleSubscriptionCheckboxChange);
        }
        common_1.WinEnv.obMan.observe([
            SUBSCRIPTION_ADDED_TOPIC,
            SUBSCRIPTION_REMOVED_TOPIC,
        ], updateDisplay);
    };
})();