"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var common_1 = require("settings/common");
(function () {
    var _a = browser.extension.getBackgroundPage(), C = _a.C, RuleUtils = _a.RuleUtils, rp = _a.rp;
    var $str = browser.i18n.getMessage.bind(browser.i18n);
    $(function () {
        common_1.$id("rulesearch").placeholder = $str("search");
        $("[name=originscheme]").prop("placeholder", $str("scheme"));
        $("[name=destscheme]").prop("placeholder", $str("scheme"));
        $("[name=originhost]").prop("placeholder", $str("host"));
        $("[name=desthost]").prop("placeholder", $str("host"));
        $("[name=originport]").prop("placeholder", $str("port"));
        $("[name=destport]").prop("placeholder", $str("port"));
    });
    var SEARCH_DELAY = 100;
    var searchTimeoutId = null;
    function populateRuleTable(filter) {
        searchTimeoutId = null;
        var table = common_1.$id("rules");
        clearRulesTable(table);
        var entries;
        var user = rp.policy.getUserRulesets().user;
        entries = user.rawRuleset.raw.entries;
        addRules(entries, "User", filter, false);
        var temp = rp.policy.getUserRulesets().temp;
        entries = temp.rawRuleset.raw.entries;
        addRules(entries, "Temporary", filter, false);
        if (!C.UI_TESTING) {
            var subscriptionLists = rp.policy.subscriptions.getRulesets();
            for (var subscriptionList in subscriptionLists) {
                for (var subscription in subscriptionLists[subscriptionList]) {
                    entries = subscriptionLists[subscriptionList][subscription].
                        rawRuleset.raw.entries;
                    addRules(entries, subscription, filter, true);
                }
            }
        }
    }
    function addRules(entries, source, filter, readOnly) {
        var table = $("#rules");
        for (var entryType in entries) {
            for (var i = 0; i < entries[entryType].length; i++) {
                var entry = entries[entryType][i];
                var origin = entry.o ?
                    RuleUtils.endpointSpecToDisplayString(entry.o) : "";
                var dest = entry.d ?
                    RuleUtils.endpointSpecToDisplayString(entry.d) : "";
                if (filter) {
                    if (origin.indexOf(filter) === -1 && dest.indexOf(filter) === -1) {
                        continue;
                    }
                }
                addRulesTableRow(table, entryType, origin, dest, entry, source, readOnly);
            }
        }
    }
    function deleteRule(event) {
        var anchor = $(event.target);
        var ruleAction = anchor.data("requestpolicyRuleAction");
        var ruleData = anchor.data("requestpolicyRuleData");
        if (ruleAction === "allow") {
            rp.policy.removeAllowRule(ruleData);
        }
        else {
            rp.policy.removeDenyRule(ruleData);
        }
        anchor.closest("tr").remove();
    }
    function clearRulesTable(table) {
        var children = table.getElementsByTagName("tr");
        while (children.length) {
            var child = children.item(0);
            child.parentNode.removeChild(child);
        }
    }
    function addRulesTableRow(table, aRuleAction, origin, dest, ruleData, source, readOnly) {
        var ruleAction = aRuleAction === "allow" ? "allow" : "block";
        var ruleActionString = $str(ruleAction);
        var row = $("<tr>").addClass(ruleAction).appendTo(table);
        row.append($("<td>").text(ruleActionString), $("<td>").text(origin), $("<td>").text(dest), $("<td>").text(source));
        if (!readOnly) {
            var anchor = $("<a>");
            anchor.text("x").addClass("deleterule");
            anchor.data("requestpolicyRuleAction", ruleAction);
            anchor.data("requestpolicyRuleData", ruleData);
            anchor.click(deleteRule);
            row.append($("<td>").append(anchor));
        }
        else {
            row.append($("<td>"));
        }
    }
    window.addRule = function () {
        try {
            addRuleHelper();
        }
        catch (e) {
            console.error("[RequestPolicy] yourpolicy: addRule():");
            console.dir(e);
            window.alert("Unable to add rule: " + e.toString());
            return;
        }
    };
    function addRuleHelper() {
        var form = document.forms.addruleform;
        var allow = form.elements.allowrule.checked ? true : false;
        var temporary = form.elements.temporary.checked ? true : false;
        var originScheme = form.elements.originscheme.value;
        var originHost = form.elements.originhost.value;
        var originPort = form.elements.originport.value;
        var destScheme = form.elements.destscheme.value;
        var destHost = form.elements.desthost.value;
        var destPort = form.elements.destport.value;
        function ruleInfoToRuleDataPart(scheme, host, port) {
            if (!scheme && !host && !port) {
                return null;
            }
            var part = {};
            if (scheme) {
                part.s = scheme;
            }
            if (host) {
                part.h = host;
            }
            if (port) {
                part.port = port;
            }
            return part;
        }
        var originPart = ruleInfoToRuleDataPart(originScheme, originHost, originPort);
        var destPart = ruleInfoToRuleDataPart(destScheme, destHost, destPort);
        if (!originPart && !destPart) {
            return;
        }
        var ruleData = {};
        if (originPart) {
            ruleData.o = originPart;
        }
        if (destPart) {
            ruleData.d = destPart;
        }
        if (allow) {
            if (temporary) {
                rp.policy.addTemporaryAllowRule(ruleData);
            }
            else {
                rp.policy.addAllowRule(ruleData);
            }
        }
        else {
            if (temporary) {
                rp.policy.addTemporaryDenyRule(ruleData);
            }
            else {
                rp.policy.addDenyRule(ruleData);
            }
        }
    }
    window.onload = function () {
        {
            var search = common_1.$id("rulesearch");
            common_1.elManager.addListener(search, "keyup", function (event) {
                if (searchTimeoutId !== null) {
                    window.clearTimeout(searchTimeoutId);
                }
                searchTimeoutId = window.setTimeout(function () {
                    populateRuleTable(event.target.value);
                }, SEARCH_DELAY);
            }, false);
            populateRuleTable(search.value);
            if (rp.services.rules.v0.oldRulesExist()) {
                $("#oldrulesexist").show();
            }
        }
        common_1.WinEnv.obMan.observe(["rpcontinued-rules-changed"], function () {
            var search = common_1.$id("rulesearch");
            populateRuleTable(search.value);
        });
    };
})();