"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var DOMUtils = require("lib/utils/dom-utils");
var app_background_1 = require("app/app.background");
function loadClassicmenuIntoWindow(window) {
    var self = {};
    var document = window.document;
    self.emptyMenu = function (menu) {
        var e_1, _a;
        var menuitems = menu.getElementsByTagName("menuitem");
        try {
            for (var menuitems_1 = __values(menuitems), menuitems_1_1 = menuitems_1.next(); !menuitems_1_1.done; menuitems_1_1 = menuitems_1.next()) {
                var item = menuitems_1_1.value;
                if (!item.hasOwnProperty("rpListener")) {
                    console.error("[RequestPolicy] There's a menuitem without listener!");
                    continue;
                }
                item.removeEventListener("command", item.rpListener, false);
                delete item.rpListener;
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (menuitems_1_1 && !menuitems_1_1.done && (_a = menuitems_1.return)) _a.call(menuitems_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        DOMUtils.removeChildren(menu);
    };
    self.addMenuSeparator = function (menu) {
        var separator = document.createElement("menuseparator");
        menu.insertBefore(separator, menu.firstChild);
        return separator;
    };
    self.addCustomMenuItem = function (menu, label, aCallback) {
        var menuItem = document.createElement("menuitem");
        menuItem.setAttribute("label", label);
        menuItem.addEventListener("command", aCallback, false);
        menuItem.rpListener = aCallback;
        menu.insertBefore(menuItem, menu.firstChild);
        return menuItem;
    };
    function getInfoFromRuleSpec(aRuleSpec) {
        var isTemp = "temp" in aRuleSpec && aRuleSpec.temp;
        var isAllow = "allow" in aRuleSpec && aRuleSpec.allow;
        var hasOrigin = "origin" in aRuleSpec && aRuleSpec.origin;
        var hasDest = "dest" in aRuleSpec && aRuleSpec.dest;
        var type = (isTemp ? "t" : "") +
            (isAllow ? "a" : "d") +
            (hasOrigin ? "o" : "") +
            (hasDest ? "d" : "");
        return Object.freeze({ isTemp: isTemp, isAllow: isAllow, hasOrigin: hasOrigin, hasDest: hasDest, type: type });
    }
    self.addMenuItem = function (aMenu, aRuleSpec, aAllowRedirectFn) {
        var _a = getInfoFromRuleSpec(aRuleSpec), isTemp = _a.isTemp, isAllow = _a.isAllow, hasOrigin = _a.hasOrigin, hasDest = _a.hasDest, type = _a.type;
        if (!isAllow) {
            console.error("[RequestPolicy] Invalid addMenuItem rule-spec:");
            console.dir(aRuleSpec);
            return;
        }
        var labelName;
        switch (type) {
            case "tao":
                labelName = "allowOriginTemporarily";
                break;
            case "ao":
                labelName = "allowOrigin";
                break;
            case "taod":
                labelName = "allowOriginToDestinationTemporarily";
                break;
            case "aod":
                labelName = "allowOriginToDestination";
                break;
            case "tad":
                labelName = "allowDestinationTemporarily";
                break;
            case "ad":
                labelName = "allowDestination";
                break;
            default:
                console.error("[RequestPolicy] Invalid addMenuItem rule-spec:");
                console.dir(aRuleSpec);
                return;
        }
        var originAndOrDestArray = [];
        if (hasOrigin) {
            originAndOrDestArray.push(aRuleSpec.origin);
        }
        if (hasDest) {
            originAndOrDestArray.push(aRuleSpec.dest);
        }
        originAndOrDestArray = Object.freeze(originAndOrDestArray);
        var callbackFn = function () {
            app_background_1.rp.policy.addRuleBySpec(aRuleSpec);
            aAllowRedirectFn();
        };
        var label = browser.i18n.getMessage(labelName, originAndOrDestArray);
        var item = self.addCustomMenuItem(aMenu, label, callbackFn);
        if (isTemp) {
            item.setAttribute("class", "rpcontinuedTemporary");
        }
        return item;
    };
    window.rpcontinued.classicmenu = self;
}
exports.loadClassicmenuIntoWindow = loadClassicmenuIntoWindow;