"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var environment_1 = require("lib/environment");
var ruleset_1 = require("lib/ruleset");
var gui_location_1 = require("lib/classes/gui-location");
var DOMUtils = require("lib/utils/dom-utils");
var WindowUtils = require("lib/utils/window-utils");
var constants_1 = require("data/constants");
var requests_1 = require("models/requests");
var app_background_1 = require("app/app.background");
var log_1 = require("app/log");
var uriService = app_background_1.rp.services.uri;
var cachedSettings = app_background_1.rp.storage.cachedSettings;
function loadMenuIntoWindow(window) {
    var document = window.document, rpcontinued = window.rpcontinued;
    var gBrowser = WindowUtils.getTabBrowser(window);
    var initialized = false;
    var lists = {
        otherOrigins: null,
        blockedDestinations: null,
        mixedDestinations: null,
        allowedDestinations: null,
        removeRules: null,
        addRules: null,
    };
    var self = {
        addedMenuItems: [],
        _originItem: null,
        _originDomainnameItem: null,
        _isUncontrollableOrigin: null,
        _originNumRequestsItem: null,
        _isCurrentlySelectedDestBlocked: null,
        _isCurrentlySelectedDestAllowed: null,
        _ruleChangeQueues: {},
    };
    var $id = function (id) {
        var element = null;
        var popupframe = window.top.document.getElementById("rpc-popup-frame");
        if (popupframe && popupframe.contentDocument) {
            element = popupframe.contentDocument.getElementById(id);
        }
        if (!element) {
            element = window.top.document.getElementById(id);
        }
        return element;
    };
    self.init = function () {
        if (initialized === false) {
            initialized = true;
            rpcontinued.overlay.OverlayEnvironment.addShutdownFunction(environment_1.Level.INTERFACE, function () {
                for (var listName in lists) {
                    emptyList(lists[listName]);
                }
            });
        }
    };
    function confirm(dialogMessage, alwaysAskPrefName, params) {
        if (params === void 0) { params = {}; }
        var shouldAsk = cachedSettings.get(alwaysAskPrefName);
        if (shouldAsk === false) {
            return true;
        }
        var dialogTitle = params.dialogTitle || "RequestPolicy";
        var checkboxObj = { value: shouldAsk };
        var checkboxText = browser.i18n.getMessage("alwaysAsk");
        if (typeof params.onBeforeDialog === "function") {
            params.onBeforeDialog.call();
        }
        var confirmed = Services.prompt.confirmCheck(window, dialogTitle, dialogMessage, checkboxText, checkboxObj);
        if (confirmed) {
            cachedSettings.set(alwaysAskPrefName, checkboxObj.value);
            return true;
        }
        return false;
    }
    function emptyList(aList) {
        var e_1, _a;
        if (!aList) {
            return;
        }
        {
            var elements = aList.getElementsByClassName("listen-click");
            try {
                for (var elements_1 = __values(elements), elements_1_1 = elements_1.next(); !elements_1_1.done; elements_1_1 = elements_1.next()) {
                    var el = elements_1_1.value;
                    el.removeEventListener("click", self.itemSelected, false);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (elements_1_1 && !elements_1_1.done && (_a = elements_1.return)) _a.call(elements_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
        }
        DOMUtils.removeChildren(aList);
    }
    self.prepareMenu = function () {
        try {
            self._originItem = $id("rpc-origin");
            self._originDomainnameItem = $id("rpc-origin-domainname");
            self._originNumRequestsItem = $id("rpc-origin-num-requests");
            lists.otherOrigins = $id("rpc-other-origins-list");
            lists.blockedDestinations = $id("rpc-blocked-destinations-list");
            lists.mixedDestinations = $id("rpc-mixed-destinations-list");
            lists.allowedDestinations = $id("rpc-allowed-destinations-list");
            lists.addRules = $id("rpc-rules-add");
            lists.removeRules = $id("rpc-rules-remove");
            var disabled = cachedSettings.alias.isBlockingDisabled();
            $id("rpc-link-enable-blocking").hidden = !disabled;
            $id("rpc-link-disable-blocking").hidden = disabled;
            var tempPermLink = $id("rpc-revoke-temporary-permissions");
            if (app_background_1.rp.policy.temporaryRulesExist()) {
                tempPermLink.className = "rpc-revoke-temporary-permissions-enable";
            }
            else {
                tempPermLink.className = "rpc-revoke-temporary-permissions-disable";
            }
            self._currentUri = rpcontinued.overlay.getTopLevelDocumentUri();
            try {
                self._currentBaseDomain = uriService.getBaseDomain(self._currentUri);
                if (self._currentBaseDomain === null) {
                    log_1.log.info("" + "Unable to prepare menu because " +
                        "the current uri has no host: " + self._currentUri);
                    self._populateMenuForUncontrollableOrigin();
                    return;
                }
            }
            catch (e) {
                log_1.log.info("" + "Unable to prepare menu because " +
                    "base domain can't be determined: " + self._currentUri);
                self._populateMenuForUncontrollableOrigin();
                return;
            }
            self._currentIdentifier = rpcontinued.overlay.
                getTopLevelDocumentUriIdentifier();
            self._currentUriObj = uriService.getUriObject(self._currentUri);
            self._isChromeUri = self._currentUriObj.scheme === "chrome";
            log_1.log.info("self._currentUri: " + self._currentUri);
            if (self._isChromeUri) {
                self._populateMenuForUncontrollableOrigin();
                return;
            }
            self._allRequestsOnDocument = requests_1.Requests.
                getAllRequestsInBrowser(gBrowser.selectedBrowser);
            self._allRequestsOnDocument.print("_allRequestsOnDocument");
            self._setPrivateBrowsingStyles();
            self._populateOrigin();
            self._populateOtherOrigins();
            self._activateOriginItem(self._originItem);
        }
        catch (e) {
            console.error("[RequestPolicy] [Fatal] Unable to prepare menu! Details:");
            console.dir(e);
            throw e;
        }
    };
    self.close = function () {
        $id("rpc-popup").hidePopup();
    };
    self._populateMenuForUncontrollableOrigin = function () {
        self._originDomainnameItem.textContent =
            browser.i18n.getMessage("noOrigin");
        self._isUncontrollableOrigin = true;
        self._originNumRequestsItem.textContent = "";
        self._originItem.removeAttribute("default-policy");
        self._originItem.removeAttribute("requests-blocked");
        [
            lists.otherOrigins,
            lists.blockedDestinations,
            lists.mixedDestinations,
            lists.allowedDestinations,
            lists.removeRules,
            lists.addRules,
        ].forEach(emptyList);
        $id("rpc-other-origins").hidden = true;
        $id("rpc-blocked-destinations").hidden = true;
        $id("rpc-mixed-destinations").hidden = true;
        $id("rpc-allowed-destinations").hidden = true;
    };
    self._populateList = function (list, values) {
        emptyList(list);
        var guiLocations = values[0] && values[0] instanceof gui_location_1.GUILocation;
        if (true === guiLocations) {
            var sorting = cachedSettings.get("menu.sorting");
            var showNumRequests = cachedSettings.get("menu.info.showNumRequests");
            if (sorting === "numRequests") {
                values.sort(gui_location_1.GUILocation.sortByNumRequestsCompareFunction);
            }
            else if (sorting === "destName") {
                values.sort(gui_location_1.GUILocation.compareFunction);
            }
            for (var i in values) {
                var guiLocation = values[i];
                var props = guiLocation.properties;
                var num = void 0;
                if (true === showNumRequests) {
                    num = props.numRequests;
                    if (props.numAllowedRequests > 0 && props.numBlockedRequests > 0) {
                        num += " (" + props.numBlockedRequests + "+" + props.numAllowedRequests + ")";
                    }
                }
                var newitem = self._addListItem(list, "rpc-od-item", guiLocation, num);
                newitem.setAttribute("default-policy", props.numDefaultPolicyRequests > 0 ? "true" : "false");
                newitem.setAttribute("requests-blocked", props.numBlockedRequests > 0 ? "true" : "false");
            }
        }
        else {
            values.sort();
            for (var i in values) {
                self._addListItem(list, "rpc-od-item", values[i]);
            }
        }
    };
    self._populateOrigin = function () {
        self._originDomainnameItem.textContent = self._currentBaseDomain;
        self._isUncontrollableOrigin = false;
        var showNumRequests = cachedSettings.get("menu.info.showNumRequests");
        var props = self._getOriginGUILocationProperties();
        var numRequests = "";
        if (true === showNumRequests) {
            if (props.numAllowedRequests > 0 && props.numBlockedRequests > 0) {
                numRequests = props.numRequests + "\u00A0(" +
                    (props.numBlockedRequests + "+" + props.numAllowedRequests + ")");
            }
            else {
                numRequests = props.numRequests;
            }
        }
        self._originNumRequestsItem.textContent = numRequests;
        self._originItem.setAttribute("default-policy", props.numDefaultPolicyRequests > 0 ? "true" : "false");
        self._originItem.setAttribute("requests-blocked", props.numBlockedRequests > 0 ? "true" : "false");
    };
    self._populateOtherOrigins = function () {
        var guiOrigins = self._getOtherOriginsAsGUILocations();
        self._populateList(lists.otherOrigins, guiOrigins);
        $id("rpc-other-origins").hidden = guiOrigins.length === 0;
    };
    self._populateDestinations = function (originIdentifier) {
        var destsWithBlockedRequests = self._getBlockedDestinationsAsGUILocations();
        var destsWithAllowedRequests = self._getAllowedDestinationsAsGUILocations();
        var destsWithSolelyBlockedRequests = [];
        var destsMixed = [];
        var destsWithSolelyAllowedRequests = [];
        for (var i = 0; i < destsWithBlockedRequests.length; i++) {
            var blockedGUIDest = destsWithBlockedRequests[i];
            if (false === gui_location_1.GUILocation.existsInArray(blockedGUIDest, destsWithAllowedRequests)) {
                destsWithSolelyBlockedRequests.push(blockedGUIDest);
            }
            else {
                destsMixed.push(blockedGUIDest);
            }
        }
        for (var i = 0; i < destsWithAllowedRequests.length; i++) {
            var allowedGUIDest = destsWithAllowedRequests[i];
            var indexRawBlocked = gui_location_1.GUIDestination.
                indexOfDestInArray(allowedGUIDest, destsWithBlockedRequests);
            var destsMixedIndex = gui_location_1.GUIDestination.
                indexOfDestInArray(allowedGUIDest, destsMixed);
            if (indexRawBlocked === -1) {
                destsWithSolelyAllowedRequests.push(allowedGUIDest);
            }
            else {
                if (destsMixedIndex !== -1) {
                    log_1.log.info("Merging dest: <" + allowedGUIDest + ">");
                    destsMixed[destsMixedIndex] = gui_location_1.GUIDestination.merge(allowedGUIDest, destsMixed[destsMixedIndex]);
                }
                else {
                    log_1.log.warn("" + "mixed dest was" +
                        " not added to `destsMixed` list: <" + allowedGUIDest + ">");
                    destsMixed.push(allowedGUIDest);
                }
            }
        }
        self._populateList(lists.blockedDestinations, destsWithSolelyBlockedRequests);
        $id("rpc-blocked-destinations").hidden =
            destsWithSolelyBlockedRequests.length === 0;
        self._populateList(lists.mixedDestinations, destsMixed);
        $id("rpc-mixed-destinations").hidden = destsMixed.length === 0;
        self._populateList(lists.allowedDestinations, destsWithSolelyAllowedRequests);
        $id("rpc-allowed-destinations").hidden =
            destsWithSolelyAllowedRequests.length === 0;
    };
    self._populateDetails = function () {
        var origin = self._currentlySelectedOrigin;
        var dest = self._currentlySelectedDest;
        emptyList(lists.removeRules);
        emptyList(lists.addRules);
        var ruleData = {
            "o": {
                "h": self._addWildcard(origin),
            },
        };
        var mayPermRulesBeAdded = WindowUtils.mayPermanentRulesBeAdded(window, cachedSettings);
        if (!self._currentlySelectedDest) {
            if (cachedSettings.alias.isDefaultAllow()) {
                if (mayPermRulesBeAdded === true) {
                    self._addMenuItemDenyOrigin(lists.addRules, ruleData);
                }
                self._addMenuItemTempDenyOrigin(lists.addRules, ruleData);
            }
            else {
                if (mayPermRulesBeAdded === true) {
                    self._addMenuItemAllowOrigin(lists.addRules, ruleData);
                }
                self._addMenuItemTempAllowOrigin(lists.addRules, ruleData);
            }
        }
        if (dest) {
            ruleData.d = {
                "h": self._addWildcard(dest),
            };
            var destOnlyRuleData = {
                "d": {
                    "h": self._addWildcard(dest),
                },
            };
            if (self._isCurrentlySelectedDestAllowed ||
                !app_background_1.rp.policy.ruleExists(constants_1.C.RULE_ACTION_DENY, ruleData) &&
                    !app_background_1.rp.policy.ruleExists(constants_1.C.RULE_ACTION_DENY, destOnlyRuleData)) {
                if (!app_background_1.rp.policy.ruleExists(constants_1.C.RULE_ACTION_ALLOW, ruleData) &&
                    !app_background_1.rp.policy.ruleExists(constants_1.C.RULE_ACTION_DENY, ruleData)) {
                    if (mayPermRulesBeAdded === true) {
                        self._addMenuItemDenyOriginToDest(lists.addRules, ruleData);
                    }
                    self._addMenuItemTempDenyOriginToDest(lists.addRules, ruleData);
                }
                if (!app_background_1.rp.policy.ruleExists(constants_1.C.RULE_ACTION_ALLOW, destOnlyRuleData) &&
                    !app_background_1.rp.policy.ruleExists(constants_1.C.RULE_ACTION_DENY, destOnlyRuleData)) {
                    if (mayPermRulesBeAdded === true) {
                        self._addMenuItemDenyDest(lists.addRules, destOnlyRuleData);
                    }
                    self._addMenuItemTempDenyDest(lists.addRules, destOnlyRuleData);
                }
            }
            if (self._isCurrentlySelectedDestBlocked ||
                !app_background_1.rp.policy.ruleExists(constants_1.C.RULE_ACTION_ALLOW, ruleData) &&
                    !app_background_1.rp.policy.ruleExists(constants_1.C.RULE_ACTION_ALLOW, destOnlyRuleData)) {
                if (!app_background_1.rp.policy.ruleExists(constants_1.C.RULE_ACTION_ALLOW, ruleData) &&
                    !app_background_1.rp.policy.ruleExists(constants_1.C.RULE_ACTION_DENY, ruleData)) {
                    if (mayPermRulesBeAdded === true) {
                        self._addMenuItemAllowOriginToDest(lists.addRules, ruleData);
                    }
                    self._addMenuItemTempAllowOriginToDest(lists.addRules, ruleData);
                }
                if (!app_background_1.rp.policy.ruleExists(constants_1.C.RULE_ACTION_ALLOW, destOnlyRuleData) &&
                    !app_background_1.rp.policy.ruleExists(constants_1.C.RULE_ACTION_DENY, destOnlyRuleData)) {
                    if (mayPermRulesBeAdded === true) {
                        self._addMenuItemAllowDest(lists.addRules, destOnlyRuleData);
                    }
                    self._addMenuItemTempAllowDest(lists.addRules, destOnlyRuleData);
                }
            }
        }
        if (self._currentlySelectedDest) {
            if (!cachedSettings.alias.isDefaultAllow() &&
                !cachedSettings.alias.isDefaultAllowSameDomain()) {
                self._populateDetailsAddSubdomainAllowRules(lists.addRules);
            }
        }
        self._populateDetailsRemoveAllowRules(lists.removeRules);
        self._populateDetailsRemoveDenyRules(lists.removeRules);
    };
    self._addListItem = function (list, cssClass, value, numRequests) {
        var box = document.createElement("div");
        box.setAttribute("class", cssClass + " listen-click");
        box.addEventListener("click", self.itemSelected, false);
        list.insertBefore(box, null);
        var destLabel = document.createElement("span");
        destLabel.textContent = value;
        destLabel.setAttribute("class", "domainname");
        box.insertBefore(destLabel, null);
        if (numRequests) {
            var numReqLabel = document.createElement("span");
            numReqLabel.textContent = numRequests;
            numReqLabel.setAttribute("class", "numRequests");
            box.insertBefore(numReqLabel, null);
        }
        return box;
    };
    self._disableIfNoChildren = function (el) {
        el.hidden = el.firstChild ? false : true;
    };
    self._setPrivateBrowsingStyles = function () {
        var mayPermRulesBeAdded = WindowUtils.mayPermanentRulesBeAdded(window, cachedSettings);
        var val = mayPermRulesBeAdded === true ? "" : "privatebrowsing";
        $id("rpc-details").setAttribute("class", val);
    };
    self._resetSelectedOrigin = function () {
        self._originItem.setAttribute("selected-origin", "false");
        for (var i = 0; i < lists.otherOrigins.childNodes.length; i++) {
            var child = lists.otherOrigins.childNodes[i];
            child.setAttribute("selected-origin", "false");
        }
    };
    self._resetSelectedDest = function () {
        for (var i = 0; i < lists.blockedDestinations.childNodes.length; i++) {
            var child = lists.blockedDestinations.childNodes[i];
            child.setAttribute("selected-dest", "false");
        }
        for (var i = 0; i < lists.mixedDestinations.childNodes.length; i++) {
            var child = lists.mixedDestinations.childNodes[i];
            child.setAttribute("selected-dest", "false");
        }
        for (var i = 0; i < lists.allowedDestinations.childNodes.length; i++) {
            var child = lists.allowedDestinations.childNodes[i];
            child.setAttribute("selected-dest", "false");
        }
    };
    self._activateOriginItem = function (item) {
        if (item.id === "rpc-origin") {
            if (self._isUncontrollableOrigin) {
                return;
            }
            self._currentlySelectedOrigin = self._originDomainnameItem.textContent;
        }
        else if (item.parentNode.id === "rpc-other-origins-list") {
            self._currentlySelectedOrigin =
                item.getElementsByClassName("domainname")[0].textContent;
        }
        self._currentlySelectedDest = null;
        self._resetSelectedOrigin();
        item.setAttribute("selected-origin", "true");
        self._populateDestinations();
        self._resetSelectedDest();
        self._populateDetails();
    };
    self._activateDestinationItem = function (item) {
        self._currentlySelectedDest =
            item.getElementsByClassName("domainname")[0].textContent;
        if (item.parentNode.id === "rpc-blocked-destinations-list") {
            self._isCurrentlySelectedDestBlocked = true;
            self._isCurrentlySelectedDestAllowed = false;
        }
        else if (item.parentNode.id === "rpc-allowed-destinations-list") {
            self._isCurrentlySelectedDestBlocked = false;
            self._isCurrentlySelectedDestAllowed = true;
        }
        else {
            self._isCurrentlySelectedDestBlocked = true;
            self._isCurrentlySelectedDestAllowed = true;
        }
        self._resetSelectedDest();
        item.setAttribute("selected-dest", "true");
        self._populateDetails();
    };
    function openSiteInfoTab(domain) {
        var url = "https://www.mywot.com/en/scorecard/" + domain;
        window.openUILinkIn(url, "tab", {
            relatedToCurrent: true,
            inBackground: true,
        });
    }
    function maybeOpenSiteInfoTab(item) {
        var domain = null;
        if (item.value) {
            domain = item.value;
        }
        else {
            var domainLabel = item.querySelector(".domainname");
            if (domainLabel !== null) {
                domain = domainLabel.textContent;
            }
        }
        if (domain === null) {
            console.error("[RequestPolicy] Failed to determine the domain under the mouse button " +
                "after the middle-click.");
            return;
        }
        var dialogMessage = browser.i18n.getMessage("siteInfoConfirm", [domain, "https://www.mywot.com"]);
        var alwaysAskPrefName = "confirmSiteInfo";
        var confirmed = confirm(dialogMessage, alwaysAskPrefName, {
            onBeforeDialog: self.close,
        });
        if (confirmed) {
            openSiteInfoTab(domain);
        }
    }
    self.itemSelected = function (event) {
        var item = event.currentTarget;
        if (item.id === "rpc-origin" ||
            item.parentNode.id === "rpc-other-origins-list") {
            if (event.button === 1) {
                maybeOpenSiteInfoTab(item);
            }
            else {
                self._activateOriginItem(item);
            }
        }
        else if (item.parentNode.id === "rpc-blocked-destinations-list" ||
            item.parentNode.id === "rpc-mixed-destinations-list" ||
            item.parentNode.id === "rpc-allowed-destinations-list") {
            if (event.button === 1) {
                maybeOpenSiteInfoTab(item);
            }
            else {
                self._activateDestinationItem(item);
            }
        }
        else if (item.parentNode.id === "rpc-rules-remove" ||
            item.parentNode.id === "rpc-rules-add") {
            self._processRuleSelection(item);
        }
        else {
            console.error("[RequestPolicy] Unable to figure out which item type was selected.");
        }
    };
    self._processRuleSelection = function (item) {
        var ruleData = item.requestpolicyRuleData;
        var ruleAction = item.requestpolicyRuleAction;
        var undo;
        if (item.getAttribute("selected-rule") === "true") {
            item.setAttribute("selected-rule", "false");
            undo = true;
        }
        else {
            item.setAttribute("selected-rule", "true");
            undo = false;
        }
        if (!ruleData) {
            console.error("[RequestPolicy] ruleData is empty in menu._processRuleSelection()");
            return;
        }
        if (!ruleAction) {
            console.error("[RequestPolicy] ruleAction is empty in menu._processRuleSelection()");
            return;
        }
        var canonicalRule = ruleset_1.Ruleset.rawRuleToCanonicalString(ruleData);
        log_1.log.log("ruleData: " + canonicalRule);
        log_1.log.log("ruleAction: " + ruleAction);
        log_1.log.log("undo: " + undo);
        var origin;
        var dest;
        if (ruleData.o && ruleData.o.h) {
            origin = ruleData.o.h;
        }
        if (ruleData.d && ruleData.d.h) {
            dest = ruleData.d.h;
        }
        if (!self._ruleChangeQueues[ruleAction]) {
            self._ruleChangeQueues[ruleAction] = {};
        }
        if (undo) {
            delete self._ruleChangeQueues[ruleAction][canonicalRule];
        }
        else {
            self._ruleChangeQueues[ruleAction][canonicalRule] = ruleData;
        }
    };
    self.processQueuedRuleChanges = function () {
        var rulesChanged = false;
        for (var ruleAction in self._ruleChangeQueues) {
            for (var canonicalRule in self._ruleChangeQueues[ruleAction]) {
                var ruleData = self._ruleChangeQueues[ruleAction][canonicalRule];
                self._processRuleChange(ruleAction, ruleData);
                rulesChanged = true;
            }
        }
        self._ruleChangeQueues = {};
        return rulesChanged;
    };
    self._processRuleChange = function (ruleAction, ruleData) {
        switch (ruleAction) {
            case "allow":
                app_background_1.rp.policy.addAllowRule(ruleData);
                break;
            case "allow-temp":
                app_background_1.rp.policy.addTemporaryAllowRule(ruleData);
                break;
            case "stop-allow":
                app_background_1.rp.policy.removeAllowRule(ruleData);
                break;
            case "deny":
                app_background_1.rp.policy.addDenyRule(ruleData);
                break;
            case "deny-temp":
                app_background_1.rp.policy.addTemporaryDenyRule(ruleData);
                break;
            case "stop-deny":
                app_background_1.rp.policy.removeDenyRule(ruleData);
                break;
            default:
                throw "action not implemented: " + ruleAction;
        }
    };
    self._getBlockedDestinationsAsGUILocations = function () {
        var reqSet = requests_1.Requests.getDeniedRequests(self._currentlySelectedOrigin, self._allRequestsOnDocument);
        var requests = reqSet.getAllMergedOrigins();
        var result = [];
        for (var destBase in requests) {
            var properties = new gui_location_1.GUILocationProperties();
            properties.accumulate(requests[destBase], constants_1.C.RULE_ACTION_DENY);
            result.push(new gui_location_1.GUIDestination(destBase, properties));
        }
        return result;
    };
    self._getAllowedDestinationsAsGUILocations = function () {
        var reqSet = requests_1.Requests.getAllowedRequests(self._currentlySelectedOrigin, self._allRequestsOnDocument);
        var requests = reqSet.getAllMergedOrigins();
        var result = [];
        for (var destBase in requests) {
            if (cachedSettings.alias.isDefaultAllow() ||
                cachedSettings.alias.isDefaultAllowSameDomain()) {
                if (destBase === self._currentlySelectedOrigin) {
                    continue;
                }
            }
            var properties = new gui_location_1.GUILocationProperties();
            properties.accumulate(requests[destBase], constants_1.C.RULE_ACTION_ALLOW);
            result.push(new gui_location_1.GUIDestination(destBase, properties));
        }
        return result;
    };
    self._getOriginGUILocationProperties = function () {
        var allRequests = self._allRequestsOnDocument.getAll();
        var properties = new gui_location_1.GUILocationProperties();
        for (var originUri in allRequests) {
            var originBase = uriService.getBaseDomain(originUri);
            if (originBase !== self._currentBaseDomain) {
                continue;
            }
            for (var destBase in allRequests[originUri]) {
                properties.accumulate(allRequests[originUri][destBase]);
            }
        }
        return properties;
    };
    self._getOtherOriginsAsGUILocations = function () {
        var allRequests = self._allRequestsOnDocument.getAll();
        var allowSameDomain = cachedSettings.alias.isDefaultAllow() ||
            cachedSettings.alias.isDefaultAllowSameDomain();
        var guiOrigins = [];
        for (var originUri in allRequests) {
            var originBase = uriService.getBaseDomain(originUri);
            if (originBase === self._currentBaseDomain) {
                continue;
            }
            var guiOriginsIndex = gui_location_1.GUIOrigin.indexOfOriginInArray(originBase, guiOrigins);
            var properties = void 0;
            if (guiOriginsIndex === -1) {
                properties = new gui_location_1.GUILocationProperties();
            }
            else {
                properties = guiOrigins[guiOriginsIndex].properties;
            }
            var addThisOriginBase = false;
            for (var destBase in allRequests[originUri]) {
                if (allowSameDomain && destBase === originBase) {
                    continue;
                }
                addThisOriginBase = true;
                properties.accumulate(allRequests[originUri][destBase]);
            }
            if (addThisOriginBase && guiOriginsIndex === -1) {
                guiOrigins.push(new gui_location_1.GUIOrigin(originBase, properties));
            }
        }
        return guiOrigins;
    };
    self._isIPAddressOrSingleName = function (hostname) {
        return uriService.isIPAddress(hostname) ||
            hostname.indexOf(".") === -1;
    };
    self._addWildcard = function (hostname) {
        if (self._isIPAddressOrSingleName(hostname)) {
            return hostname;
        }
        else {
            return "*." + hostname;
        }
    };
    self._addMenuItemStopAllowingOrigin = function (list, ruleData, subscriptionOverride) {
        var originHost = ruleData.o.h;
        var ruleAction = subscriptionOverride ? "deny" : "stop-allow";
        return self._addMenuItemHelper(list, ruleData, "stopAllowingOrigin", [originHost], ruleAction, "rpc-stop-rule rpc-stop-allow");
    };
    self._addMenuItemStopAllowingDest = function (list, ruleData, subscriptionOverride) {
        var destHost = ruleData.d.h;
        var ruleAction = subscriptionOverride ? "deny" : "stop-allow";
        return self._addMenuItemHelper(list, ruleData, "stopAllowingDestination", [destHost], ruleAction, "rpc-stop-rule rpc-stop-allow");
    };
    self._addMenuItemStopAllowingOriginToDest = function (list, ruleData, subscriptionOverride) {
        var originHost = ruleData.o.h;
        var destHost = ruleData.d.h;
        var ruleAction = subscriptionOverride ? "deny" : "stop-allow";
        return self._addMenuItemHelper(list, ruleData, "stopAllowingOriginToDestination", [originHost, destHost], ruleAction, "rpc-stop-rule rpc-stop-allow");
    };
    self._addMenuItemAllowOrigin = function (list, ruleData) {
        var originHost = ruleData.o.h;
        return self._addMenuItemHelper(list, ruleData, "allowOrigin", [originHost], "allow", "rpc-start-rule rpc-allow");
    };
    self._addMenuItemAllowDest = function (list, ruleData) {
        var destHost = ruleData.d.h;
        return self._addMenuItemHelper(list, ruleData, "allowDestination", [destHost], "allow", "rpc-start-rule rpc-allow");
    };
    self._addMenuItemAllowOriginToDest = function (list, ruleData) {
        var originHost = ruleData.o.h;
        var destHost = ruleData.d.h;
        return self._addMenuItemHelper(list, ruleData, "allowOriginToDestination", [originHost, destHost], "allow", "rpc-start-rule rpc-allow");
    };
    self._addMenuItemTempAllowOrigin = function (list, ruleData) {
        var originHost = ruleData.o.h;
        return self._addMenuItemHelper(list, ruleData, "allowOriginTemporarily", [originHost], "allow-temp", "rpc-start-rule rpc-allow rpc-temporary");
    };
    self._addMenuItemTempAllowDest = function (list, ruleData) {
        var destHost = ruleData.d.h;
        return self._addMenuItemHelper(list, ruleData, "allowDestinationTemporarily", [destHost], "allow-temp", "rpc-start-rule rpc-allow rpc-temporary");
    };
    self._addMenuItemTempAllowOriginToDest = function (list, ruleData) {
        var originHost = ruleData.o.h;
        var destHost = ruleData.d.h;
        return self._addMenuItemHelper(list, ruleData, "allowOriginToDestinationTemporarily", [originHost, destHost], "allow-temp", "rpc-start-rule rpc-allow rpc-temporary");
    };
    self._addMenuItemStopDenyingOrigin = function (list, ruleData, subscriptionOverride) {
        var originHost = ruleData.o.h;
        var ruleAction = subscriptionOverride ? "allow" : "stop-deny";
        return self._addMenuItemHelper(list, ruleData, "stopDenyingOrigin", [originHost], ruleAction, "rpc-stop-rule rpc-stop-deny");
    };
    self._addMenuItemStopDenyingDest = function (list, ruleData, subscriptionOverride) {
        var destHost = ruleData.d.h;
        var ruleAction = subscriptionOverride ? "allow" : "stop-deny";
        return self._addMenuItemHelper(list, ruleData, "stopDenyingDestination", [destHost], ruleAction, "rpc-stop-rule rpc-stop-deny");
    };
    self._addMenuItemStopDenyingOriginToDest = function (list, ruleData, subscriptionOverride) {
        var originHost = ruleData.o.h;
        var destHost = ruleData.d.h;
        var ruleAction = subscriptionOverride ? "allow" : "stop-deny";
        return self._addMenuItemHelper(list, ruleData, "stopDenyingOriginToDestination", [originHost, destHost], ruleAction, "rpc-stop-rule rpc-stop-deny");
    };
    self._addMenuItemDenyOrigin = function (list, ruleData) {
        var originHost = ruleData.o.h;
        return self._addMenuItemHelper(list, ruleData, "denyOrigin", [originHost], "deny", "rpc-start-rule rpc-deny");
    };
    self._addMenuItemDenyDest = function (list, ruleData) {
        var destHost = ruleData.d.h;
        return self._addMenuItemHelper(list, ruleData, "denyDestination", [destHost], "deny", "rpc-start-rule rpc-deny");
    };
    self._addMenuItemDenyOriginToDest = function (list, ruleData) {
        var originHost = ruleData.o.h;
        var destHost = ruleData.d.h;
        return self._addMenuItemHelper(list, ruleData, "denyOriginToDestination", [originHost, destHost], "deny", "rpc-start-rule rpc-deny");
    };
    self._addMenuItemTempDenyOrigin = function (list, ruleData) {
        var originHost = ruleData.o.h;
        return self._addMenuItemHelper(list, ruleData, "denyOriginTemporarily", [originHost], "deny-temp", "rpc-start-rule rpc-deny rpc-temporary");
    };
    self._addMenuItemTempDenyDest = function (list, ruleData) {
        var destHost = ruleData.d.h;
        return self._addMenuItemHelper(list, ruleData, "denyDestinationTemporarily", [destHost], "deny-temp", "rpc-start-rule rpc-deny rpc-temporary");
    };
    self._addMenuItemTempDenyOriginToDest = function (list, ruleData) {
        var originHost = ruleData.o.h;
        var destHost = ruleData.d.h;
        return self._addMenuItemHelper(list, ruleData, "denyOriginToDestinationTemporarily", [originHost, destHost], "deny-temp", "rpc-start-rule rpc-deny rpc-temporary");
    };
    self._addMenuItemHelper = function (list, ruleData, fmtStrName, fmtStrArgs, ruleAction, cssClass) {
        var label = browser.i18n.getMessage(fmtStrName, fmtStrArgs);
        var item = self._addListItem(list, "rpc-od-item", label);
        item.requestpolicyRuleData = ruleData;
        item.requestpolicyRuleAction = ruleAction;
        item.setAttribute("class", "rpc-od-item " + cssClass);
        var canonicalRule = ruleset_1.Ruleset.rawRuleToCanonicalString(ruleData);
        if (self._ruleChangeQueues[ruleAction]) {
            if (self._ruleChangeQueues[ruleAction][canonicalRule]) {
                item.setAttribute("selected-rule", "true");
            }
        }
        return item;
    };
    self._addMenuItemRemoveAllowRule = function (list, rawRule, subscriptionOverride) {
        if (rawRule.o && rawRule.d) {
            return self._addMenuItemStopAllowingOriginToDest(list, rawRule, subscriptionOverride);
        }
        else if (rawRule.o) {
            return self._addMenuItemStopAllowingOrigin(list, rawRule, subscriptionOverride);
        }
        else if (rawRule.d) {
            return self._addMenuItemStopAllowingDest(list, rawRule, subscriptionOverride);
        }
        else {
            throw "Invalid rule data: no origin or destination parts.";
        }
    };
    self._addMenuItemRemoveDenyRule = function (list, rawRule, subscriptionOverride) {
        if (rawRule.o && rawRule.d) {
            return self._addMenuItemStopDenyingOriginToDest(list, rawRule, subscriptionOverride);
        }
        else if (rawRule.o) {
            return self._addMenuItemStopDenyingOrigin(list, rawRule, subscriptionOverride);
        }
        else if (rawRule.d) {
            return self._addMenuItemStopDenyingDest(list, rawRule, subscriptionOverride);
        }
        else {
            throw "Invalid rule data: no origin or destination parts.";
        }
    };
    self._populateDetailsRemoveAllowRules = function (list) {
        var reqSet = requests_1.Requests.getAllowedRequests(self._currentlySelectedOrigin, self._allRequestsOnDocument);
        var requests = reqSet.getAllMergedOrigins();
        var userRules = {};
        var subscriptionRules = {};
        for (var destBase in requests) {
            if (self._currentlySelectedDest &&
                self._currentlySelectedDest !== destBase) {
                continue;
            }
            for (var destIdent in requests[destBase]) {
                var destinations = requests[destBase][destIdent];
                for (var destUri in destinations) {
                    if (!destinations[destUri]) {
                        log_1.log.log("" + "destinations[destUri] is null or undefined for " +
                            "destUri: " + destUri);
                        continue;
                    }
                    var results = destinations[destUri][0];
                    for (var i in results.matchedAllowRules) {
                        var _a = __read(results.matchedAllowRules[i], 2), ruleset = _a[0], match = _a[1];
                        var rawRule = ruleset_1.Ruleset.matchToRawRule(match);
                        if (!self._currentlySelectedDest) {
                            if (rawRule.d && rawRule.d.h) {
                                continue;
                            }
                        }
                        var rawRuleStr = ruleset_1.Ruleset.rawRuleToCanonicalString(rawRule);
                        if (ruleset.userRuleset) {
                            userRules[rawRuleStr] = rawRule;
                        }
                        else {
                            subscriptionRules[rawRuleStr] = rawRule;
                        }
                    }
                }
            }
        }
        for (var i in userRules) {
            self._addMenuItemRemoveAllowRule(list, userRules[i], false);
        }
        for (var i in subscriptionRules) {
            self._addMenuItemRemoveAllowRule(list, subscriptionRules[i], true);
        }
    };
    self._populateDetailsRemoveDenyRules = function (list) {
        var reqSet = requests_1.Requests.getDeniedRequests(self._currentlySelectedOrigin, self._allRequestsOnDocument);
        var requests = reqSet.getAllMergedOrigins();
        var userRules = {};
        var subscriptionRules = {};
        reqSet.print("deniedRequests");
        for (var destBase in requests) {
            if (self._currentlySelectedDest &&
                self._currentlySelectedDest !== destBase) {
                continue;
            }
            for (var destIdent in requests[destBase]) {
                var destinations = requests[destBase][destIdent];
                for (var destUri in destinations) {
                    if (!destinations[destUri]) {
                        log_1.log.log("" + "destinations[destUri] is null or undefined " +
                            "for destUri: " + destUri);
                        continue;
                    }
                    var results = destinations[destUri][0];
                    for (var i in results.matchedDenyRules) {
                        var _a = __read(results.matchedDenyRules[i], 2), ruleset = _a[0], match = _a[1];
                        var rawRule = ruleset_1.Ruleset.matchToRawRule(match);
                        if (!self._currentlySelectedDest) {
                            if (rawRule.d && rawRule.d.h) {
                                continue;
                            }
                        }
                        var rawRuleStr = ruleset_1.Ruleset.rawRuleToCanonicalString(rawRule);
                        if (ruleset.userRuleset) {
                            userRules[rawRuleStr] = rawRule;
                        }
                        else {
                            subscriptionRules[rawRuleStr] = rawRule;
                        }
                    }
                }
            }
        }
        for (var i in userRules) {
            self._addMenuItemRemoveDenyRule(list, userRules[i], false);
        }
        for (var i in subscriptionRules) {
            self._addMenuItemRemoveDenyRule(list, subscriptionRules[i], true);
        }
    };
    self._populateDetailsAddSubdomainAllowRules = function (list) {
        var origin = self._currentlySelectedOrigin;
        var reqSet = requests_1.Requests.getDeniedRequests(self._currentlySelectedOrigin, self._allRequestsOnDocument);
        var requests = reqSet.getAllMergedOrigins();
        var destHosts = {};
        for (var destBase in requests) {
            if (self._currentlySelectedDest &&
                self._currentlySelectedDest !== destBase) {
                continue;
            }
            for (var destIdent in requests[destBase]) {
                var destinations = requests[destBase][destIdent];
                for (var destUri in destinations) {
                    destHosts[uriService.getHost(destUri)] = null;
                }
            }
        }
        var mayPermRulesBeAdded = WindowUtils.mayPermanentRulesBeAdded(window, cachedSettings);
        for (var destHost in destHosts) {
            var ruleData = {
                "o": {
                    "h": self._addWildcard(origin),
                },
                "d": {
                    "h": destHost,
                },
            };
            if (!app_background_1.rp.policy.ruleExists(constants_1.C.RULE_ACTION_ALLOW, ruleData) &&
                !app_background_1.rp.policy.ruleExists(constants_1.C.RULE_ACTION_DENY, ruleData)) {
                if (mayPermRulesBeAdded === true) {
                    self._addMenuItemAllowOriginToDest(list, ruleData);
                }
                self._addMenuItemTempAllowOriginToDest(list, ruleData);
            }
            var destOnlyRuleData = {
                "d": {
                    "h": destHost,
                },
            };
            if (!app_background_1.rp.policy.ruleExists(constants_1.C.RULE_ACTION_ALLOW, destOnlyRuleData) &&
                !app_background_1.rp.policy.ruleExists(constants_1.C.RULE_ACTION_DENY, destOnlyRuleData)) {
                if (mayPermRulesBeAdded === true) {
                    self._addMenuItemAllowDest(list, destOnlyRuleData);
                }
                self._addMenuItemTempAllowDest(list, destOnlyRuleData);
            }
        }
    };
    window.rpcontinued.menu = self;
}
exports.loadMenuIntoWindow = loadMenuIntoWindow;