"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var environment_1 = require("lib/environment");
var manager_for_message_listeners_1 = require("lib/manager-for-message-listeners");
var manager_for_pref_observer_1 = require("lib/manager-for-pref-observer");
var RequestProcessor = require("lib/request-processor");
var WindowUtils = require("lib/utils/window-utils");
var JSUtils = require("lib/utils/js-utils");
var Utils = require("lib/utils/misc-utils");
var DOMUtils = require("lib/utils/dom-utils");
var constants_1 = require("data/constants");
var compatibility_rules_1 = require("models/compatibility-rules");
var requests_1 = require("models/requests");
var try_catch_utils_1 = require("lib/utils/try-catch-utils");
var app_background_1 = require("app/app.background");
var log_1 = require("app/log");
var uriService = app_background_1.rp.services.uri;
var cachedSettings = app_background_1.rp.storage.cachedSettings;
var LOG_FLAG_STATE = constants_1.C.LOG_FLAG_STATE;
function loadOverlayIntoWindow(window) {
    var rpcontinued = window.rpcontinued;
    var gBrowser = WindowUtils.getTabBrowser(window);
    var $str = browser.i18n.getMessage.bind(browser.i18n);
    var OverlayEnvironment = new environment_1.Environment(environment_1.MainEnvironment, "OverlayEnv");
    var mlManager = new manager_for_message_listeners_1.ManagerForMessageListeners(OverlayEnvironment, window.messageManager);
    function setTimeout(aFn, aDelay) {
        return window.setTimeout(function () {
            if (OverlayEnvironment.isShuttingDownOrShutDown()) {
                console.log("[RequestPolicy] [RequestPolicy] Not calling delayed function " +
                    "because of add-on shutdown.");
                return;
            }
            aFn.call(null);
        }, aDelay);
    }
    var initialized = false;
    var toolbarButtonId = "rpcontinuedOffAmoToolbarButton";
    var overlayId = 0;
    var blockedContentStateUpdateDelay = 250;
    var blockedContentCheckTimeoutId = null;
    var popupElement = null;
    var isFennec = false;
    var self = {
        requestLog: null,
        OverlayEnvironment: OverlayEnvironment,
    };
    var $id = function (id) {
        var element = window.top.document.getElementById(id);
        if (!element) {
            var popupframe = window.top.document.getElementById("rpc-popup-frame");
            if (popupframe && popupframe.contentDocument) {
                element = popupframe.contentDocument.getElementById(id);
            }
        }
        return element;
    };
    self.toString = function () {
        return "[rpcontinued.overlay " + overlayId + "]";
    };
    self.init = function () {
        try {
            if (initialized === false) {
                initialized = true;
                overlayId = (new Date()).getTime();
                rpcontinued.menu.init();
                popupElement = $id("rpc-popup");
                var appInfo = Cc["@mozilla.org/xre/app-info;1"].
                    getService(Ci.nsIXULAppInfo);
                isFennec = appInfo.name === "Fennec";
                browser.runtime.getBrowserInfo().then(function (appInfo) {
                    if (appInfo.name === "Fennec") {
                        log_1.log.log("Detected Fennec.");
                        popupElement.setAttribute("fennec", "true");
                        popupElement.setAttribute("position", "after_end");
                    }
                    return;
                }).catch(function (e) {
                    console.error("[RequestPolicy] Error on Fennec detection. Details:");
                    console.dir(e);
                });
                requests_1.Requests.onRequest.addListener(self.observeRequest);
                setContextMenuEntryEnabled(cachedSettings.get("contextMenu"));
                OverlayEnvironment.shutdownOnUnload(window);
                OverlayEnvironment.startup();
                mlManager.addListener("isOverlayReady", function () {
                    return true;
                }, function () {
                    return false;
                });
                window.messageManager.broadcastAsyncMessage(constants_1.C.MM_PREFIX + "overlayIsReady", true);
            }
        }
        catch (e) {
            console.error("[RequestPolicy] [FATAL] Unable to initialize rpcontinued.overlay. " +
                "Details:");
            console.dir(e);
            throw e;
        }
    };
    function setContextMenuEntryEnabled(isEnabled) {
        var contextMenuEntry = $id("rpcontinuedContextMenuEntry");
        contextMenuEntry.setAttribute("hidden", !isEnabled);
    }
    OverlayEnvironment.addShutdownFunction(environment_1.Level.INTERFACE, function () {
        requests_1.Requests.onRequest.removeListener(self.observeRequest);
        unwrapAddTab();
        self._removeHistoryObserver();
        self._removeLocationObserver();
    });
    OverlayEnvironment.addShutdownFunction(environment_1.Level.UI, function () {
        var requestLog = $id("rpcontinued-requestLog");
        if (!!requestLog && requestLog.hidden === false) {
            self.toggleRequestLog();
        }
    });
    function addAppcontentTabSelectListener() {
        var appcontent = $id("appcontent");
        if (appcontent) {
            if (isFennec) {
                OverlayEnvironment.elManager.addListener(appcontent, "TabSelect", self.tabChanged, false);
            }
        }
    }
    OverlayEnvironment.addStartupFunction(environment_1.Level.INTERFACE, addAppcontentTabSelectListener);
    function addContextMenuListener() {
        var contextMenu = $id("contentAreaContextMenu");
        if (contextMenu) {
            OverlayEnvironment.elManager.addListener(contextMenu, "popupshowing", self._contextMenuOnPopupShowing, false);
        }
    }
    OverlayEnvironment.addStartupFunction(environment_1.Level.INTERFACE, addContextMenuListener);
    function addTabContainerTabSelectListener() {
        if (!isFennec) {
            var container = gBrowser.tabContainer;
            var tabSelectCallback = function (event) {
                self.tabChanged();
            };
            OverlayEnvironment.elManager.addListener(container, "TabSelect", tabSelectCallback, false);
            wrapAddTab();
            self._addLocationObserver();
            self._addHistoryObserver();
        }
    }
    OverlayEnvironment.addStartupFunction(environment_1.Level.INTERFACE, addTabContainerTabSelectListener);
    mlManager.addListener("notifyTopLevelDocumentLoaded", function (message) {
        self._setContentBlockedState(false);
        self._updateBlockedContentStateAfterTimeout();
    });
    mlManager.addListener("notifyDOMFrameContentLoaded", function (message) {
        self._stopBlockedContentCheckTimeout();
        self._updateBlockedContentState(message.target);
    });
    mlManager.addListener("handleMetaRefreshes", function (message) {
        self.handleMetaRefreshes(message);
    });
    mlManager.addListener("notifyLinkClicked", function (message) {
        RequestProcessor.registerLinkClicked(message.data.origin, message.data.dest);
    });
    mlManager.addListener("notifyFormSubmitted", function (message) {
        RequestProcessor.registerFormSubmitted(message.data.origin, message.data.dest);
    });
    self.handleMetaRefreshes = function (message) {
        log_1.log.log("Handling meta refreshes...");
        var _a = message.data, documentURI = _a.documentURI, metaRefreshes = _a.metaRefreshes;
        var browser = message.target;
        for (var i = 0, len = metaRefreshes.length; i < len; ++i) {
            var _b = metaRefreshes[i], delay = _b.delay, destURI = _b.destURI, originalDestURI = _b.originalDestURI;
            log_1.log.log("meta refresh to <" + destURI + "> (" + delay + " second delay) " +
                (" found in document at <" + documentURI + ">"));
            if (originalDestURI) {
                log_1.log.log("meta refresh destination <" + originalDestURI + "> " +
                    ("appeared to be relative to <" + documentURI + ">, so ") +
                    ("it has been resolved to <" + destURI + ">"));
            }
            if (!cachedSettings.alias.isBlockingDisabled() &&
                !RequestProcessor.isAllowedRedirect(documentURI, destURI)) {
                if (uriService.getUriObject(destURI).schemeIs("javascript")) {
                    log_1.log.warn("Ignoring redirect to javascript URI <" + destURI + ">");
                    continue;
                }
                self._showRedirectNotification(browser, destURI, delay);
            }
        }
    };
    function cropUri(aUri, aMaxLength) {
        if (aUri.length < aMaxLength) {
            return aUri;
        }
        else {
            var prePathLength = uriService.getPrePath(aUri).length + 1;
            var len = Math.max(prePathLength, aMaxLength);
            return aUri.substring(0, len) + "...";
        }
    }
    self._showRedirectNotification = function (vBrowser, redirectTargetUri, delay, aRedirectOriginUri, replaceIfPossible) {
        var isOriginUndefined = aRedirectOriginUri === undefined;
        var redirectOriginUri = aRedirectOriginUri || self.getTopLevelDocumentUri();
        if (isFennec) {
            log_1.log.warning("Should have shown redirect notification to <" + redirectTargetUri + ">, but it's not implemented yet on Fennec.");
            return false;
        }
        var notificationBox = gBrowser.getNotificationBox(vBrowser);
        var notificationValue = "request-policy-meta-redirect";
        var notificationLabel;
        if (isOriginUndefined) {
            notificationLabel = $str("redirectNotification", [cropUri(redirectTargetUri, 50)]);
        }
        else {
            notificationLabel = $str("redirectNotificationWithOrigin", [
                cropUri(redirectOriginUri, 50),
                cropUri(redirectTargetUri, 50),
            ]);
        }
        var addRuleMenuName = "rpcontinuedRedirectAddRuleMenu";
        var addRulePopup = $id(addRuleMenuName);
        var classicmenu = rpcontinued.classicmenu;
        classicmenu.emptyMenu(addRulePopup);
        var m = rpcontinued.menu;
        var originBaseDomain = uriService.getBaseDomain(redirectOriginUri);
        var destBaseDomain = uriService.getBaseDomain(redirectTargetUri);
        var origin = null;
        var dest = null;
        if (originBaseDomain !== null) {
            origin = m._addWildcard(originBaseDomain);
        }
        if (destBaseDomain !== null) {
            dest = m._addWildcard(destBaseDomain);
        }
        var mayPermRulesBeAdded = WindowUtils.mayPermanentRulesBeAdded(window, cachedSettings);
        var allowRedirection = function () {
            RequestProcessor.registerAllowedRedirect(redirectOriginUri, redirectTargetUri);
            var data = {
                uri: redirectTargetUri,
            };
            if (replaceIfPossible) {
                data.replaceUri = redirectOriginUri;
            }
            vBrowser.messageManager.sendAsyncMessage(constants_1.C.MM_PREFIX + "setLocation", data);
        };
        function addMenuItem(aRuleSpec) {
            aRuleSpec.allow = true;
            classicmenu.addMenuItem(addRulePopup, aRuleSpec, function () {
                if (cachedSettings.get("autoReload")) {
                    allowRedirection();
                }
            });
        }
        function addMenuSeparator() {
            classicmenu.addMenuSeparator(addRulePopup);
        }
        {
            var label = $str("allowAllRedirections");
            classicmenu.addCustomMenuItem(addRulePopup, label, function () {
                maybeOpenLinkInNewTab(browser.runtime.getURL("settings/defaultpolicy.html"), [], true);
            });
            addMenuSeparator();
        }
        if (destBaseDomain !== null) {
            addMenuItem({ dest: dest });
            if (mayPermRulesBeAdded) {
                addMenuItem({ dest: dest });
            }
        }
        if (originBaseDomain !== null && destBaseDomain !== null) {
            addMenuSeparator();
        }
        if (originBaseDomain !== null) {
            addMenuItem({ origin: origin, temp: true });
            if (mayPermRulesBeAdded) {
                addMenuItem({ origin: origin });
            }
        }
        if (originBaseDomain !== null && destBaseDomain !== null) {
            addMenuSeparator();
            addMenuItem({ origin: origin, dest: dest, temp: true });
            if (mayPermRulesBeAdded) {
                addMenuItem({ origin: origin, dest: dest });
            }
        }
        var notification = notificationBox.
            getNotificationWithValue(notificationValue);
        if (notification) {
            notification.label = notificationLabel;
        }
        else {
            var buttons = [
                {
                    label: $str("allow"),
                    accessKey: $str("allow_accesskey"),
                    popup: null,
                    callback: allowRedirection,
                },
                {
                    label: $str("deny"),
                    accessKey: $str("deny_accesskey"),
                    popup: null,
                    callback: function () {
                    },
                },
                {
                    label: $str("addRule"),
                    accessKey: $str("addRule_accesskey"),
                    popup: addRuleMenuName,
                    callback: null,
                },
            ];
            var priority = notificationBox.PRIORITY_WARNING_MEDIUM;
            var notificationElem = notificationBox.appendNotification(notificationLabel, notificationValue, "chrome://rpcontinued/skin/requestpolicy-icon-blocked.png", priority, buttons);
            notificationElem.timeout = Date.now() + 300;
        }
        return true;
    };
    self.tabChanged = function () {
    };
    self._updateBlockedContentState = function () {
        RequestProcessor.whenReady.then(function () {
            var browser = gBrowser.selectedBrowser;
            var uri = uriService.stripFragment(browser.currentURI.spec);
            if (LOG_FLAG_STATE) {
                log_1.log.log("Checking for blocked requests from page <" + uri + ">");
            }
            var documentContainsBlockedContent = requests_1.Requests.
                getAllRequestsInBrowser(browser).containsBlockedRequests();
            self._setContentBlockedState(documentContainsBlockedContent);
            if (LOG_FLAG_STATE) {
                var logText = documentContainsBlockedContent ?
                    "Requests have been blocked." :
                    "No requests have been blocked.";
                log_1.log.log(logText);
            }
            return Promise.resolve();
        }).catch(function (e) {
            console.error("[RequestPolicy] [SEVERE] " +
                "Unable to complete _updateBlockedContentState actions. Details:");
            console.dir(e);
        });
    };
    self._setContentBlockedState = function (isContentBlocked) {
        var button = $id(toolbarButtonId);
        var contextMenuEntry = $id("rpcontinuedContextMenuEntry");
        if (button) {
            button.setAttribute("rpcontinuedBlocked", isContentBlocked);
            contextMenuEntry.setAttribute("rpcontinuedBlocked", isContentBlocked);
        }
    };
    function updatePermissiveStatus() {
        var button = $id(toolbarButtonId);
        var contextMenuEntry = $id("rpcontinuedContextMenuEntry");
        if (button) {
            var isPermissive = cachedSettings.alias.isBlockingDisabled();
            button.setAttribute("rpcontinuedPermissive", isPermissive);
            contextMenuEntry.setAttribute("rpcontinuedPermissive", isPermissive);
        }
    }
    function updatePermissiveStatusOnPrefChanges() {
        manager_for_pref_observer_1.ManagerForPrefObservers.get(OverlayEnvironment).
            addListener("startWithAllowAllEnabled", updatePermissiveStatus);
    }
    OverlayEnvironment.addStartupFunction(environment_1.Level.INTERFACE, updatePermissiveStatusOnPrefChanges);
    OverlayEnvironment.addStartupFunction(environment_1.Level.UI, updatePermissiveStatus);
    self.observeRequest = function (_a) {
        var isAllowed = _a.isAllowed, originUri = _a.originUri, destUri = _a.destUri;
        if (isAllowed) {
            if (self.requestLog) {
                self.requestLog.addAllowedRequest(originUri, destUri);
            }
        }
        else {
            self._updateNotificationDueToBlockedContent();
            if (self.requestLog) {
                self.requestLog.addBlockedRequest(originUri, destUri);
            }
        }
    };
    self.observeBlockedTopLevelDocRequest = function (browser, originUri, destUri) {
        setTimeout(function () {
            rpcontinued.overlay._showRedirectNotification(browser, destUri, 0, originUri);
        }, 0);
    };
    self._updateNotificationDueToBlockedContent = function () {
        if (!blockedContentCheckTimeoutId) {
            self._updateBlockedContentStateAfterTimeout();
        }
    };
    self._updateBlockedContentStateAfterTimeout = function () {
        var browser = gBrowser.selectedBrowser;
        blockedContentCheckTimeoutId = setTimeout(function () {
            try {
                rpcontinued.overlay._updateBlockedContentState(browser);
            }
            catch (e) {
            }
        }, blockedContentStateUpdateDelay);
    };
    self._stopBlockedContentCheckTimeout = function () {
        if (blockedContentCheckTimeoutId) {
            window.clearTimeout(blockedContentCheckTimeoutId);
            blockedContentCheckTimeoutId = null;
        }
    };
    self._contextMenuOnPopupShowing = function () {
        rpcontinued.overlay._wrapOpenLink();
    };
    function onOpenLinkViaContextMenu() {
        var origin = window.gContextMenuContentData ?
            window.gContextMenuContentData.docLocation :
            this.target.ownerDocument.URL;
        var dest = this.linkURL;
        RequestProcessor.registerLinkClicked(origin, dest);
    }
    function wrapFunctionErrorCallback(aMessage, aError) {
        console.error("[RequestPolicy] " + aMessage);
        console.dir(aError);
    }
    self._wrapOpenLink = function () {
        Utils.wrapFunction(window.gContextMenu, "openLink", wrapFunctionErrorCallback, onOpenLinkViaContextMenu);
        Utils.wrapFunction(window.gContextMenu, "openLinkInPrivateWindow", wrapFunctionErrorCallback, onOpenLinkViaContextMenu);
        Utils.wrapFunction(window.gContextMenu, "openLinkInCurrent", wrapFunctionErrorCallback, onOpenLinkViaContextMenu);
    };
    function wrapAddTab() {
        Utils.wrapFunction(gBrowser, "addTab", wrapFunctionErrorCallback, tabAdded);
    }
    function unwrapAddTab() {
        Utils.unwrapFunction(gBrowser, "addTab");
    }
    function tabAdded(aURI, aReferrerURI) {
        var referrerURI = aReferrerURI;
        if (typeof aReferrerURI === "object" &&
            !(referrerURI instanceof Ci.nsIURI)) {
            referrerURI = aReferrerURI.referrerURI;
        }
        if (referrerURI) {
            RequestProcessor.registerLinkClicked(referrerURI.spec, aURI);
        }
    }
    self._addLocationObserver = function () {
        self.locationListener = {
            onLocationChange: function (aProgress, aRequest, aURI) {
                rpcontinued.overlay._stopBlockedContentCheckTimeout();
                rpcontinued.overlay.
                    _updateBlockedContentState(gBrowser.selectedBrowser);
            },
            QueryInterface: XPCOMUtils.generateQI(["nsIWebProgressListener",
                "nsISupportsWeakReference"]),
        };
        gBrowser.addProgressListener(self.locationListener);
    };
    self._removeLocationObserver = function () {
        gBrowser.removeProgressListener(self.locationListener);
    };
    self._addHistoryObserver = function () {
        self.historyListener = {
            OnHistoryGoBack: function (backURI) {
                RequestProcessor.registerHistoryRequest(backURI.asciiSpec);
                return true;
            },
            OnHistoryGoForward: function (forwardURI) {
                RequestProcessor.registerHistoryRequest(forwardURI.asciiSpec);
                return true;
            },
            OnHistoryGotoIndex: function (index, gotoURI) {
                RequestProcessor.registerHistoryRequest(gotoURI.asciiSpec);
                return true;
            },
            OnHistoryNewEntry: function (newURI) {
            },
            OnHistoryPurge: function (numEntries) {
                return true;
            },
            OnHistoryReload: function (reloadURI, reloadFlags) {
                return true;
            },
            QueryInterface: function (aIID, aResult) {
                if (aIID.equals(Ci.nsISHistoryListener) ||
                    aIID.equals(Ci.nsISupportsWeakReference) ||
                    aIID.equals(Ci.nsISupports)) {
                    return this;
                }
                throw Cr.NS_NOINTERFACE;
            },
            GetWeakReference: function () {
                return Cc["@mozilla.org/appshell/appShellService;1"].
                    createInstance(Ci.nsIWeakReference);
            },
        };
        var tries = 0;
        var waitTime = 20;
        var maxTries = 10;
        var tryAddingSHistoryListener = function () {
            ++tries;
            var result = try_catch_utils_1.addSessionHistoryListener(gBrowser, self.historyListener);
            if (!result.error)
                return;
            var e = result.error;
            if (tries >= maxTries) {
                console.error("[RequestPolicy] " + "[SEVERE] Can't add session history listener, even " +
                    "after " + tries + " tries. Details:");
                console.dir(e);
            }
            else {
                setTimeout(function () {
                    tryAddingSHistoryListener();
                }, waitTime);
            }
        };
        tryAddingSHistoryListener();
    };
    self._removeHistoryObserver = function () {
        var result = try_catch_utils_1.removeSessionHistoryListener(gBrowser, self.historyListener);
        if (result.error) {
        }
    };
    self.onPopupShowing = function (event) {
        rpcontinued.menu.prepareMenu();
    };
    self.onPopupHidden = function (event) {
        var rulesChanged = rpcontinued.menu.processQueuedRuleChanges();
        if (rulesChanged || self._needsReloadOnMenuClose) {
            if (cachedSettings.get("autoReload")) {
                var mm = gBrowser.selectedBrowser.messageManager;
                mm.sendAsyncMessage(constants_1.C.MM_PREFIX + "reload");
            }
        }
        self._needsReloadOnMenuClose = false;
    };
    self.getTopLevelDocumentUriIdentifier = function () {
        return uriService.getIdentifier(self.getTopLevelDocumentUri());
    };
    self.getTopLevelDocumentUri = function () {
        var uri = gBrowser.selectedBrowser.currentURI.spec;
        return compatibility_rules_1.CompatibilityRules.getTopLevelDocTranslation(uri) ||
            uriService.stripFragment(uri);
    };
    self.toggleTemporarilyAllowAll = function () {
        var disabled = !cachedSettings.alias.isBlockingDisabled();
        cachedSettings.alias.setBlockingDisabled(disabled);
        return disabled;
    };
    self.revokeTemporaryPermissions = function (event) {
        app_background_1.rp.policy.revokeTemporaryRules();
        self._needsReloadOnMenuClose = true;
        popupElement.hidePopup();
    };
    self.openMenuAtContent = function () {
        var popupWidth = popupElement.clientWidth === 0 ? 730 :
            popupElement.clientWidth;
        var anchor = $id("content");
        var contentWidth = anchor.clientWidth;
        var xOffset = contentWidth - popupWidth - 2;
        popupElement.openPopup(anchor, "overlap", xOffset);
    };
    self.openMenuAtToolbarButton = function () {
        var anchor = $id("rpcontinuedOffAmoToolbarButton");
        popupElement.openPopup(anchor, "after_start", 0, 0, true, true);
    };
    self.openMenu = function () {
        setTimeout(function () {
            if (self.isToolbarButtonVisible()) {
                self.openMenuAtToolbarButton();
            }
            else {
                self.openMenuAtContent();
            }
        }, 0);
    };
    self.closeMenu = function () {
        rpcontinued.menu.close();
    };
    self.toggleMenu = function () {
        if ($id("rpc-popup").state === "closed") {
            self.openMenu();
        }
        else {
            self.closeMenu();
        }
    };
    self.isToolbarButtonVisible = function () {
        return DOMUtils.isElementVisible($id("rpcontinuedOffAmoToolbarButton"));
    };
    function openLinkInNewTab(url, relatedToCurrent) {
        window.openUILinkIn(url, "tab", { relatedToCurrent: !!relatedToCurrent });
        popupElement.hidePopup();
    }
    function maybeOpenLinkInNewTab(url, equivalentURLs, relatedToCurrent) {
        var e_1, _a;
        var possibleURLs = equivalentURLs.concat(url);
        var tabbrowser = window.gBrowser;
        var selectedTabIndex = tabbrowser.tabContainer.selectedIndex;
        var numTabs = tabbrowser.tabs.length;
        var indexes = JSUtils.leftRotateArray(JSUtils.range(numTabs), selectedTabIndex);
        try {
            for (var indexes_1 = __values(indexes), indexes_1_1 = indexes_1.next(); !indexes_1_1.done; indexes_1_1 = indexes_1.next()) {
                var index = indexes_1_1.value;
                var currentBrowser = tabbrowser.getBrowserAtIndex(index);
                var currentURI = currentBrowser.currentURI.spec;
                if (JSUtils.arrayIncludes(possibleURLs, currentURI)) {
                    tabbrowser.selectedTab = tabbrowser.tabContainer.childNodes[index];
                    popupElement.hidePopup();
                    return;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (indexes_1_1 && !indexes_1_1.done && (_a = indexes_1.return)) _a.call(indexes_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        openLinkInNewTab(url, relatedToCurrent);
    }
    self.openPrefs = maybeOpenLinkInNewTab.bind(null, "about:requestpolicy", [
        browser.runtime.getURL("settings/basicprefs.html"),
    ], true);
    self.openPolicyManager = maybeOpenLinkInNewTab.bind(null, browser.runtime.getURL("settings/yourpolicy.html"), [], true);
    self.openHelp = maybeOpenLinkInNewTab.bind(null, "https://github.com/" +
        "RequestPolicyContinued/requestpolicy/wiki/Help-and-Support", []);
    self.clearRequestLog = function () {
        self.requestLog.clear();
    };
    self.toggleRequestLog = function () {
        var requestLog = $id("rpcontinued-requestLog");
        var requestLogSplitter = $id("rpcontinued-requestLog-splitter");
        var requestLogFrame = $id("rpcontinued-requestLog-frame");
        var closeRequestLog = {};
        if (requestLog.hidden) {
            requestLogFrame.setAttribute("src", "chrome://rpcontinued/content/ui/request-log/request-log.xul");
            requestLog.hidden = false;
            requestLogSplitter.hidden = false;
            closeRequestLog.hidden = false;
        }
        else {
            requestLogFrame.setAttribute("src", "about:blank");
            requestLog.hidden = true;
            requestLogSplitter.hidden = true;
            closeRequestLog.hidden = true;
            self.requestLog = null;
        }
    };
    window.rpcontinued.overlay = self;
}
exports.loadOverlayIntoWindow = loadOverlayIntoWindow;