"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var WindowUtils = browser.extension.getBackgroundPage().WindowUtils;
function loadRLFilteringIntoWindow(window) {
    var requestLog = window.rpcontinued.requestLog;
    var filterText = null;
    var elements = WindowUtils.getElementsByIdOnLoad(window, {
        filterTextbox: "rpcontinued-requestLog-requestFilter",
        clearFilterButton: "rpcontinued-requestLog-clearFilter",
    });
    requestLog.filterChanged = function () {
        var filterValue = elements.filterTextbox.value;
        filterText = filterValue.length === 0 ? null : new RegExp(filterValue, "i");
        elements.clearFilterButton.disabled = filterValue.length === 0;
        loadTable();
    };
    requestLog.clearFilter = function () {
        elements.filterTextbox.value = "";
        elements.filterTextbox.focus();
        requestLog.filterChanged();
    };
    requestLog.isRowFilteredOut = function (aRow) {
        if (filterText === null) {
            return false;
        }
        return aRow[0].search(filterText) === -1 &&
            aRow[1].search(filterText) === -1;
    };
    function addRowOrFilterOut(aRow) {
        if (requestLog.isRowFilteredOut(aRow)) {
            return;
        }
        requestLog.visibleRows.push(aRow);
    }
    function loadTable() {
        var oldRowCount = requestLog.treeView.rowCount;
        if (!filterText) {
            requestLog.visibleRows = requestLog.rows;
        }
        else {
            requestLog.visibleRows = [];
            requestLog.rows.forEach(addRowOrFilterOut);
        }
        var newRowCount = requestLog.treeView.rowCount;
        requestLog.treebox.rowCountChanged(0, newRowCount - oldRowCount);
    }
}
exports.loadRLFilteringIntoWindow = loadRLFilteringIntoWindow;