"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var DomainUtil = browser.extension.getBackgroundPage().DomainUtil;
function loadRLInterfaceIntoWindow(window) {
    var requestLog = window.rpcontinued.requestLog;
    requestLog.clear = function () {
        var count = requestLog.treeView.rowCount;
        if (count === 0) {
            return;
        }
        requestLog.rows = [];
        requestLog.visibleRows = [];
        if (!requestLog.treebox) {
            return;
        }
        requestLog.treebox.rowCountChanged(0, -count);
    };
    requestLog.copyToClipboard = function (columnName) {
        var content = requestLog.treeView.getCellText(requestLog.tree.currentIndex, requestLog.tree.columns.getNamedColumn(columnName));
        var clipboardHelper = Cc["@mozilla.org/widget/clipboardhelper;1"].
            getService(Ci.nsIClipboardHelper);
        clipboardHelper.copyString(content);
    };
    requestLog.openInNewTab = function (columnName) {
        var content = requestLog.treeView.getCellText(requestLog.tree.currentIndex, requestLog.tree.columns.getNamedColumn(columnName));
        var forbidden = true;
        try {
            var uri = DomainUtil.getUriObject(content);
            if (uri.scheme === "http" || uri.scheme === "https" ||
                uri.scheme === "ftp") {
                forbidden = false;
            }
        }
        catch (e) {
            console.error("[RequestPolicy] " + e);
        }
        if (forbidden) {
            var alertTitle = browser.i18n.getMessage("actionForbidden");
            var alertText = browser.i18n.getMessage("urlCanOnlyBeCopiedToClipboard");
            Services.prompt.alert(null, alertTitle, alertText);
            return;
        }
        window.top.openUILinkIn(content, "tab", { relatedToCurrent: true });
    };
    function addRow(aRow) {
        requestLog.rows.push(aRow);
        if (!requestLog.isRowFilteredOut(aRow)) {
            if (requestLog.isEmptyMessageDisplayed) {
                requestLog.visibleRows.shift();
                requestLog.isEmptyMessageDisplayed = false;
                requestLog.treebox.rowCountChanged(0, -1);
            }
            requestLog.visibleRows.push(aRow);
            if (!requestLog.treebox) {
                return;
            }
            requestLog.treebox.rowCountChanged(0, 1);
        }
    }
    requestLog.addAllowedRequest = function (originURI, destURI) {
        addRow([originURI, destURI, false, (new Date()).toLocaleTimeString()]);
    };
    requestLog.addBlockedRequest = function (originURI, destURI) {
        addRow([originURI, destURI, true, (new Date()).toLocaleTimeString()]);
    };
}
exports.loadRLInterfaceIntoWindow = loadRLInterfaceIntoWindow;